#!/usr/bin/env python
# Author: Kristjan Haule, Jan 2008


import os,sys,re
from bisect import bisect

if 'ROOT' not in globals().keys():
    ROOT = os.environ.get('LMTO_DMFT_ROOT')
    sys.path.append( ROOT )
    sys.path.append( ROOT + '/fort')

from ld_base import *
# import optics
from pylab import *
from matplotlib.font_manager import fontManager, FontProperties
import copy

# MPI start
from mpi4py import MPI
Master = 0
# for the version 0.5
MPI.rprint = MPI._rprint
MPI.rank = MPI.WORLD_RANK
MPI.size = MPI.WORLD_SIZE
MPI.rprint('mpi.(root,size) =  %d %d'%(Master,MPI.size), MPI.COMM_WORLD, Master)

font = {'family' : 'serif', 'weight' : 'bold', 'size'   : 20,}
rc('font', **font)  # pass in the font dict as kwargs
    

def Cmp_Akw(lmt, mu, Sigc, phamf, polap):
    """
    """
    pnkp = shape(phamf)[2]
    nk = int(sqrt(pnkp))
    if nk**2 != pnkp:
        print 'Warning: Number of k-points is not square of an integer!'
        
    Akw=[]
    for ikx in range(nk):
        kAkw=[]
        for iky in range(nk):
            z = mu
            ikp = ikx*nk + iky
            akw = linalg.inv( polap[:,:,ikp]*z - phamf[:,:,ikp] - Sigc )
            takw = sum([-akw[p,p].imag/pi for p in range(len(akw))])
            kAkw.append(takw)
        Akw.append(kAkw)
    return Akw
    
    

#-------------- for upfolding -------------------------
#------------------------------------------------------
def strip_comments(data):
    newdata=[]
    for i in range(len(data)):
        stripped  = re.sub(r'(.*)\#.*', r"\1",  data[i].strip())
        stripped = stripped.split()
        if (len(stripped)>0):
            newdata.append(stripped)
    return newdata

def pars_index(ndim, lines, i0):
    """ reading the rules to transform
    columns of the sigma file to matrix"""
    sind=zeros((ndim,ndim), dtype=int)
    coef = zeros((ndim,ndim), dtype=complex)
    for p in range(ndim):
        ph = lines[i0]
        del lines[i0]

        for q in range(ndim):
            cols = re.findall(r'\$(\d+)', ph[q])
            if (cols):
                sind[p,q] = int(cols[0])

                tsc = re.sub(r'\$'+cols[0], '1', ph[q])
                coef[p,q] = eval(tsc)

        mess=[]
    return (sind, coef)


def pars_coeff(sc):
    """ Given a list string such as '$1$1 + 1./4.*$6$1 + (1.+1j)*$3$3*2'
        returns list of column numbers [(1,1),(6,1),(3,3)] and coefficients
        which correspond to each column  [1, 0.25, 2+2j]
    """
    cols = re.findall(r'\$(\d+)\$(\d+)', sc)
    scols = re.findall(r'(\$\d+\$\d+)', sc)
    cols = [(int(x[0]),int(x[1])) for x in cols]

    coef=[]
    for ic in range(len(cols)):
        tsc = sc
        for iq in range(len(cols)):
            rg = r'\$'+str(cols[iq][0])+'\$'+str(cols[iq][1])
            if (cols[ic]==cols[iq]):
                tsc = re.sub(rg, '1', tsc)
            else:
                tsc = re.sub(rg, '0', tsc)

        coef.append(eval(tsc))

    return (scols, cols, coef)

def read_sind(fh_sind, ndim):
    """ Reads index array to make a matrix of self-energy from columns of the file
    """
    f = open(fh_sind, 'r')

    data = f.readlines()
    lines = strip_comments(data)

    # Transformation from momentum to real space base
    Uk = identity(ndim)
    if (['transformation'] in lines):
        i0 = lines.index(['transformation'])
        del lines[i0]

        # parsing transformation
        Uk0=[]
        for p in range(ndim):
            Uk0.append(map(complex,lines[i0]))
            del lines[i0]
        Uk = array(Uk0)


    corind = ones((ndim,ndim),dtype=int)
    if (['correlated_blocks'] in lines):
        i0 = lines.index(['correlated_blocks'])
        del lines[i0]

        ones_ = ones(ndim, dtype=int)
        # parsing the shape of correlated blocks
        corind=[]
        for p in range(ndim):
            ind = map(int, lines[i0]) - ones_
            corind.append(ind)
            del lines[i0]
        corind = array(corind)

    NMcoef = ones((ndim,ndim))
    NMsind = ones((ndim,ndim),dtype=int)
    if (['NM_structure'] in lines):
        i0 = lines.index(['NM_structure'])
        del lines[i0]
        # reading the rules to transform
        # columns of the sigma file to matrix
        (NMsind, NMcoef) = pars_index(ndim, lines, i0)

    return (Uk, corind, NMsind, NMcoef)

def CreateSelfEnergyMatrixUpfold(Sig, Uk, sind, coef, gamma=0.01):
    # Here we create a matrix of self-energy Sig. The most important array in this class
    sig = matrix(zeros((len(sind),len(sind)),dtype=complex))
    #print shape(sig), shape(Sig), shape(Uk), shape(sind), shape(coef)
    for p in range(len(sig)):
        for q in range(len(sig)):
            ind = sind[p,q]-1
            #print p, q, ind
            if (ind>=0):
                if (ind<shape(Sig)[0]):
                    sig[p,q] = Sig[ind]*coef[p,q]
                else:
                    sig[p,q] = -gamma*1j

    return matrix(Uk) * matrix(sig) * matrix(conjugate(Uk.transpose()))
#------------------------------------------------------
#-------------- for upfolding -------------------------


import errno
def force_symlink(file1, file2):
    try:
        os.symlink(file1, file2)
    except OSError, e:
        if e.errno == errno.EEXIST:
            os.remove(file2)
            os.symlink(file1, file2)

class LdaRun(object):
    
    def __init__(self, inpdir, outdir, kpath, Nkp=9, lda_exe=ROOT+'/lda/lda'):
        
        self.lda_exe = lda_exe

        ini_files = glob.glob( os.path.join(inpdir, "*.ini") )
        if (len(ini_files)==0):
            print >> sys.stderr, "Could not find *.ini file therefore could not start calculation"
            sys.exit(1)
        else:
            if (len(ini_files)>1):
                print >> sys.stderr, "Found more than one ini file. Will use ", ini_files[0], "!"
                print >> sys.stderr, "All found files are=", ini_files

        m = re.match(".*\/(.*)\.ini", ini_files[0])
        self.project_name = m.group(1)        # It remembers name of the project and path to it

        print "Project name is ", self.project_name

        self.outdir = outdir
        if len(glob.glob(self.outdir))==0 : os.mkdir( self.outdir )


        self.fh_info = open(self.outdir+'/info.out', 'w')
        print >> self.fh_info, '-'*80
        print >> self.fh_info, '*'*20, 'Akw calculation'
        print >> self.fh_info, '-'*80, '\n'


        rinp = os.path.realpath(inpdir)
        rout = os.path.realpath(outdir)
        self.oini = os.path.join(rinp, self.project_name + ".ini")
        self.ostr = os.path.join(rinp, self.project_name + ".str")
        self.oscs = os.path.join(rinp, self.project_name + ".scs")
        self.kini = os.path.join(rout, self.project_name + ".ini")
        self.kstr = os.path.join(rout, self.project_name + ".str")
        self.kscs = os.path.join(rout, self.project_name + ".scs")
        self.kfat = os.path.join(rout, self.project_name + ".fat")
        
        if not os.path.exists(self.oini):
            print >> sys.stderr, "Could not find ini file at ", oini, "exiting..."
            sys.exit(1)
        if not os.path.exists(self.ostr):
            print >> sys.stderr, "Could not find str file at ", ostr, "exiting..."
            sys.exit(1)
        if not os.path.exists(self.oscs):
            print >> sys.stderr, "Could not find str file at ", ostr, "exiting..."
            sys.exit(1)

        force_symlink(self.oini, self.kini)
        force_symlink(self.ostr, self.kstr)
        force_symlink(self.oscs, self.kscs)
        
        ffat = open(self.kfat, 'w')
        print >> ffat, ' <FILE=FATFILE,INPUT=MODERN>'
        print >> ffat, '*'*50
        print >> ffat, ' <SECTION=CTRL>               ! CONTROL PARAMETERS:'
        print >> ffat, ' nDivDir= %d                   ! # of divisions' % Nkp
        print >> ffat, ' <SECTION=DIRS>                ! HIGH-SYMMETRY LINES:'
        print >> ffat, len(kpath), '                           ! # of directions in BZ'
        for rkp in kpath:
            print >> ffat, ' %s-%s' % rkp[0]
            print >> ffat, '    %f , %f , %f , %f , %f, %f' % tuple(rkp[1] + rkp[2])
        ffat.close()
        
    def exe(self):
        """ Below we execute lda"""
        lda_cmd = 'cd '+self.outdir+'; ' + self.lda_exe + ' <<EOF\n' + self.project_name + '\nini+str+scs\nfat+out\nEOF 2>&1\n'
        print 'Running ---- LDA -----'
        lda_out = os.popen(lda_cmd).read()
        print >> self.fh_info, lda_out
        

        
if __name__ == '__main__':

    Ry2eV = 13.6058
    inpdir = '.'
    sig = 'sig.inp'
    gamma = [0.01,0.01]
    Nkp=29                 # Number of k-points between two successive points in the k-path
    upfold=False
    total=True
    vmm=[0,1.]            # Color adjustment: minimum and maximum value plotted (in units of maximum value)
    om_mm = [(-5,5),100]

    
    execfile('PARAMS.akw') # File containing all parameters!


    a_inpdir = inpdir+'/ksum'
    a_outdir = inpdir+'/akw'


    #Nkp = 29
    #KT = [(-0.5,-0.5,0.0), (0.5,-0.5,0.0), (0.5,0.5,0.0)]

    print 'Nkp=', Nkp
    print 'KT=', KT
    
    KT = array(KT)    
    kpath =[]
    for i1 in range(Nkp):
        kt = [(KT[0] + (KT[2]-KT[1])*i1/(Nkp-1.)).tolist(),  (KT[1] + (KT[2]-KT[1])*i1/(Nkp-1.)).tolist()]
        kpath.append([('k'+str(i1), 'k'+str(i1+1)), kt[0], kt[1]])

    lda = LdaRun(a_inpdir, a_outdir, kpath, Nkp-1)

    
    lda.exe()
    
    #ldb = open(a_outdir+'/bandsFile.dat')
    #Npnt = int(ldb.next().split()[5])
    #Nbnd = int(ldb.next().split()[5])
    #Ndir = int(ldb.next().split()[5])
    #Ndiv = int(ldb.next().split()[5])
    #ldb.next()
    #psymb=[]
    #for i in range(Ndir):
    #    line = ldb.next().split()
    #    if int(line[3]) != i+1: print 'Something wrong reading bandsFile.dat 0'
    #    psymb.append([line[5],line[7]])
    #plngt=[]
    #for i in range(Ndir):
    #    line = ldb.next().split()
    #    if int(line[3]) != i+1: print 'Something wrong reading bandsFile.dat 1'
    #    plngt.append(line[5])
    #ldb.next()
    #
    #Ek=[]
    #Xk=[]
    #unique=zeros(Npnt,dtype=int)
    #for ib in range(Nbnd):
    #    ldb.next()
    #    ldb.next()
    #    xk=[]
    #    ek=[]
    #    xp=-100
    #    for ik in range(Npnt):
    #        line = ldb.next().split()
    #        if int(line[0])!=ik+1 : print 'Something wrong reading bandsFile.dat 2', line[0], ik+1
    #        xt = float(line[1])
    #        if (abs(xt-xp)>1e-10):
    #            xk.append(float(line[1]))
    #            ek.append(float(line[2]))
    #            unique[ik]=1
    #        xp=xt
    #    Ek.append(ek)
    #    Xk.append(xk)
    #Ek=array(Ek)
    #Xk=array(Xk)

    # Reads LMTO data
    lmt = Lmtart(a_outdir, sys.stdout, dioloc_file=a_inpdir+'/dioloc.dat')
    lmt.read_cix(filename=a_inpdir+'/kcix.dat')



    # Determin the chemical potential
    finf = open(inpdir+'/info.iterate', 'r')
    finfd = finf.readlines()
    mu = float(finfd[-1].split()[2])
    
    # Determines the last self-energy
    sfil = glob.glob(inpdir+'/sig.inp.*')
    kfil={}
    for ip,sf in enumerate(sfil):
        m = re.match(r'.*sig.inp\.(\d+)\.(\d+)', sf)
        if m is not None:
            (i1,i2) = map(int, m.groups())
            (dirName, fileName) = os.path.split(sf)
            kfil[i1*1000 + i2] = fileName
    sig = kfil[max(kfil.keys())]

    execfile('PARAMS.akw') # File containing all parameters! To override some settings

    print >> lda.fh_info, 'From input dir and PARAMS.akw found the self-energy to read', sig
    print >> lda.fh_info, 'From info.iterate and PARAMS.akw it was determined mu=', mu
    lda.fh_info.flush()
    
    # Reads self-energy from file
    (om,Sigc,Sigma_oo_IMPv,Edc_IMP) = ReadSelfEnergyColumns(inpdir+"/"+sig, contains_DC=True) # New self-energy
    # interpolating at zero frequency
    omx=0.0
    ip = bisect(om, omx)
    if ip<=1: ip=1
    if ip>=len(om): ip=len(om)-1
    Sigc = array(Sigc)
    Sigcx = Sigc[ip-1] + (Sigc[ip]-Sigc[ip-1])*(omx-om[ip-1])/(om[ip]-om[ip-1])
    
    Sigma_oo_lattice_v = Sigma_oo_IMPv-Edc_IMP

    print 'Was reading self-energy ', sig

    Sigma_oo_lattice = CreateSelfEnergyMatrix(Sigma_oo_lattice_v, lmt.Sigind)

    SigOm = CreateSelfEnergyMatrix(Sigcx, lmt.Sigind, gamma)+Sigma_oo_lattice
    
    trn.transform2(lmt.olap, lmt.hamf, lmt.Utk)


    
    dcorr = utl.flatten(lmt.vSigind)
    icorr = [lmt.Sigind[ic][ic] for ic in dcorr]
    if sum(sort(icorr)-array(range(1,len(icorr)+1)))==0: # icor contains right seuqence of integers
        corr = [dcorr[i-1] for i in icorr]               # kcix.dat contains  sequence of orbitals
        print 'reordering corr'
    else:
        corr = dcorr
    print 'corr = ', corr

    domh = (om_mm[0][1]-om_mm[0][0])/float(om_mm[1])
    omp = arange(om_mm[0][0], om_mm[0][1]+domh, domh)
    
    #if upfold:
    #    s_ndim = len(corr)
    #    (Uk, corind, NMsind, NMcoef) = read_sind(f_sind, s_ndim)
    #    
    #    SigOm=[]
    #    for iom in range(len(om)):
    #        sig = CreateSelfEnergyMatrixUpfold(Sigc[iom], Uk, NMsind, NMcoef, gamma[0])
    #        
    #        Sig = zeros((lmt.ndim,lmt.ndim), dtype=complex)
    #        for ip in range(len(Sig)): Sig[ip,ip] = -gamma[1]*1j
    #        
    #        for iq0,q0 in enumerate(corr):
    #            for iq1,q1 in enumerate(corr):
    #                Sig[q0,q1] = sig[iq0,iq1]
    #        SigOm.append(Sig)
    #    lmt.mu += dmu
    #    
    #else:
    #    Sigc = array(Sigc)
    #    SigOm=[]
    #    for omx in omp:
    #
    #        ip = bisect(om, omx)
    #        if ip<=1: ip=1
    #        if ip>=len(om): ip=len(om)-1
    #        
    #        Sigcx = Sigc[ip-1] + (Sigc[ip]-Sigc[ip-1])*(omx-om[ip-1])/(om[ip]-om[ip-1])
    #        
    #        SigOm.append(CreateSelfEnergyMatrix(Sigcx, lmt.Sigind, gamma)+Sigma_oo_lattice)
    #        
    print >> lda.fh_info, 'Self-energy created'  
    lda.fh_info.flush()

    
    
    Akw = Cmp_Akw(lmt, mu, SigOm, lmt.hamf, lmt.olap)
    data = array(Akw)
    
    #data=[]
    #for iom in range(len(omp)):
    #    sdata=[]
    #    for ikp in range(lmt.nkp):
    #        if unique[ikp]:
    #            if (total):
    #                value = sum(Akw[iom][ikp])
    #            else:
    #                value=0
    #                for ip in dcorr: value += Akw[iom][ikp][ip]
    #            sdata.append(value)
    #    data.append(sdata)
    #data=array(data)


    # For further processing saving the following variables: data, Ek, om
    fout = open(a_outdir+'/Akw.dat', 'w')
    print >> fout, 'data=', data.tolist()
    print >> fout, 'mu=', mu
    fout.close()
    
    
    #------------------------------------------------------
    #  Plotting
    #------------------------------------------------------
    Ry2eV = 13.6058
    color_bar = True
    
    execfile('PARAMS.akw') # File containing all parameters!

    print shape(data)

    # find maximum
    copydata = copy.deepcopy(data)
    copydata = sort(copydata.flatten())
    vmax = copydata[len(copydata)*0.99]
    vmm = [vmm[0], vmm[1]*vmax]
    print 'vmm=', vmm
    
    xmm=[0,1.]
    ymm=[0,1.]
    imshow(data, interpolation='bilinear', cmap=cm.hot, origin='lower', extent=[xmm[0],xmm[1],ymm[0],ymm[1]], aspect=1.0, vmin=vmm[0], vmax=vmm[1])

    if color_bar: colorbar()

    # Memorize the axis range
    ax = axis()

    
    #Klabel = [kpath[i][0][0] for i in range(len(kpath))]
    #Klabel.append(kpath[-1][0][1])
    #
    #xticks(range(len(Klabel)), tuple(Klabel), fontsize='x-large')
    #yticks(fontsize='x-large')
    #ylabel('$\omega$', fontsize='x-large')
    #
    #for ik,kl in enumerate(Klabel):
    #    plot((ik, ik), (ax[2], ax[3]), '--w', linewidth = 1, zorder=1)
    #
    #axis(ax)
    show()
