!**************************************************!
!*******  Copyright by Kristjan Haule, 2002 ******!
!**************************************************!
SUBROUTINE creorder( e,isrt,flg )
!!!----------------------------------------------------------------------!!!
!!!  subroutine creorder( e,isrt,flg )                                   !!!
!!!  This subroutine reorders four complex energies, e(i), in such       !!!
!!!  a way that any identical energies are in the low index array        !!!
!!!  slots.  There are a total of 15 different possibilities that        !!!
!!!  can occur and to distinguish between these we set up a unique       !!!
!!!  array of prime numbers and then use the product of the array        !!!
!!!  elements to identify which case occurs.  Note that the way we       !!!
!!!  check for identical elements could in principle result in more      !!!
!!!  than 15 possibilities if say element 1 and 2 are close within       !!!
!!!  tolerance and element 2 and 3 are close within tolerance it         !!!
!!!  could occur that element 1 and 3 would not be counted as equal      !!!
!!!  if they were not within the tolerance limit.  This case is          !!!
!!!  identified by a negative value for the flag, flg and should be      !!!
!!!  treated (as error) in the calling routine.                          !!!
!!!                                                                      !!!
!!!  Inputs:  e     - energy points (overwritten)                        !!!
!!!  Outputs: isrt  - pointer array                                      !!!
!!!           flg   - 1 if all energies different                        !!!
!!!                   2 if two energies are identical                    !!!
!!!                   3 if two pairs of energies are identical           !!!
!!!                   4 if three energies are identical                  !!!
!!!                   5 if all energies are identical                    !!!
!!!                  -1 if no order is found                             !!!
!!!----------------------------------------------------------------------!!!
  !---------- Passed variables ----------
  INTEGER    :: flg,isrt(4)
  COMPLEX*16 :: e(4)
  !---------- Local parameters ----------
  real*8, parameter :: eps = 1.d-6   
  !---------- Local variables ----------
  INTEGER    :: p,q,idx,prime
  REAL*8     :: rez,imz
  COMPLEX*16 :: z,w,e_new(4)
  !---------- Set up prime number matrix ----------
  idx = 1
  DO p = 1,3
     DO q = p+1,4
        z = e(p)-e(q)
        rez = dabs( dreal(z) )
        imz = dabs( dimag(z) )
        IF( rez.LE.eps.AND.imz.LE.eps ) THEN
           idx = idx*prime(p,q)
        ENDIF
     ENDDO
  ENDDO
  !---------- Find flg ----------
  flg = 1
  IF(idx.GT.1) flg = flg+1
  IF(idx.GT.13) flg = flg+1
  IF(idx.GT.35) flg = flg+1
  IF(idx.GT.1001) flg = flg+1
  !---------- Find new ordering ----------
  IF(idx.EQ.1.OR.idx.EQ.2.OR.idx.EQ.42.OR.idx.EQ.30030) THEN
     CALL setmap(isrt,1,2,3,4)
  ELSE IF(idx.EQ.3) THEN
     CALL setmap(isrt,1,3,2,4)
  ELSE IF(idx.EQ.5) THEN
     CALL setmap(isrt,1,4,2,3)
  ELSE IF(idx.EQ.7.OR.idx.EQ.35) THEN
     CALL setmap(isrt,2,3,1,4)
  ELSE IF(idx.EQ.11.OR.idx.EQ.33) THEN
     CALL setmap(isrt,2,4,1,3)
  ELSE IF(idx.EQ.13.OR.idx.EQ.26) THEN
     CALL setmap(isrt,3,4,1,2)
  ELSE IF(idx.EQ.195) THEN
     CALL setmap(isrt,1,3,4,2)
  ELSE IF(idx.EQ.110) THEN
     CALL setmap(isrt,1,2,4,3)
  ELSE IF(idx.EQ.1001) THEN
     CALL setmap(isrt,2,3,4,1)
  ELSE
     flg = -1
     CALL setmap(isrt,1,2,3,4)
  ENDIF
  !---------- Reorder energies ----------
  DO p = 1,4
     e_new(p) = e(isrt(p))
  ENDDO
  DO p = 1,4
     e(p) = e_new(p)
  ENDDO
  RETURN
END SUBROUTINE creorder

FUNCTION prime(p,q)
!!!------------------------------------------------------------------------!!!
!!!  This function associates a prime number to pairs of small integers.   !!!
!!!  The association is the following:                                     !!!
!!!  (1,2) = 2, (1,3) = 3, (1,4) = 5                                       !!!
!!!             (2,3) = 7, (2,4) = 11                                      !!!
!!!                        (3,4) = 13                                      !!!
!!!------------------------------------------------------------------------!!!
  INTEGER :: prime
  INTEGER :: p,q
  prime = p*q
  IF(prime.EQ.4.OR.prime.EQ.6.OR.prime.EQ.12) THEN
     prime = prime+1
     RETURN
  ELSE IF(prime.EQ.8) THEN
     prime = prime+3
     RETURN
  ENDIF
  RETURN
END FUNCTION prime

SUBROUTINE setmap(v,a,b,c,d)
!!!-----------------------------------------------------------!!!
!!!                                                           !!!
!!!  This subroutine takes in one vector, v, of integers and  !!!
!!!  four numbers a,b,c,d and assigns them the elements of    !!!
!!!  the vector v, which is of length four.                   !!!
!!!                                                           !!!
!!!-----------------------------------------------------------!!!
  INTEGER v(4),a,b,c,d
  v(1) = a
  v(2) = b
  v(3) = c
  v(4) = d
  RETURN
END SUBROUTINE setmap
