!**************************************************!
!*******  Copyright by Kristjan Haule, 2002 ******!
!**************************************************!
SUBROUTINE eigsys(ham, olm, zek, evl, evr, ndim)
!!!----------------------------------------------------------------------!!!
!!! This routine solves the generalized right/left eigenvalue problems   !!!
!!!       H.Ev^{R} = E O.Ev^{R}    and    Ev^{L}H = E Ev^{L}.O           !!!
!!! ( . denotes matrix multiplication )  where H is a general matrix and !!!
!!! O is a Hermitian positive definite matrix.                           !!!
!!! The problem is solved in several steps:                              !!!
!!!                                                                      !!!
!!! 1) Cholesky decompose O    :  O = U^{+}.U                            !!!
!!! 2) Invert Cholesky factor  :  B = U^{-1}                             !!!
!!! 3) Transform Hamiltonian   :  H' = B^{+}.H.B                         !!!
!!! 4) Solve eigenvalue problem:  H'.A^{R} = EA^{R} & A^{L}.H' = EA^{L}  !!!
!!! 5) Normalize eigenvectors  :  A^{L}.A^{R} = 1                        !!!
!!! 6) Transform eigenvectors  :  Ev^{R} = B.A^{R} & Ev^{L} = A^{L}.B^{+}!!!
!!! 7) Reorder eigenvalues                                               !!!
!!!                                                                      !!!
!!! Variable contents on entry:                                          !!!
!!! ===========================                                          !!!
!!!                                                                      !!!
!!! ham        : Hamiltonian                                             !!!
!!! olm        : Overlap matrix                                          !!!
!!! ndim       : Dimension of matrices                                   !!!
!!! fh_stderr  : Standard error file handle (Unit number)                !!!
!!!                                                                      !!!
!!! Variable contents on exit:                                           !!!
!!! ==========================                                           !!!
!!!                                                                      !!!
!!! ham        : Right eigenvectors as columns                           !!!
!!! olm        : Left eigenvectors as rows                               !!!
!!! zek        : Eigenvalues                                             !!!
!!!----------------------------------------------------------------------!!!
  IMPLICIT NONE
  !---------- Passed variables ----------
  COMPLEX*16, INTENT(in)  :: ham(ndim,ndim)  ! Hamiltonian
  COMPLEX*16, INTENT(in)  :: olm(ndim,ndim)  ! Overlap matrix
  COMPLEX*16, INTENT(out) :: zek(ndim)       ! Vector of eigenvalues 
  COMPLEX*16, INTENT(out) :: evl(ndim,ndim)   ! left eigenvectors
  COMPLEX*16, INTENT(out) :: evr(ndim,ndim)   ! right eigenvectors
  INTEGER, INTENT(in)     :: ndim            ! Dimension of hamiltonian
  !f2py integer intent(hide), depend(ham)  :: ndim=shape(ham,0)
  !---------- Local variables ----------
  REAL*8, PARAMETER :: smalleps = 1e-5
  CHARACTER*1, PARAMETER :: uplo  = "U"  ! Job parameter for zpotrf/ztrtri
  CHARACTER*1, PARAMETER :: diag  = "N"  ! Job parameter for ztrtri
  CHARACTER*1, PARAMETER :: jobvl = "V"  ! Jop parameter for zgeev
  CHARACTER*1, PARAMETER :: jobvr = "V"  ! Job parameter for zgeev
  CHARACTER*1, PARAMETER :: transn  = "N" ! Job parameter for ztrmm
  CHARACTER*1, PARAMETER :: transc  = "C" ! Job parameter for ztrmm
  CHARACTER*1, PARAMETER :: left   = "L" ! Job parameter for ztrmm
  CHARACTER*1, PARAMETER :: right  = "R" ! Job parameter for ztrmm
  INTEGER :: ierr                        ! Error parameter for lapack
  INTEGER :: irow                        ! Loop index for rows
  INTEGER :: icol                        ! Loop index for columns
  INTEGER :: p,q,r                       ! Loop index 
  COMPLEX*16 :: ctmp                     ! Temporary variable 
  COMPLEX*16  alpha_one                  ! For ztrmm
  COMPLEX*16 :: umat(ndim,ndim)  ! Cholesky factor
  COMPLEX*16 :: bmat(ndim,ndim)  ! Inverse Cholesky factor
  COMPLEX*16 :: htmp(ndim,ndim)  ! Transformed Hamiltonian
  COMPLEX*16 :: scaler(ndim)     ! Array of normalization parameters
  COMPLEX*16 :: cworkvec(8*ndim) ! Work array for zgeev
  REAL*8     :: rworkvec(8*ndim) ! Work array for zgeev

  !========== Step 1, Cholesky decompose O, O = U^{+}U ==========
  umat = olm
  CALL zpotrf(uplo,ndim,umat,ndim,ierr)
  IF(ierr.NE.0) THEN 
     WRITE(6,*) 'Error code of zpotrf ', ierr
     WRITE(0,*) 'Error in dia_gho! Stopp the code!'
  ENDIF
  !---------- umat is now upper triangular, lower
  !---------- triangular part is not referenced

  !========== Step 2, Invert Cholesky matrix U, B = U^{-1} ==========
  bmat = umat
  CALL ztrtri(uplo,diag,ndim,bmat,ndim,ierr)
  IF(ierr.NE.0) THEN
     WRITE(6,*) 'Error code of ztrtri ', ierr
     WRITE(0,*) 'Error in dia_gho! Stopp the code!'
  ENDIF
  !---------- bmat is now upper triangular, lower
  !---------- triangular part is not referenced

  !========== Step 3, Compute new Hamiltonian, H' = B^{+}HB ==========
  alpha_one = 1.0
  htmp = ham
  CALL ztrmm(right, uplo, transn, diag, ndim, ndim, alpha_one, bmat, ndim, htmp, ndim)
  CALL ztrmm(left,  uplo, transc, diag, ndim, ndim, alpha_one, bmat, ndim, htmp, ndim)

  !========== Step 4, Solve eigenvalue problem, H'v' = Ev' ==========
  CALL zgeev(jobvl,jobvr,ndim,htmp,ndim,zek,evl,ndim,evr,ndim,cworkvec,8*ndim,rworkvec,ierr)
  IF(ierr.NE.0) THEN
     WRITE(6,*) 'Error code of zgeev ', ierr
     WRITE(0,*) 'Error in dia_gho! Stopp the code!'
  ENDIF
  ! transpose left eigenvectors
  evl = dconjg(TRANSPOSE(evl))

  !========== Step 5, Make degenerate eigenvectors orthogonal
  DO q=1,ndim
     DO p=1,q-1
        IF (abs(zek(p)-zek(q)).LT.smalleps .AND. abs(scalprod(evl(p,:),evr(:,q),ndim)) .GT.smalleps) THEN
           evr(:,q) = evr(:,q) - scalprod(evl(p,:),evr(:,q),ndim)/scalprod(evl(p,:),evr(:,p),ndim) * evr(:,p)
        ENDIF
     ENDDO
     DO p=1,q-1
        IF (abs(zek(p)-zek(q)).LT.smalleps .AND. abs(scalprod(evl(q,:),evr(:,p),ndim)) .GT.smalleps) THEN
           evl(q,:) = evl(q,:) - scalprod(evl(q,:),evr(:,p),ndim)/scalprod(evl(p,:),evr(:,p),ndim) * evl(p,:)
        ENDIF
     ENDDO
  ENDDO

  !========= Step 6, Normalize eigenvectors
  DO p = 1,ndim
     ctmp = 0.d0
     DO q = 1,ndim
        ctmp = ctmp+evl(p,q)*evr(q,p)
     ENDDO
     scaler(p) = SQRT(ctmp)
  ENDDO
  DO p = 1,ndim
     evl(p,:) = evl(p,:)/scaler(p)
     evr(:,p) = evr(:,p)/scaler(p)
  ENDDO

  !========== Step 7, Transform eigenvectors and store ==========
  CALL ztrmm(left,  uplo, transn, diag, ndim, ndim, alpha_one, bmat, ndim, evr, ndim)
  CALL ztrmm(right, uplo, transc, diag, ndim, ndim, alpha_one, bmat, ndim, evl, ndim)

  RETURN

CONTAINS
  COMPLEX*16 FUNCTION scalprod(a,b,ndim)
    IMPLICIT NONE
    COMPLEX*16 :: a(:), b(:)
    INTEGER    :: ndim
    INTEGER    :: i
    scalprod = 0.0
    DO i=1,ndim
       scalprod = scalprod + a(i)*b(i)
    ENDDO
  END FUNCTION scalprod

END SUBROUTINE eigsys


SUBROUTINE eigvals(ham, olm, zek, ndim)
!!!----------------------------------------------------------------------!!!
!!! This routine solves the generalized right/left eigenvalue problems   !!!
!!!       H.Ev^{R} = E O.Ev^{R}    and    Ev^{L}H = E Ev^{L}.O           !!!
!!! ( . denotes matrix multiplication )  where H is a general matrix and !!!
!!! O is a Hermitian positive definite matrix.                           !!!
!!! The problem is solved in several steps:                              !!!
!!!                                                                      !!!
!!! 1) Cholesky decompose O    :  O = U^{+}.U                            !!!
!!! 2) Invert Cholesky factor  :  B = U^{-1}                             !!!
!!! 3) Transform Hamiltonian   :  H' = B^{+}.H.B                         !!!
!!! 4) Solve eigenvalue problem:  H'.A^{R} = EA^{R} & A^{L}.H' = EA^{L}  !!!
!!! 5) Normalize eigenvectors  :  A^{L}.A^{R} = 1                        !!!
!!! 6) Transform eigenvectors  :  Ev^{R} = B.A^{R} & Ev^{L} = A^{L}.B^{+}!!!
!!! 7) Reorder eigenvalues                                               !!!
!!!                                                                      !!!
!!! Variable contents on entry:                                          !!!
!!! ===========================                                          !!!
!!!                                                                      !!!
!!! ham        : Hamiltonian                                             !!!
!!! olm        : Overlap matrix                                          !!!
!!! ndim       : Dimension of matrices                                   !!!
!!! fh_stderr  : Standard error file handle (Unit number)                !!!
!!!                                                                      !!!
!!! Variable contents on exit:                                           !!!
!!! ==========================                                           !!!
!!!                                                                      !!!
!!! ham        : Right eigenvectors as columns                           !!!
!!! olm        : Left eigenvectors as rows                               !!!
!!! zek        : Eigenvalues                                             !!!
!!!----------------------------------------------------------------------!!!
  IMPLICIT NONE
  !---------- Passed variables ----------
  COMPLEX*16, INTENT(in)  :: ham(ndim,ndim)  ! Hamiltonian
  COMPLEX*16, INTENT(in)  :: olm(ndim,ndim)  ! Overlap matrix
  COMPLEX*16, INTENT(out) :: zek(ndim)       ! Vector of eigenvalues 
  INTEGER, INTENT(in)     :: ndim            ! Dimension of hamiltonian
  !f2py integer intent(hide), depend(ham)  :: ndim=shape(ham,0)
  !---------- Local variables ----------
  REAL*8, PARAMETER :: smalleps = 1e-5
  CHARACTER*1, PARAMETER :: uplo  = "U"  ! Job parameter for zpotrf/ztrtri
  CHARACTER*1, PARAMETER :: diag  = "N"  ! Job parameter for ztrtri
  CHARACTER*1, PARAMETER :: jobvl = "N"  ! Jop parameter for zgeev
  CHARACTER*1, PARAMETER :: jobvr = "N"  ! Job parameter for zgeev
  CHARACTER*1, PARAMETER :: transn  = "N" ! Job parameter for ztrmm
  CHARACTER*1, PARAMETER :: transc  = "C" ! Job parameter for ztrmm
  CHARACTER*1, PARAMETER :: left   = "L" ! Job parameter for ztrmm
  CHARACTER*1, PARAMETER :: right  = "R" ! Job parameter for ztrmm
  INTEGER :: ierr                        ! Error parameter for lapack
  INTEGER :: irow                        ! Loop index for rows
  INTEGER :: icol                        ! Loop index for columns
  INTEGER :: p,q,r                       ! Loop index 
  COMPLEX*16 :: ctmp                     ! Temporary variable 
  COMPLEX*16  alpha_one                  ! For ztrmm
  COMPLEX*16 :: umat(ndim,ndim)  ! Cholesky factor
  COMPLEX*16 :: bmat(ndim,ndim)  ! Inverse Cholesky factor
  COMPLEX*16 :: htmp(ndim,ndim)  ! Transformed Hamiltonian
  COMPLEX*16 :: scaler(ndim)     ! Array of normalization parameters
  COMPLEX*16 :: cworkvec(8*ndim) ! Work array for zgeev
  REAL*8     :: rworkvec(8*ndim) ! Work array for zgeev
  COMPLEX*16 :: evl(1,1)   ! left eigenvectors
  COMPLEX*16 :: evr(1,1)   ! right eigenvectors

  !========== Step 1, Cholesky decompose O, O = U^{+}U ==========
  umat = olm
  CALL zpotrf(uplo,ndim,umat,ndim,ierr)
  IF(ierr.NE.0) THEN 
     WRITE(6,*) 'Error code of zpotrf ', ierr
     WRITE(0,*) 'Error in dia_gho! Stopp the code!'
  ENDIF
  !---------- umat is now upper triangular, lower
  !---------- triangular part is not referenced

  !========== Step 2, Invert Cholesky matrix U, B = U^{-1} ==========
  bmat = umat
  CALL ztrtri(uplo,diag,ndim,bmat,ndim,ierr)
  IF(ierr.NE.0) THEN
     WRITE(6,*) 'Error code of ztrtri ', ierr
     WRITE(0,*) 'Error in dia_gho! Stopp the code!'
  ENDIF
  !---------- bmat is now upper triangular, lower
  !---------- triangular part is not referenced

  !========== Step 3, Compute new Hamiltonian, H' = B^{+}HB ==========
  alpha_one = 1.0
  htmp = ham
  CALL ztrmm(right, uplo, transn, diag, ndim, ndim, alpha_one, bmat, ndim, htmp, ndim)
  CALL ztrmm(left,  uplo, transc, diag, ndim, ndim, alpha_one, bmat, ndim, htmp, ndim)

  !========== Step 4, Solve eigenvalue problem, H'v' = Ev' ==========
  CALL zgeev(jobvl,jobvr,ndim,htmp,ndim,zek,evl,ndim,evr,ndim,cworkvec,8*ndim,rworkvec,ierr)
  IF(ierr.NE.0) THEN
     WRITE(6,*) 'Error code of zgeev ', ierr
     WRITE(0,*) 'Error in dia_gho! Stopp the code!'
  ENDIF
  RETURN

END SUBROUTINE eigvals

