!**************************************************!
!******* § Copyright by Kristjan Haule, 2002 ******!
!**************************************************!

DOUBLE COMPLEX FUNCTION lv(x,y)
!!!-------------------------------------------------------!!!
!!! We define the Lambin-Vigneron function as follows     !!!
!!!  lv(x,y) = z( 1 - z*Log[ 1 + 1/z ] )  where z = x/y   !!!
!!! or                                                    !!!
!!!  lv(x,y) = ( 1 - Log[ 1 + w ]/w ) /w  where w = y/x.  !!!
!!! For small z or w we use the series expansions:        !!!
!!! lv(x,y) =  z(1+z(log(z)-z(1-z(1/2-z(1/3-z(1/4-z ...   !!!
!!! lv(x,y) =  1/2-w(1/3-w(1/4-w(1/5- ...                 !!!
!!!-------------------------------------------------------!!!
  ! Returned type
  !---------- Passed variables ----------
  DOUBLE COMPLEX, intent(in) :: x
  DOUBLE COMPLEX, intent(in) :: y
  !---------- Local parameters ----------
  REAL*8,     PARAMETER :: eps = 1.d-1
  !---------- Local variables ----------
  DOUBLE COMPLEX :: z
  DOUBLE COMPLEX :: w
  DOUBLE COMPLEX :: i2
  DOUBLE COMPLEX :: i3
  DOUBLE COMPLEX :: i4
  DOUBLE COMPLEX :: i5
  DOUBLE COMPLEX :: i6
  DOUBLE COMPLEX :: i7
  DOUBLE COMPLEX :: i8
  !---------- Function body ----------
  i2 = dcmplx(1.d0/2.d0)
  i3 = dcmplx(1.d0/3.d0)
  i4 = dcmplx(1.d0/4.d0)
  i5 = dcmplx(1.d0/5.d0)
  i6 = dcmplx(1.d0/6.d0)
  i7 = dcmplx(1.d0/7.d0)
  i8 = dcmplx(1.d0/8.d0)
  IF(ABS(y).GT.ABS(x)) THEN
     z = x/y
     lv = z*( 1.d0 - z* (LOG(x+y) - LOG(x)))

     !IF(ABS(z).GT.eps) THEN
     !   lv = z*( 1.d0 - z*LOG( 1 + 1/z ) )
     !ELSE
     !   lv = z*( 1.d0 + z*( LOG(z) - z*( 1.d0 - z*( i2 - z*( i3 - &
     !        z*( i4 - z*( i5 - z*( i6 - z*( i7 - z*i8 ))))))))) 
     !ENDIF
  ELSE
     w = y/x
     IF(ABS(w).GT.eps) THEN
        !lv = ( 1.d0 - LOG( 1 + w )/w )/w
        lv = (1.d0 - (LOG(x+y)-LOG(x))/w )/w
     ELSE
        lv = i2 - w*( i3 - w*( i4 - w*( i5 - w*( i6 - w*(i7 - w*i8 )))))
     ENDIF
  ENDIF
  RETURN
END FUNCTION lv
  
SUBROUTINE arbek_optimized( z, e, res )
  IMPLICIT NONE
  !---------- Passed variables ----------
  DOUBLE COMPLEX, intent(in) :: z
  DOUBLE COMPLEX, intent(in) :: e(1:4)
  DOUBLE COMPLEX, intent(out) :: res(1:4)
  !---------- External functions -------
  DOUBLE COMPLEX :: lv
  !---------- Local variables ----------
  INTEGER :: i,j
  DOUBLE COMPLEX :: zmie(1:4)
  DOUBLE COMPLEX :: edif(1:4,1:4)
  DOUBLE COMPLEX :: u(1:4), v1, v2, v3, v4
  DOUBLE COMPLEX :: p12,p13,p14,p21,p23,p24,p31,p32,p34,p41,p42,p43
  !---------- Subroutine body ----------

  DO i = 1,4
     zmie(i) = z - e(i)
     DO j = 1,4
        edif(j,i) = e(j)-e(i)
     ENDDO
  ENDDO

  u(1) = zmie(1)/(          edif(2,1)*edif(3,1)*edif(4,1))
  u(2) = zmie(2)/(edif(1,2)          *edif(3,2)*edif(4,2))
  u(3) = zmie(3)/(edif(1,3)*edif(2,3)          *edif(4,3))
  u(4) = zmie(4)/(edif(1,4)*edif(2,4)*edif(3,4)          )

  p21 = edif(2,1)*lv(zmie(2),edif(2,1))
  p31 = edif(3,1)*lv(zmie(3),edif(3,1))
  p41 = edif(4,1)*lv(zmie(4),edif(4,1))
  p32 = edif(3,2)*lv(zmie(3),edif(3,2))
  p42 = edif(4,2)*lv(zmie(4),edif(4,2))
  p43 = edif(4,3)*lv(zmie(4),edif(4,3))

  v1 = 1/zmie(1)
  v2 = 1/zmie(2)
  v3 = 1/zmie(3)
  v4 = 1/zmie(4)

  p12 = zmie(1)*(1-zmie(1)*v2*(1-v2*p21))
  p13 = zmie(1)*(1-zmie(1)*v3*(1-v3*p31))
  p14 = zmie(1)*(1-zmie(1)*v4*(1-v4*p41))
  p23 = zmie(2)*(1-zmie(2)*v3*(1-v3*p32))
  p24 = zmie(2)*(1-zmie(2)*v4*(1-v4*p42))
  p34 = zmie(3)*(1-zmie(3)*v4*(1-v4*p43))

  res(1) =           - u(2)*p21 - u(3)*p31 - u(4)*p41
  res(2) = -u(1)*p12            - u(3)*p32 - u(4)*p42
  res(3) = -u(1)*p13 - u(2)*p23            - u(4)*p43
  res(4) = -u(1)*p14 - u(2)*p24 - u(3)*p34
END SUBROUTINE arbek_optimized

SUBROUTINE ek2idn( z, e, res )
  !---------- Passed variables ----------
  DOUBLE COMPLEX, intent(in)  :: z
  DOUBLE COMPLEX, intent(in)  :: e(1:4)
  DOUBLE COMPLEX, intent(out) :: res(1:4)
  !---------- External functions -------
  DOUBLE COMPLEX :: lv
  !---------- Local variables ----------
  INTEGER :: i,j
  DOUBLE COMPLEX :: zmie(2:4)
  DOUBLE COMPLEX :: edif(2:4,2:4)
  DOUBLE COMPLEX :: head(2:4)
  DOUBLE COMPLEX :: haus(3:4)
  DOUBLE COMPLEX :: kopf(3:4)
  !---------- Subroutine body ----------
  DO i = 2,4
     zmie(i) = z - e(i)
     DO j = 2,4
        IF(j==i) CYCLE
        edif(i,j) = e(i)-e(j)
     ENDDO
  ENDDO
  head(2) = zmie(2)/(edif(3,2)*edif(4,2))
  head(3) = zmie(3)/(edif(2,3)*edif(4,3))
  head(4) = zmie(4)/(edif(2,4)*edif(3,4))
  haus(3) = zmie(3)/edif(2,3)**2
  haus(4) = zmie(4)/edif(2,4)**2
  kopf(3) = zmie(3)/(edif(4,2)*edif(2,3))
  kopf(4) = zmie(4)/(edif(3,2)*edif(2,4))

  res(2)  = 0.5d0*head(2)  &
       + head(3)*lv(zmie(3),edif(3,2)) &
       + head(4)*lv(zmie(4),edif(4,2)) 

  res(3)  = head(2) &
       + (2*kopf(3)-haus(4))*lv(zmie(2),edif(2,3)) &
       + haus(4)*lv(zmie(4),edif(4,3)) 

  res(4)  = head(2) &
       + (2*kopf(4)-haus(3))*lv(zmie(2),edif(2,4)) &
       + haus(3)*lv(zmie(3),edif(3,4)) 
  res(1)  = res(2)
  RETURN
END SUBROUTINE ek2idn

SUBROUTINE ek22idn( z, e, res )
  !---------- Passed variables ----------
  DOUBLE COMPLEX, intent(in) :: z
  DOUBLE COMPLEX, intent(in) :: e(1:4)
  DOUBLE COMPLEX, intent(out) :: res(1:4)
  !---------- External functions -------
  DOUBLE COMPLEX :: lv
  !---------- Local variables ----------
  INTEGER :: i,j
  DOUBLE COMPLEX :: zmie(2:3)
  DOUBLE COMPLEX :: edif(2:3,2:3)
  DOUBLE COMPLEX :: head(2:3)
  !---------- Subroutine body ----------
  DO i = 2,3
     zmie(i) = z-e(i)
     DO j = 2,3
        IF(j==i) CYCLE
        edif(i,j) = e(i)-e(j)
     ENDDO
  ENDDO

  head(2) = zmie(2)/edif(3,2)**2
  head(3) = zmie(3)/edif(2,3)**2
  res(2)  = 0.5d0*head(2)+head(3)-3*head(2)*lv(zmie(3),edif(3,2))
  res(3)  = 0.5d0*head(3)+head(2)-3*head(3)*lv(zmie(2),edif(2,3))
  res(1)  = res(2)
  res(4)  = res(3)
  RETURN
END SUBROUTINE ek22idn

SUBROUTINE ek3idn( z, e, res )
  !---------- Passed variables ----------
  DOUBLE COMPLEX, intent(in) :: z
  DOUBLE COMPLEX, intent(in) :: e(1:4)
  DOUBLE COMPLEX, intent(out) :: res(1:4)
  !---------- External functions -------
  DOUBLE COMPLEX :: lv
  !---------- Local variables ----------
  INTEGER :: i,j
  DOUBLE COMPLEX :: zmie(3:4)
  DOUBLE COMPLEX :: edif(3:4,3:4)
  DOUBLE COMPLEX :: head(3:4)
  DOUBLE COMPLEX :: c1
  DOUBLE COMPLEX :: c5
  !---------- Subroutine body ----------
  c1 = dcmplx(1.d0/3.d0)
  c5 = dcmplx(5.d0/6.d0)
  DO i = 3,4
     zmie(i) = z-e(i)
     DO j = 3,4
        IF(j==i) CYCLE
        edif(i,j) = e(i)-e(j)
     ENDDO
  ENDDO
  head(3) = zmie(3)/edif(4,3)**2
  head(4) = zmie(4)/edif(3,4)**2
  res(3)  = c1*head(3)-c5*head(4) + head(4)*lv(zmie(4),edif(4,3))
  res(4)  = 0.5d0*head(3)+head(4)-3*head(3)*lv(zmie(4),edif(4,3))
  res(1)  = res(3)
  res(2)  = res(3)
  RETURN
END SUBROUTINE ek3idn

SUBROUTINE ek4idn( z, e, res )
  !---------- Passed variables ----------
  DOUBLE COMPLEX, intent(in) :: z
  DOUBLE COMPLEX, intent(in) :: e(1:4)
  DOUBLE COMPLEX, intent(out) :: res(1:4)
  !---------- Subroutine body ----------
  res(:) = 0.25d0/(z-e(1))
  RETURN
END SUBROUTINE ek4idn

