#!/usr/bin/env python
# Author: Kristjan Haule, Jan 2007


import os,sys

if 'ROOT' not in globals().keys():
    ROOT = os.environ.get('LMTO_DMFT_ROOT')
    sys.path.append( ROOT )
    sys.path.append( ROOT + '/fort')

from ld_base import *

import ldau

def cmp_Dos_Gloc_Delta(lmt, omega, Sigma, max_metropolis_steps=0):
    """
    Computes partial DOS, gloc, correlated gf and Delta
    Input:
       lmt                     -- lmtart class
       omega                   -- frequency (real or matsubara)
       Sigma[ndim,ndim]        -- self-energy matrix
       max_metropolis_steps    -- metropolis steps in sorting bands
       ---------------------------------------------
          The following members of class lmt are used:
             mu                -- the chemical potential
             olap[ndim,ndim]   -- overlap matrix
             hamf[ndim,ndim]   -- hamiltonian matrix
             itt[5,ntet]       -- index of tetrahedron mesh
             T2C[ndim,ndim]    -- matrix which rotates from spheric to cubis/relativistic coordinates
             bndindx[ndim,4]   -- index from orbital to atom and l
             ndim              -- size of the ham matrix
             Sigind[ndim,ndim] -- correlated index matrix
    Output:
       tDOS                    -- total DOS
       pDOS[natom,nmax]        -- partial DOS
       gc                      -- columns of the Green's function of the correlated orbitals 
       Delta                   -- hybridization function of the correlated orbitals
    """
    z = omega + lmt.mu

    # Actually computes local Green's function - sum over irreducible BZ
    gloc = cmp_Gloc(z, lmt.olap, lmt.hamf, Sigma, lmt.itt, max_metropolis_steps)
    # Array has to be converted to Fortran type array to symmetrize
    gloc = array(gloc, order='F')
    # Symmetrization replaces summation over full 1-st BZ
    sym.symmetrize_cubic(gloc, lmt.T2C)

    MDelta = z*identity(lmt.ndim) - linalg.inv(matrix(gloc)) - matrix(Sigma)
    
    # Partial DOS is computed for each atom and each l
    # We need information about lmax and natom to do that
    # Information is obtained from lmt.bndindx
    lmax  = int(max([x[1] for x in lmt.bndindx]))
    natom = int(max([x[0] for x in lmt.bndindx]))
    # Partial DOS is stored below
    dos_tot=0
    pdos = zeros((natom,lmax+1),dtype=float)
    for p in range(lmt.ndim):
        wdos = -gloc[p,p].imag/pi
        iatom = int(lmt.bndindx[p][0]-1)
        il    = int(lmt.bndindx[p][1])
        pdos[iatom, il] += wdos
        dos_tot += wdos
    # Green's function of the correlated block is stored below
    # We need information about the nonzero entries in the matrix
    sind = lmt.Sigind.flatten()
    cols = utl.union(sind)
    deg = utl.repeat(sind,cols)
    
    gc = zeros(max(cols),dtype=complex)
    Delta = zeros(max(cols),dtype=complex)
    for p in range(lmt.ndim):
        for q in range(lmt.ndim):
            if (lmt.Sigind[p,q]>0):
                ii = lmt.Sigind[p,q]-1
                gc[ii] += gloc[p,q]
                Delta[ii] += MDelta[p,q]
                
    for p in range(len(gc)):
        gc[p] /= deg[p+1]
        Delta[p] /= deg[p+1]

    return (dos_tot, pdos, gc, Delta)


def Print_Dos_Gloc_Delta(inpdir, outdir, max_metropolis_steps=0):
    """
    Computes and prints three quantities: DOS, Gf, Delta
    This function Reads self-energy from the input file
    at inpdir/sig.inp, calls cmp_Dos_Gloc_Delta compute the
    three quantities and prints to three files:
    outdir/dos.out, outdir/gf.out and outdir/Delta.out
    """

    (om,Sigc,Sigma_oo_IMPv,Edc_IMP) = ReadSelfEnergyColumns(inpdir+"/sig.inp", contains_DC=True)
    Sigma_oo_lattice_v = Sigma_oo_IMPv-Edc_IMP
    
    lmt = Lmtart(inpdir)
    lmt.read_cix()

    Sigma_oo_lattice = CreateSelfEnergyMatrix(Sigma_oo_lattice_v, lmt.Sigind)
    
    trn.transform2(lmt.olap, lmt.hamf, lmt.Utk)
    

    fh_dos = open(outdir+'/dos.out','w')
    fh_gf  = open(outdir+'/gf.out', 'w')
    fh_delta = open(outdir+'/Delta.out', 'w')
    
    for iom in range(len(om)):
        print om[iom]
        
        Sigma = CreateSelfEnergyMatrix(Sigc[iom], lmt.Sigind) + Sigma_oo_lattice

        (tdos, pdos, gc, Delta) = cmp_Dos_Gloc_Delta(lmt, om[iom], Sigma, max_metropolis_steps)

        fh_dos.write("%f \t%f " % (om[iom], tdos))
        for iatom in range(shape(pdos)[0]):
            for il in range(shape(pdos)[1]):
                fh_dos.write("\t%f " % pdos[iatom,il] )
        fh_dos.write("\n")

        fh_gf.write("%f " % (om[iom]))
        for g in gc:
            fh_gf.write("\t%f  %f " % (g.real, g.imag))
        fh_gf.write("\n")

        fh_delta.write("%f " % (om[iom]))
        for g in Delta:
            fh_delta.write("\t%f  %f " % (g.real, g.imag))
        fh_delta.write("\n")



def print_diagonal(Sinfty):
    for i in range(shape(Sinfty)[0]):
        fc = Sinfty[i,i]
        if (abs(fc.imag)<1e-6):
            fc=fc.real
        if (abs(fc)<1e-6):
            fc=0.0
        print i, fc



    
if __name__ == '__main__':

    om_a = -6
    om_b = 6
    Nom = 500
    p_cix ={1:3}
    
    inpdir = '.'
        
    Print_Dos_Gloc_Delta(inpdir, inpdir)

    
    
