      subroutine lib_mklegw(n,z,w,ipr)
C- Quadrature weights for Legendre Gaussian quadrature                          
C ----------------------------------------------------------------------        
Ci Inputs                                                                       
Ci   n:   Number of mesh points for numerical integration                       
Ci        (on interval -1, 1)                                                   
Ci   ipr: verbosity                                                             
Co Outputs                                                                      
Co   z,w                                                                        
Cr Remarks                                                                      
Cr   Integrates 2(n-1) order polynomial exactly on interval (-1,1)              
C ----------------------------------------------------------------------        
      implicit none                                                             
C Passed parameters                                                             
      integer n                                                                 
      double precision z(n),w(n)                                                
      double precision plegn,pnp                                                
C Local parameters                                                              
      integer iroot,in,ipr                                                      
      double precision root,delta,machep,pi                              
      pi = 4 * datan(1.d0)                                                       
      machep = 1.d-13
                                                                                
C --- Find all the roots of P_n ---                                             
      do  200  iroot = 1, n                                                     
        z(iroot) = dcos(pi*(2*iroot-0.5d0)/(2*n+1))                              
        root = z(iroot)                                                         
        if (ipr .ge. 110)                                                       
     .    print *, 'genleg: ',iroot,' starting guess is ',root                  
  100   continue                                                                
        delta = -plegn(n,root)/pnp(n,root)                                      
        if (ipr .ge. 110) write(*,*) 'delta is ',delta                          
        root = root + delta                                                     
        if (dabs(delta) .gt. dabs(machep*root)) goto 100                        
        z(iroot) = root                                                         
  200 continue                                                                  
                                                                                
C --- Debugging:  check for identical roots ---                                 
      do  300  in = 1, n-1                                                      
        if (dabs(z(in)-z(in+1)) .lt. machep)                                    
     .    call fullstop('genleg: identical roots')
  300 continue                                                                  
                                                                                
C --- Make the weights ---                                                      
      do  700  in = 1, n                                                        
        w(in) = 2/((1-z(in)**2)*(pnp(n,z(in))**2))                              
  700 continue                                                                  
                                                                                
C --- Printout ---                                                              
      if (ipr .lt. 50) return                                                   
      print *                                                                   
      print *, 'Genleg: roots and weighting factors'                            
      do  400  in = 1, n                                                        
        write(*,410) z(in),w(in)                                                
  410   format(1x,2(1pe26.15))                                                  
  400 continue                                                                  
      end                                                                       
                                                                                
      double precision function plegn(n,x)                                      
C- Calculates Legendre polynomical using a recursion relation                   
C ----------------------------------------------------------------------        
Ci Inputs                                                                       
Ci   n,x                                                                        
Co Outputs                                                                      
Co   plegn: P_n(x)                                                              
Cr Remarks                                                                      
Cr   Recursion relation is                                                      
Cr   P_n = ((2*n-1)*x*P_(n-1) - (n-1)*P_(n-2))/n                                
C ----------------------------------------------------------------------        
      implicit none                                                             
C Passed parameters                                                             
      integer n                                                                 
      double precision x                                                        
C Local parameters                                                              
      double precision jpjm1,cj,pjp1                                            
      integer j                                                                 
                                                                                
c jpjm1 is j*p_(j-1);  cj is 2*j - 1;  pjp1 is p_(j+1)                          
      jpjm1 = 0                                                                 
      plegn = 1                                                                 
      cj = 1                                                                    
      do  100  j = 1, n                                                         
        pjp1 = (cj*x*plegn - jpjm1)/j                                           
        jpjm1 = j*plegn                                                         
        cj = cj + 2                                                             
        plegn = pjp1                                                            
  100 continue                                                                  
      end                                                                       
                                                                                
      double precision function pnp(n,x)                                        
C- Calculates derivative of Legendre polynomical from recursion relation        
C ----------------------------------------------------------------------        
Ci Inputs                                                                       
Ci   n,x                                                                        
Co Outputs                                                                      
Co   pnp: P'_n(x)                                                               
Cr Remarks                                                                      
Cr   Recursion relations for P and P' are                                       
Cr   P_n (x) = ((2*n-1)*x*P_(n-1) - (n-1)*P_(n-2))/n                            
Cr   P'_n(x) = n/(1-x^2) (-x*P_n + P_(n-1) )                                    
C ----------------------------------------------------------------------        
      implicit none                                                             
C Passed parameters                                                             
      integer n                                                                 
      double precision x                                                        
C Local parameters                                                              
      double precision jpjm1,cj,pjp1,ln                                         
      integer j                                                                 
                                                                                
c jpjm1 is j*p_(j-1);  cj is 2*j - 1;  pjp1 is p_(j+1)                          
      jpjm1 = 0                                                                 
      ln = 1                                                                    
      cj = 1                                                                    
      do  100  j = 1, n                                                         
        pjp1 = (cj*x*ln - jpjm1)/j                                              
        jpjm1 = j*ln                                                            
        cj = cj + 2                                                             
        ln = pjp1                                                               
  100 continue                                                                  
      pnp = 1/(1-x**2)*(-n*x*ln + jpjm1)                                        
      end                                                                       
