      subroutine lib_padecoef( a,z,g,n )
c|-----------------------------------------------------------------------|
c|                                                                       |
c|               Computation of Pade coefficients.                       |
c|                                                                       |
c|    Inputs:                                                            |
cio    a  -  On input is a function for approximation, on output         |
c|           contains Pade coefficients.                                 |
ci     z  -  complex points in which function a is determined.           |
ci     n  -  size of arrays.                                             |
c|    Works:                                                             |
cw     g  -  work array.                                                 |
cr     Remarks:                                                          |
c|           (J. of Low Temp. Phys., v29, n3/4, 1977)                    |     
c|                                                                       |
c|-----------------------------------------------------------------------|
      implicit none
c--->  Passed variables
      integer n
      complex*16 a(n),z(n),g(n,n)
c--->  Local variables 
      integer i,j
       
      do i = 1,n
	g(1,i) = a(i)
      enddo

      do j = 2,n
	do i = 2,j
	  g(i,j) = ( g(i-1,i-1)-g(i-1,j) )/( z(j)-z(i-1) )/g(i-1,j)
	enddo
c       write(6,*)j,(i,g(i,j),i=1,n)
      enddo
      do i = 1,n
        a(i) = g(i,i)
      enddo
      end


       subroutine lib_pade( f,z1,z,p,n )
c|----------------------------------------------------------------------|
c|                                                                      |
c|            Calculation of analitically continued function            |
c|    for a given Pade coefficients in the arbitrary complex point.     |
c|                                                                      |
c|    Inputs:                                                           |
ci     p  -  Pade coefficients.                                         |
ci     z  -  set of points from which analytical continue is performed. |
ci     n  -  size of arrays.                                            |
ci     z1 -  complex point.                                             |
c|    Outputs:                                                          |
ci     f  -  value of function.                                         |
c|                                                                      |
c|----------------------------------------------------------------------|

      implicit none
c---> Passed variables
       integer n
       complex*16 z(n),p(n)
       real*8 sum
       complex*16 z1,f
c---> Local variables
       integer i,np
       real*8 cof1,cof2
       complex*16 a1,a2,anew,b1,b2,bnew
c---> Intrinsic
       intrinsic cdabs,dnint,dlog10,idint
 
      sum = 0.d0
      do i = 1,n-1
        sum = sum + cdabs( (z1-z(i))*p(i+1) )
      end do  
      if( sum.ge.1.d0 )then
        sum = dnint(n*sum/(n-1)) 
        sum = dlog10(sum)
        np  = idint(sum) + 1
        cof1 = 1.d-250
        cof2 = 1.d0
        if( np.gt.275 ) cof2 = 1.d-275
      else  
        sum = dnint((n-1)/(n*sum)) 
        sum = dlog10(sum)
        np  = -idint(sum) - 1
        cof1 = 1.d+250
        cof2 = 1.d0
        if( np.lt.-275 ) cof2 = 1.d+275
      endif

      a1 = dcmplx(0.d0,0.d0)
      a2 = cof1*p(1)
      b1 = dcmplx(cof1,0.d0)
      b2 = dcmplx(cof1,0.d0)
      do i = 1,n-1
	anew = a2+(z1-z(i))*p(i+1)*a1
	bnew = b2+(z1-z(i))*p(i+1)*b1
        a1 = a2
        b1 = b2
        a2 = anew
        b2 = bnew
      end do
      a2 = cof2*a2
      b2 = cof2*b2
      f = a2/b2

      end







	subroutine lib_padepoles(z,f,p,w,mpol)
	implicit none
      complex*16 c0,c1,v
      real*8 d
      integer i,m,x,n,mpol,ifail
	complex*16,allocatable :: pcof(:),r(:),c(:,:,:),a(:),g(:,:),t(:)
	complex*16 w(mpol),p(mpol),z(2*mpol),f(2*mpol)
c memory allocations
      n=2*mpol
      allocate(pcof(0:mpol))
      allocate(r(0:mpol))
      allocate(t(mpol))
      allocate(c(2,0:n,0:n))
      allocate(a(n))
      allocate(g(n,n))
c     compute Pade coefficients
      do i=1,n
      a(i)=f(i)
      enddo 
      call lib_padecoef(a,z,g,n)
c     Below we define initial conditions:
	do m=1,2
	do i=0,n,1
      c(m,0,i)=(0.d0,0.d0)
      enddo
      enddo
      c(1,1,0)=(0.d0,0.d0)
      c(1,1,1)= f(1)
      c(2,1,0)=(1.d0,0.d0)
      c(2,1,1)=(1.d0,0.d0)
c     This is the recursion relationship that defines the pade
c     approximation.  The first index, x, decides whether we are
c     finding the numerator or denominator polynomial (1=numerator).
c     The second index decides which coefficient we are looking at.
c     That is, "m" defines which power of z the coefficient will
c     be used for.  And "i" is a measurement of how many recursions
c     we want to use.
      do x=1,2
      do m=2,n
      c(x,m,0)=(0.d0,0.d0)
      c(x,m,1)=(0.d0,0.d0)
      enddo
      do m=1,n
      do i=2,n,1
      c(x,m,i)=c(x,m, i-1)+a(i)*c(x,m-1,i-2)-z(i-1)*a(i)*c(x,m,i-2)
      enddo
      enddo
      enddo
      do m=0,mpol
      pcof(m)=c(2,m+1,n)
      enddo
      ifail=0
      call lib_pzeros(mpol,pcof,p,1.d-14,ifail)
      do i=0,mpol
      r(i)=c(1,i+1,n)
      enddo
      do m=1,mpol
      t(m)=p(m)
      enddo
      v=pcof(mpol)
      call lib_padeweights(mpol,r,p,v,w)
c memory deallocations
	deallocate(t,r,c,g,a,pcof)
      end





	subroutine lib_padeweights(mpol,pcof,p,v,w)
	integer mpol,i,m
	complex*16 pcof(0:mpol),p(mpol),w(mpol),v
	complex*16, allocatable :: sum(:,:),product(:,:)
	allocate(sum(mpol,-1:mpol))
      allocate(product(mpol,-1:mpol))
	do m=1,mpol
	sum(m,-1)=(0.d0,0.d0)
	product(m,0)=(1.d0,0.d0)
      enddo
	product(1,1)=(1.d0,0.d0)
	do m=1,mpol
	do i=0,mpol-1
	sum(m,i)=sum(m,i-1)+(pcof(i)*(p(m)**i))
      enddo
      enddo
	do m=2,mpol ! +1
	do i=1,m-1
	product(m,i)=product(m,i-1)*(p(m)-p(i))
      enddo
	product(m,m)=product(m,m-1)
	do i=m+1,mpol
	product(m,i)=product(m,i-1)*(p(m)-p(i))
      enddo
      enddo
	do i=2,mpol
	product(1,i)=product(1,i-1)*(p(1)-p(i))
      enddo
      if(mpol.gt.1)then
      product(1,2)=(p(1)-p(2))
      do i=3,mpol
	product(1,i)=product(1,i-1)*(p(1)-p(i))
	enddo
	endif
	do i=1,mpol
	w(i)=(1/v)*(sum(i,mpol-1)/product(i,mpol))
      enddo
      deallocate(sum,product)
	end
