      !------------------------------------------------------------------------------!
      ! RAPX :       INTERPOLATION SCHEME FOR DENSITY OF STATES PRESENTED BY ITS     !
      !              MOMENTS USING ORTHOGONAL POLINOMIALS (GAUSSIAN INTEGRATION)     !
      ! M(0:M_MAX) - DOS MOMENTS                                                     !
      ! M_MAX      - MAXIMAL NUMBER OF MOMENTS (MUST BE ODD NUMBER)                  !
      ! NPOLE      - NUMBER OF POINTS AND WEIGHTS :   M_MAX=2*NPOLE-1                !
      ! WW(NPOLE)  - WEIGHTS OF INTERPOLATIONS SCHEME                                !
      ! XP(NPOLE)  - POINTS OF INTERPOLATIONS SCHEME                                 !
      ! IOUT       - OUTPUT CHANNEL                                                  !
      ! IERR       - RETURN CODE = NONZERO, IN CASE OF ABNORMAL TERMINATION          !
      ! THE "EFFECTIVE" NUMBER OF POLES WILL BE REDUCED TO ACTUAL NUMBER OF SPECTRAL !
      ! PEAKS                                                                        !
      !------------------------------------------------------------------------------!
      SUBROUTINE LIB_RAPX(M,M_MAX,WW,XP,NPOLE,IERR,PDEF,IOUT)
        IMPLICIT NONE
        INTEGER(KIND=4), INTENT(IN)   :: M_MAX,NPOLE         
        INTEGER(KIND=4), INTENT(INOUT):: IERR               
        INTEGER(KIND=4), INTENT(IN)   :: IOUT                
        REAL(KIND=8),    INTENT(IN)   :: M(0:M_MAX),PDEF
        REAL(KIND=8),    INTENT(INOUT):: WW(NPOLE),XP(NPOLE) 
        !--------------------------------------------------!
        REAL(KIND=8), ALLOCATABLE     :: ANORM(:)          ! POLINOMIAL'S NORM        
        REAL(KIND=8), ALLOCATABLE     :: C(:,:)            ! COEFFICIENTS OF P(N,X)
        REAL(KIND=8)                  :: MWXP
        INTEGER(KIND=4)               :: I,J,JMAX

        IERR=0
        WW = 0.D0
        XP = PDEF
        IF(ABS(M(0)).LE.1.D-10) RETURN
        IF(M_MAX /= 2*NPOLE-1) THEN
           IF(IOUT.GT.0)WRITE(IOUT,"(/'NUMBER OF MOMENTS (',I2,') =/= 2*N_POLE(',I2,')-1'/)") &
                        M_MAX,NPOLE
           IERR=1
           RETURN
        END IF
        ALLOCATE(C(0:NPOLE,0:NPOLE))
        ALLOCATE(ANORM(-1:NPOLE))
        CALL LIB_POLINOMIALS(NPOLE,JMAX,.TRUE.,IOUT)
        CALL LIB_ZERO(JMAX,IOUT)
        CALL LIB_WEIGHTS(JMAX)
        DEALLOCATE(C)
        DEALLOCATE(ANORM)
        ! ************************ CHECK (WW,XP)-SET ************************ !
        IF(IOUT.GT.0)WRITE(IOUT,"(/20X,'LIB_RAPX : CHECK MOMENTS')")
   		IF(IOUT.GT.0)WRITE(IOUT,"(/T15,'(IN)',T39,'(OUT)',T60,'(DM/M_IN)'/)")
		DO I=0,JMAX ! <=> NPOLE
		   MWXP=0.0_8
		   DO J=1,JMAX ! <=> NPOLE
		      MWXP=MWXP + WW(J)*XP(J)**I
		   END DO
		   IF(IOUT.GT.0)WRITE(IOUT,"(I3,4X,F22.7,3x,F22.7,3x,F12.9)") I,M(I),MWXP,ABS((M(I)-MWXP)/M(I))
		END DO

        CONTAINS
        !------------------------------------------------------------!
        ! LIB_POLINOMIALS: CALCULATES ORTHOGONAL POLINOMIALS         !
        ! N                 - MAXIMAL ORDER OF POLINOMIALS           !
        ! JMAX <= N         - ACTUAL ORDER OF POLYNOMIALS            !
        !------------------------------------------------------------!
        SUBROUTINE LIB_POLINOMIALS(N,JMAX,PRINT_C,IOUT)
          IMPLICIT NONE
          INTEGER(KIND=4), INTENT(IN)  :: N ,IOUT
          INTEGER(KIND=4), INTENT(OUT) :: JMAX
          !------------------------------!
          REAL(KIND=8)                 :: AJ,BJ,CJMCJN,PJXPJ,CJM1,CJ1M
          REAL(KIND=8),  PARAMETER     :: ZERO=0.0_8
          INTEGER(KIND=4)              :: J,MM,NN
          LOGICAL                      :: PRINT_C
        
          C=ZERO
          ANORM(-1)=1.D0
          C(0,0)=1.0_8
          JMAX=0
          J_CYCLE : DO J=0,N-1
             ANORM(J)= ZERO
             PJXPJ =  ZERO
             DO MM=0,J
             DO NN=0,J
                CJMCJN=C(J,MM)*C(J,NN)
                ANORM(J) = ANORM(J) + CJMCJN*M(MM+NN)
                PJXPJ   = PJXPJ + CJMCJN*M(MM+NN+1)
             END DO
             END DO
             IF(ABS(ANORM(J)) <= 1.d-10) THEN ! NO POLYNOMIALS LEFT
               IF(IOUT.GT.0) WRITE(IOUT,"('NPOLE REDUCED : ',I3,' -> ',I3)") NPOLE, JMAX
               EXIT J_CYCLE 
             END IF
             JMAX=J+1
             AJ=PJXPJ/ANORM(J)
             BJ=ANORM(J)/ANORM(J-1)
             C(J+1,J+1)=C(J,J)
             DO MM=0,J
                IF(MM-1>=0) THEN
                   CJM1=C(J,MM-1)
                ELSE
                   CJM1=ZERO
                END IF
                IF(J-1-MM >= 0) THEN
                   CJ1M=C(J-1,MM)
                ELSE
                   CJ1M=ZERO
                END IF
                C(J+1,MM)=CJM1 - AJ*C(J,MM) - BJ*CJ1M
             END DO
          END DO J_CYCLE
          IF(PRINT_C) THEN
             IF(IOUT.GT.0)WRITE(IOUT,"(/10X,'POLINOMIAL COEFFICIENTS'/)")
             DO J=0,N
                IF(IOUT.GT.0)WRITE(IOUT,"(3X,I2,3X,15(F12.5))") J,C(J,:)
             END DO
          END IF
        END SUBROUTINE LIB_POLINOMIALS
        !------------------------------------------------------------!
        ! ZERO : FINDS ZEROES OF POLINOMIAL OF ORDER N USING         !
        !        LAGUERRE METHOD                                     !
        !------------------------------------------------------------!
        SUBROUTINE LIB_ZERO(N,IOUT)
           IMPLICIT NONE
           INTEGER(KIND=4), INTENT(IN) :: N,IOUT
           !-----------------------------------!
           REAL(KIND=8)                :: PNZ
           COMPLEX(KIND=8)             :: A(N+1),ROOTS(N)
           INTEGER(KIND=4)             :: I,K,MM
           A(:)=CMPLX(0.0_8,0.0_8)
           A(1:N+1)=C(N,0:N)
           CALL LIB_ZROOTS(A,N,ROOTS,.FALSE.)
           XP(1:N)=REAL(ROOTS(1:N))
           IF(IOUT.GT.0)WRITE(IOUT,"(/T12,'LIB_RAPX : ZEROES OF POLINOMIALS'&
                        &//T17,'X',T33,'P(X)'/)")
           DO K=1,N
              PNZ=0.0_8
              DO MM=0,N
                 PNZ=PNZ + C(N,MM)*XP(K)**MM
              END DO
              IF(IOUT.GT.0)WRITE(IOUT,"(I3,3X,F15.6,3X,F15.6)") K,XP(K),PNZ
           END DO 
           RETURN
        END SUBROUTINE LIB_ZERO
        !------------------------------------------------------------!
        ! WEIGHTS : CALCULATES WEIGHTS OF N-POINT INTEGRATION SCHEME ! 
        !           W(J)= <P_N-1 | P_N-1 > / P_N-1(X_J)*P'_N(X_J)    !
        !------------------------------------------------------------!
        SUBROUTINE LIB_WEIGHTS(NPOLE)
          IMPLICIT NONE
          INTEGER(KIND=4), INTENT(IN) :: NPOLE
          !----------------------------------!
          REAL(KIND=8)                :: XJ_MM,PN1,DPN
          INTEGER(KIND=4)             :: J,MM
        
          DO J=1,NPOLE
             PN1=0.0_8
             DPN=0.0_8
             DO MM=0,NPOLE-1
                XJ_MM=XP(J)**MM
                PN1=PN1 + C(NPOLE-1,MM)*XJ_MM
                DPN=DPN + (MM+1)*C(NPOLE,MM+1)*XJ_MM
             END DO
             WW(J) = ANORM(NPOLE-1)/(PN1*DPN)
          END DO
        END SUBROUTINE LIB_WEIGHTS
                                     
      END SUBROUTINE LIB_RAPX
      
      SUBROUTINE LIB_ZROOTS(A,M,ROOTS,POLISH)
         IMPLICIT NONE
         INTEGER(KIND=4), INTENT(IN)  :: M
         COMPLEX(KIND=8), INTENT(IN)  :: A(M+1)
         COMPLEX(KIND=8), INTENT(OUT) :: ROOTS(M)
         LOGICAL,         INTENT(IN)  :: POLISH
         !------------------------------------!
         INTEGER(KIND=4), PARAMETER   :: MAXM=101
         INTEGER(KIND=4)              :: I,J,JJ,ITS
         COMPLEX(KIND=8)              :: AD(MAXM),X,B,C
         REAL(KIND=8),    PARAMETER   :: EPS=1.E-6
         
         DO J=1,M+1
            AD(J)=A(J)
         END DO
         DO J=M,1,-1
            X=CMPLX(0.0_8,0.0_8)
            CALL LIB_LAGUER(AD,J,X,ITS)
            IF(ABS(DIMAG(X)).LE.2*EPS**2*ABS(DREAL(X))) X=CMPLX(DREAL(X),0.0_8)
            ROOTS(J)=X
            B=AD(J+1)
            DO JJ=J,1,-1
               C=AD(JJ)
               AD(JJ)=B
               B=X*B+C
            END DO 
         END DO
         IF(POLISH) THEN
           DO J=1,M
              CALL LIB_LAGUER(A,M,ROOTS(J),ITS)
           END DO
         END IF
         DO J=2,M
            X=ROOTS(J)
            DO I=J-1,1,-1
               IF(DREAL(ROOTS(I)).LE.DREAL(X)) GO TO 10
               ROOTS(I+1)=ROOTS(I)
            END DO
            I=0
      10    ROOTS(I+1)=X
         END DO
         RETURN
      END SUBROUTINE LIB_ZROOTS

      SUBROUTINE LIB_LAGUER(A,M,X,ITS)
         IMPLICIT NONE
         INTEGER(KIND=4), INTENT(IN)    :: M
         INTEGER(KIND=4), INTENT(INOUT) :: ITS
         COMPLEX(KIND=8), INTENT(IN)    :: A(M+1)
         COMPLEX(KIND=8), INTENT(INOUT) :: X
         !---------------------------------!
         REAL(KIND=8),    PARAMETER     :: EPSS=2.E-30
         INTEGER(KIND=4), PARAMETER     :: MR=8,MT=10
         INTEGER(KIND=4)                :: ITER,J,MAXIT
         REAL(KIND=8)                   :: ABX,ABP,ABM,ERR,FRAC(MR)
         COMPLEX(KIND=8)                :: DX,X1,B,D,F,G,H,SQ,GP,GM,G2
         SAVE FRAC
         DATA FRAC /.5,.25,.75,.13,.38,.62,.88,1./
            
         MAXIT=MR*MT
         DO ITER=1,MAXIT
            ITS=ITER
            B=A(M+1)
            ERR=ABS(B)
            D=CMPLX(0.0,0.0)
            F=CMPLX(0.0,0.0)
            ABX=ABS(X)
            DO J=M,1,-1
               F=X*F+D
               D=X*D+B
               B=X*B+A(J)
               ERR=ABS(B)+ABX*ERR
            END DO
            ERR=EPSS*ERR
            IF(ABS(B).LE.ERR) THEN
               RETURN
            ELSE
               G=D/B
               G2=G*G
               H=G2-2.*F/B
               SQ=SQRT((M-1)*(M*H-G2))
               GP=G+SQ
               GM=G-SQ
               ABP=ABS(GP)
               ABM=ABS(GM)
               IF(ABP.LT.ABM) GP=GM
               IF(MAX(ABP,ABM).GT.0.0) THEN
                  DX=M/GP
               ELSE
                  DX=EXP(CMPLX(LOG(1.0+ABX),FLOAT(ITER)))
               END IF
            END IF
            X1=X-DX
            IF(X.EQ.X1) THEN
               RETURN
            END IF
            IF(MOD(ITER,MT).NE.0) THEN
               X=X1
            ELSE
               X=X-DX*FRAC(ITER/MT)
            END IF
         END DO
         !IF(IOUT.GT.0)WRITE(IOUT,*) "TOO MANY ITERATIONS IN LAGUER"
         RETURN
      END SUBROUTINE LIB_LAGUER
      !-----------------------------------------------------------!
