 module mod_cls_L  
   use com_labels, only : PATHSCRT
   use com_lmtinp, only : PROJECT
   use com_extens, only : EXTNSCR
   use mod_cls_l_lib
   use mod_cls_l_time        
   implicit none
   
   type t_vector
      complex(kind=8), pointer :: coor(:)
      real(kind=8)             :: eigval
      real(kind=8)             :: ML         ! Projection of impurity orbital momentum.
      real(kind=8)             :: MS         ! Projection of impurity spin momentum.
      real(kind=8)             :: MJ         ! Projection of impurity total angular momentum.
      real(kind=8)             :: S          ! Total impurity spin momentum.
      real(kind=8)             :: L          ! Total impurity orbital momentum.
      real(kind=8)             :: J          ! Total impurity angular momentum.
      real(kind=8),    pointer :: Occ(:)     ! Occupation number of cluster.
      real(kind=8)             :: MSC        ! Projection of cluster spin momentum.
      real(kind=8)             :: SC         ! Total cluster spin momentum.         
   end type t_vector
   
   type t_fockspace
      integer(kind=4)          :: dim        ! Fockspace dimension
      integer(kind=kind_phi), pointer :: phis(:)    ! Bath basis
      integer(kind=kind_phi), pointer :: phid(:)    ! Impurity basis
      integer(kind=kind_phi), pointer :: phi(:)     ! Total basis
      integer(kind=4)          :: n_rec      ! Number of H-records 
      real(kind=8)             :: emin       ! Minimal eigenvalue of H
      real(kind=8)             :: emax       ! Maximal eigenvalue of H
      real(kind=8)             :: a          ! Scaling parameter
      real(kind=8)             :: b          ! Scaling parameter
      real(kind=8)             :: Z0         ! Z-term      
      logical                  :: h_deg      ! High degeneracy if .TRUE. => lift it
      type(t_vector), pointer  :: eigv(:)    ! Keep 10 exact eigenvectors
      character(len=100)       :: H_file     ! Hamiltonian file-name
      character(len=100)       :: ptr_file   ! COL_PTR file-name
   end type t_fockspace
   
   type(t_fockspace), allocatable, target :: fockspace(:)
   complex(kind=8),   allocatable         :: HCOL(:),HP(:)
   complex(kind=8),   allocatable         :: GF(:,:,:) ! Green's Function
   real(kind=8),      allocatable         :: x(:)      ! Random numbers
   integer(kind=4),   allocatable         :: row_ind(:),col_ptr(:) ! CCS-addresses
   integer(kind=4)                        :: alloc_stat 
   character(len=200)                     :: fname
   !-------------------------------! Lanczos-block variables !--------------------------------!
   complex(kind=8), allocatable :: ul(:,:)           ! Lanczos vectors   
   real(kind=8),    allocatable :: alp(:), bet(:)    ! Diagonal and off-diagonal elements
   complex(kind=8), allocatable :: al(:,:)           ! Eigenvectors of three-diagonal matrix
   complex(kind=8), allocatable :: vna(:)            ! |na>-vector
   real(kind=8)                 :: EL                ! Minimal eigenvalue (to project out) 
   integer(kind=4)              :: n_L               ! Max.number of Lanczos iterations (==dim)
   integer(kind=4)              :: N_states          ! Number of states incl. ground state
   logical,         parameter   :: SPECTRUM=.TRUE.   ! Find a part of spectrum
   integer(kind=4)              :: lz_unit           ! Lanczos output unit
   !-------------------------------!   KPM-block variables   !--------------------------------!
   complex(kind=8), allocatable :: ksi(:,:)          ! Random vector / Ground state vector     
   real(kind=8),    allocatable :: mu(:),g(:),g2(:)  ! Scalar moments and Kernels
   real(kind=8),    allocatable :: xr(:)             !
   integer(kind=4), allocatable :: sr(:)             ! Random initialization
   integer(kind=4), parameter   :: K_Jackson=1, &    ! Kernel's types
                                   K_Lorentz=2       !
   integer(kind=4)              :: nrv               ! number of random vectors
   
   interface
     subroutine cls_l_diag(h,e,n,a)
       integer(kind=4), intent(in)            :: n
       real(kind=8),    intent(out)           :: e(n)
       complex(kind=8), intent(inout)         :: h(n,n)
       complex(kind=8), optional, intent(out) :: a(n,n)
       !**********************************************!
     end subroutine cls_l_diag
   end interface
   
   private
   public cls_l_main
   
   contains
   !-----------------------------------------------------------------------------------------------------!
   ! Mode ( two-digit integer )                                                                          !
   ! First digit controls:                                                                               !
   ! 1 -- Only calculates Eigenvalues & Eigenvectors.                                                    !
   ! 2 -- Returns the above quantities plus angular moments.                                             !
   ! 3 -- Returns Eigenvalues, Green's function, self-energy, density matrix, DOS.                       !
   ! 4 -- Returns Eigenvalues, Green's function, self-energy, density matrix, DOS, plus angular moments. ! 
   ! Second digit controls:                                                                              !
   ! 1 -- Fix mean filling, search for chemical potential.                                               !
   ! 2 -- Fix chemical potential, search for mean filling.                                               !
   !-----------------------------------------------------------------------------------------------------!
   ! Green's function is presented as   GF = Green1(ms1,ms2,w) + i Green2(ms1,ms2,w)                     !
   ! Real axes      : Green1/Green2(ms1,ms2,w)  are complex (except of ms1=ms2)                          !
   ! Imaginary axes : Green1/Green2(ms1,ms2,iw) are REAL                                                 !
   !-----------------------------------------------------------------------------------------------------!
   subroutine cls_l_main(Nd, Ns, m_max_GF, N_exact, Ed, Es, U, V, B1, B2, B3, SOC, E_grid, Omega,   &
                         Mode, Fill, dfill, drange, Chem, Chem1, Chem2, Temp, Locc,        &
                         Channel, filename, Green1, Green2, Self_E, Self_Einf, Nmom, Mom, Den, DOS)
     !------------- All input & output matrices are in Spherical Harmonics representation ---------------!
     integer(kind=4),    intent(in)    :: Nd                           ! Degeneracy of correlated shell
     integer(kind=4),    intent(in)    :: Ns                           ! Degeneracy of uncorrelated shell
     integer(kind=4),    intent(in)    :: Nmom                         ! Max. number of moments for GF
     integer(kind=4),    intent(in)    :: m_max_GF                     ! Maximal number of Moments (Green's Function) in KPM
     integer(kind=4),    intent(in)    :: N_exact                      ! # of exact eigenstates to resolve with Lancos method
     complex(kind=8),    intent(inout) :: Ed(Nd,Nd)                    ! One-eletron energy matrix of correlated shell
     complex(kind=8),    intent(in)    :: Es(Ns,Ns)                    ! One-eletron energy matrix of uncorrelated shell
     real(kind=8),       intent(in)    :: U(Nd/2,Nd/2,Nd/2,Nd/2)       ! Correlation matrix
     complex(kind=8),    intent(in)    :: V(Nd,Ns)                     ! Hybridization matrix
     real(kind=8),       intent(in)    :: B1                           ! Impurity spin magnetic field
     real(kind=8),       intent(in)    :: B2                           ! Orbital magnetic field
     real(kind=8),       intent(in)    :: B3                           ! Bath spin magnetic field
     real(kind=8),       intent(in)    :: SOC                          ! Spin-orbital coupling
     integer(kind=4),    intent(in)    :: E_grid                       ! Number of energy points in Green's function or self-energy
     complex(kind=8),    intent(in)    :: Omega(E_grid)                ! Energy points array
     integer(kind=4),    intent(in)    :: Mode                         ! Running mode control
     real(kind=8),       intent(inout) :: Fill                         ! Mean filling
     integer(kind=4),    intent(inout) :: dfill                        ! Impurity mean filling
     integer(kind=4),    intent(inout) :: drange                       ! Impurity electron number fluctuation 
     real(kind=8),       intent(inout) :: Chem1,Chem2,Chem             ! Initial values and Chemical potential
     real(kind=8),       intent(in)    :: Temp                         ! Temperature
     integer(kind=4),    intent(in)    :: Locc(0:Nd+Ns)                ! Number of elecrons control array
     integer(kind=4),    intent(in)    :: Channel(5)                   ! I/O channel
     character(len=130), intent(in)    :: filename(5)                  ! Filename string
     complex(kind=8),    intent(inout) :: Green1(Ns+Nd, Ns+Nd, E_grid) ! Real Green's function matrix.
     complex(kind=8),    intent(inout) :: Green2(Ns+Nd, Ns+Nd, E_grid) ! Imagnary Green's function matrix.
     complex(kind=8),    intent(inout) :: Self_E(Ns+Nd, Ns+Nd, E_grid) ! Self-Energy matrix.
     complex(kind=8),    intent(inout) :: Self_Einf(Ns+Nd, Ns+Nd)      ! Self-Energy at infinity.
     complex(kind=8),    intent(inout) :: Mom(Ns+Nd, Ns+Nd,-Nmom:+Nmom)! Moments of Green's function.
     complex(kind=8),    intent(inout) :: Den(Ns+Nd, Ns+Nd,-Nmom:+Nmom)! Moments of Density matrix 
     real(kind=8)   ,    intent(inout) :: DOS(E_grid)                  ! Density Of States
     !--------------------------------------------------------------------------------------------------------!
     complex(kind=8),allocatable :: RG(:,:)                  !
     integer(kind=4)             :: N                        ! Total degeneracy.
     integer(kind=4)             :: ne,nne                   ! Current number of electrons and (+/-)1
     integer(kind=4)             :: nd_min, nd_max           ! Boundaries for electron number on impurity
     integer(kind=4), pointer    :: fdim                     !
     integer(kind=4)             :: ie,je,i,nu,r,gf_sign     !
     integer(kind=4)             :: max_mu_number            !
     integer(kind=4)             :: m_max,n_iter             !
     integer(kind=4)             :: m_unit,na_unit ! Moment's and eigenstates related IO-units
     integer(kind=4)             :: nemin,nemax,nn,na,ma     ! 
     integer(kind=4)             :: ms,ms1,ms2,iw            !
     integer(kind=4)             :: nch,ich,ich_unit         !
     integer(kind=4)             :: h_unit,colptr_unit       !
     integer(kind=4),parameter   :: nrv=20                   ! Number of Random vectors (for scalar moments)
     real(kind=8)                :: emin,emax,enmin,z,pi     ! 
     real(kind=8)                :: avn,avn1,avn2,e_cut      !
     real(kind=8)                :: beta,bb,aa,ab,T_exp      ! Inverse temperature,... temperature-factor
     real(kind=8)                :: T_cut,TSUM,En_max        ! Temperature and energy cut-offs
     real(kind=8)                :: dch,tst                  !
     real(kind=8),   parameter   :: zero=0.0_8               !
     real(kind=8)                :: TML,TMS,TMJ,TS,TL        ! Angular  /    
     real(kind=8)                :: TJ,TMSC,TSC              ! Momentum / 
     real(kind=8),   allocatable :: TOCC(:)                  ! Data
     logical                     :: nch_calc=.false.         ! <N>(chem) calculations 
     character(len=4)            :: chne                     !
     
     write(channel(5),"(/' CLS_L_MAIN ENTERED'/)")          
     call init_timers()
     call start_timer(T_CLS_MAIN)
     if(.NOT.even(Nd)) call FULLSTOP("CLS_L_MAIN : NOT EVEN Nd")
     if(.NOT.even(Ns)) call FULLSTOP("CLS_L_MAIN : NOT EVEN Ns")
     N = Ns + Nd
     if (mod(Mode,10) == 1 .and. nint(Fill) > N) & 
        call fullstop("Wrong!! Mean filling can not be larger than total degeneracy!")

     nd_min = dfill-drange
     if(nd_min <= 0) nd_min = 0
     nd_max = dfill+drange
     if(nd_max >= Nd) nd_max = Nd
     write(*,*) "N, ND_MIN ND_MAX", N, nd_min,nd_max
    
     allocate(fockspace(0:N), stat=alloc_stat)
     call check_allocate(alloc_stat,"FOCKSPACE IN CLS_L_MAIN",+1)
     pi=dacos(-1.d0)
     !            ********* Ne-cycle I *********            ! 
     beta     = 1.d0/Temp    
     lz_unit=free_io_unit()
     fname = trim(adjustl(PROJECT))//"_EL.lnz"   
     open(lz_unit, file=fname) 
     call kernel(m_max_gf,K_Jackson,"allocate") 
     call kernel(m_max_gf,K_Jackson,"generate")
     allocate(TOCC(Nd+Ns), stat=alloc_stat)
     call check_allocate(alloc_stat,"TOCC IN CLS_L_MAIN",+1)      
     ne_I: do ne=0,N
       if(locc(ne) == 0) cycle
         write(channel(5),"(' CLS_L_MAIN: Current number of electrons :',i7)") ne
         write(chne,"(i4)") ne
         fockspace(ne)%H_file = trim(adjustl(PATHSCRT))//trim(adjustl(PROJECT))&
                               //"_H_PACKED_"//trim(adjustl(chne))//"."//trim(adjustl(EXTNSCR))
         fockspace(ne)%ptr_file = trim(adjustl(PATHSCRT))//trim(adjustl(PROJECT))&
                              //"_col_ptr_"//trim(adjustl(chne))//"."//trim(adjustl(EXTNSCR))                               
         call get_fockbasis(ne,Nd,Ns,N,nd_min,nd_max,Es)
         fdim => fockspace(ne)%dim                        
         write(channel(5),"(' CLS_L_MAIN: Dimension of Fock space     :',i7)") fdim
         call print_memory(channel(5),"HAMILTONIAN",16.d0*fdim**2)     
         call Hamiltonian(ne,N,Ns,Nd,Es,Ed,U,V,B1,B2,B3,SOC,1)
         !     ********** Lanczos to find spectrum boundaries **********     !
         emin=zero
         emax=zero
         call Lanczos(fdim,emin,emax,1,.NOT.SPECTRUM,0,0.0_8,0,lz_unit)
         call Lanczos(fdim,emin,emax,2,.NOT.SPECTRUM,0,0.0_8,0,lz_unit)
         if(abs(emax-emin) < 1.d-04)emax = emax + 1.d-04
         fockspace(ne)%emin = emin
         fockspace(ne)%emax = emax
         fockspace(ne)%a    = (emax+emin)/2.d0
         fockspace(ne)%b    = (emax-emin)/(2.d0-0.1d0)
         !     *********** Find N_exact lower states of H(n) ***********     !
         na_unit=free_io_unit()
         fname=trim(adjustl(PATHSCRT))//trim(adjustl(PROJECT))//"_H_VE.scr"
         open(na_unit, file=fname, form='unformatted')  
         allocate(fockspace(ne)%eigv(N_exact), stat=alloc_stat)
         call check_allocate(alloc_stat,"EIGV_EXACT IN CLS_L_MAIN",+1)
         write (channel(5),*) "  #   Energy    ML     MS    MJ    L     &
                                     &S     J    MSC   SC     Occupancy -> N"
         do N_states=1,N_exact
            call Lanczos(fdim,EL,emax,1,SPECTRUM,N_states-1,fockspace(ne)%emax,na_unit,lz_unit)
            write(na_unit) EL,vna
            allocate(fockspace(ne)%eigv(N_states)%coor(fdim), stat=alloc_stat)
            call check_allocate(alloc_stat,"COOR IN CLS_L_MAIN",+1)
            allocate(fockspace(ne)%eigv(N_states)%OCC(Nd+Ns), stat=alloc_stat)
            call check_allocate(alloc_stat,"OCC IN CLS_L_MAIN",+1)
            fockspace(ne)%eigv(N_states)%coor   = dcmplx(zero,zero)
            fockspace(ne)%eigv(N_states)%coor   = vna
            fockspace(ne)%eigv(N_states)%eigval = EL 
            fockspace(ne)%eigv(N_states)%OCC = zero
            call ang(Nd,Ns,ne,fdim,TML,TMS,TMJ,TS,TL,TJ,TOCC,TMSC,TSC)           
            fockspace(ne)%eigv(N_states)%ML  = TML
            fockspace(ne)%eigv(N_states)%MS  = TMS
            fockspace(ne)%eigv(N_states)%MJ  = TMJ
            fockspace(ne)%eigv(N_states)%S   = TS
            fockspace(ne)%eigv(N_states)%L   = TL
            fockspace(ne)%eigv(N_states)%J   = TJ
            fockspace(ne)%eigv(N_states)%OCC = TOCC 
            fockspace(ne)%eigv(N_states)%MSC = TMSC
            fockspace(ne)%eigv(N_states)%SC  = TSC
            write (channel(5),"(I4,F10.3,8F6.2,3X,32F6.2)") N_states,EL,TML,TMS,TMJ,&
                                                            TL,TS,TJ,TMSC,TSC,TOCC(1:N),sum(TOCC)                                               
         end do
         close(na_unit, status="delete")
         deallocate(vna,stat=alloc_stat)
         call check_allocate(alloc_stat,"VNA IN CLS_L_MAIN",-1)
         !     ************* Calculate all types of moments and Z0(ne) *************      !
         bb       = beta*fockspace(ne)%b
         aa       = beta*fockspace(ne)%a  
         m_max    = max_mu_number(bb,1.d-8)
         !     ************* Scale Hamiltonian *************     !
         do ie=1,fdim
            do i=col_ptr(ie),col_ptr(ie+1)-1
               if(row_ind(i)==ie) HP(i)=HP(i)-fockspace(ne)%a
            end do
         end do
         HP=HP / fockspace(ne)%b
         call ksi_set(ne,fdim,nrv,"allocate")
         call ksi_set(ne,fdim,nrv,"generate")
         call moments(m_max,ne,fdim,nrv,K_Jackson,N_exact)
         call ksi_set(ne,fdim,nrv,"deallocate")
         fockspace(ne)%Z0 = mu(0)*bessi(0,bb)  
         do nu=1,m_max
            fockspace(ne)%Z0 = fockspace(ne)%Z0 + 2*(-1)**nu*mu(nu)*bessi(nu,bb)
         end do
         fockspace(ne)%Z0 = dabs(fockspace(ne)%Z0) 
         deallocate(mu, stat=alloc_stat)
         call check_allocate(alloc_stat,"MU IN CLS_L_MAIN",-1)
         deallocate(g, stat=alloc_stat)
         call check_allocate(alloc_stat,"G  IN CLS_L_MAIN",-1)         
         deallocate(HP,stat=alloc_stat)
         call check_allocate(alloc_stat,"HP IN CLS_L_MAIN",-1)
     end do ne_I! Ne-cycle I ]  
     deallocate(TOCC, stat=alloc_stat)
     call check_allocate(alloc_stat,"TOCC IN CLS_L_MAIN",-1)
     !        ********* Calcualte <N>(ch) *********        !
     if(nch_calc) then
       ich_unit=free_io_unit()
       fname = trim(adjustl(PROJECT))//"_CHEM.out"
       open(ich_unit,file=fname)
       chem1=1.5d0
       chem2=3.5d0
       nch=1000
       dch=(chem2-chem1)/nch
       do ich=1,nch
          chem=chem1+(ich-1)*dch
          call get_ZFill(N,locc,beta,chem,N_exact,avn1,Z,enmin)
          write(ich_unit,"(f10.7,3x,f10.7,3x,f10.7)") chem,avn1,Z
       end do
       close(ich_unit)
       call FULLSTOP("CHEM")
     end if
     ! Find Chemical Potential 
     if (mod(Mode,10) == 1) then
        write (channel(5),*) 'Start searching for chemical potential...' 
     call get_ZFill(N,locc,beta,chem,N_exact,avn,Z,enmin)
       if(abs(avn-fill) > 1.d-05)then
       n_iter = 0
       chem1 = chem - 0.183746880653123D0
       chem2 = chem + 0.183746880653123D0
       call lib_findroot(chem,0.d0,chem1,chem2,n_iter,ie,channel(5),1.d-05,50)
       
       find_chem : do 
       n_iter=n_iter+1
       call get_ZFill(N,locc,beta,chem,N_exact,avn,Z,enmin)
	   write(channel(5),fmt='(1X,G14.7,1X,G14.7)')chem,avn
	   call lib_findroot(chem,fill-avn,chem1,chem2,n_iter,ie,channel(5),1.d-05,50)
	   if(ie.eq.1)exit find_chem
       end do find_chem
       end if
     else if (mod(Mode,10) == 2) then 
        write (channel(5),*) 'Start searching for mean filling...' 
     end if
     
     call get_ZFill(N,locc,beta,chem,N_exact,avn,Z,enmin)
     
     write(channel(5),"(' CHEM      = ',f10.6)") chem
     write(channel(5),"('   N       = ',f10.6)") avn
     T_cut=1.d-9
     !            ********* Ne-cycle II *********            ! 
     write(lz_unit,"(/'************* Second cycle *************'/)")
     na_unit=free_io_unit()
     fname=trim(adjustl(PATHSCRT))//trim(adjustl(PROJECT))//"_H_VE.scr"
     open(na_unit, file=fname, form='unformatted')     
     allocate(GF(E_grid,Nd+Ns,Nd+Ns), stat=alloc_stat)
     call check_allocate(alloc_stat,"GF IN CLS_L_MAIN",+1)
     GF=dcmplx(zero,zero)
     Green1 = dcmplx(zero,zero)
     Green2 = dcmplx(zero,zero)
     MOM    = dcmplx(zero,zero)
     DEN    = dcmplx(zero,zero)
     TSUM=zero 
     ne_II : do ne=0,N
        if(locc(ne)==0) cycle ne_II
        E_cut = chem*ne + enmin - dlog(T_cut*Z)/beta
        write(channel(5),"(' n=',i3,4x,'E_cut=',f14.10)") ne, E_cut        
        write(channel(5),"(' Emin=',f14.10,' -> E_max =',f14.10)") fockspace(ne)%emin,E_cut
        if(fockspace(ne)%emin > E_cut) cycle ne_II                                          
        fdim => fockspace(ne)%dim    
        call Hamiltonian(ne,N,Ns,Nd,Es,Ed,U,V,B1,B2,B3,SOC,0)
        N_states=0        
        do 
           if(N_states < N_exact) then
             if(.NOT.allocated(vna)) then
               allocate(vna(fdim), stat=alloc_stat)
               call check_allocate(alloc_stat,"VNA IN CLS_L_MAIN",+1)
             end if
             vna = fockspace(ne)%eigv(N_states+1)%coor
             EL  = fockspace(ne)%eigv(N_states+1)%eigval
             deallocate(fockspace(ne)%eigv(N_states+1)%coor, stat=alloc_stat)
             call check_allocate(alloc_stat,"EIGV(N_STATES+1)%COOR IN CLS_L_MAIN",-1)
             deallocate(fockspace(ne)%eigv(N_states+1)%OCC, stat=alloc_stat)
             call check_allocate(alloc_stat,"EIGV(N_STATES+1)%OCC IN CLS_L_MAIN",-1)             
           else
             write(channel(5),"('ADD NEXT STATE ',i4,' ABOVE N_EXACT = ',i2,' -> LANCZOS')") &
                   N_states+1, N_exact        
             call Lanczos(fdim,EL,emax,1,SPECTRUM,N_states,fockspace(ne)%emax,na_unit,lz_unit)
           end if
           if(EL > E_cut) exit
           N_states=N_states+1
           write(na_unit) EL,vna
           write(channel(5),*) "NE EL Emax", NE, EL, emax
           if(N_states==fdim) exit ! NO STATES LEFT           
        end do
        deallocate(HP, stat=alloc_stat)
        do nn=1,N_exact
           if(associated(fockspace(ne)%eigv(nn)%coor)) then
             deallocate(fockspace(ne)%eigv(nn)%coor, stat=alloc_stat)
             call check_allocate(alloc_stat,"EIGV(NN)%COOR IN CLS_L_MAIN",-1)
             deallocate(fockspace(ne)%eigv(nn)%OCC, stat=alloc_stat)
             call check_allocate(alloc_stat,"EIGV(NN)%OCC IN CLS_L_MAIN",-1)
           end if
        end do
        call check_allocate(alloc_stat,"HP IN CLS_L_MAIN",-1)
        rewind(na_unit)       
        cycle_na : do na=1,N_states
           read(na_unit) EL,vna
           tst = EL
           T_exp = dexp(-beta*(EL-chem*ne-enmin))/Z
           TSUM = TSUM + T_exp 
           !  ********* Calculate Green's Function using Moments of H(n -/+ 1) *********  !          
           do nn=1,2 
              gf_sign=2*nn-3
              nne=ne+gf_sign
              if(locc(nne) /=0 ) then
                 fdim => fockspace(nne)%dim
                 write(channel(5),"(' N (N +/- 1) dim : ',3i5)") ne,nne,fdim                      
                 call Hamiltonian(nne,N,Ns,Nd,Es,Ed,U,V,B1,B2,B3,SOC,0)
                 do ie=1,fdim
                    do i=col_ptr(ie),col_ptr(ie+1)-1
                       if(row_ind(i)==ie) HP(i)=HP(i)-fockspace(nne)%a
                    end do
                 end do
                 HP= gf_sign*HP / fockspace(nne)%b  
                 call moments_GF(Nd,Ns,m_max_GF,nrv,ne,omega,E_grid,chem,T_exp,gf_sign,&
                              Green1,Green2,MOM,DEN,Nmom)
                 deallocate(HP, stat=alloc_stat)
                 call check_allocate(alloc_stat,"HP IN CLS_L_MAIN",+1)            
              end if ! H(n -/+ 1)
           end do ! nn
           write(channel(5),*) "****************************"      
        end do cycle_na 
        deallocate(vna, stat=alloc_stat)
        call check_allocate(alloc_stat,"VNA IN CLS_L_MAIN",-1)
        rewind(na_unit)
     end do ne_II ! ne
     write(channel(5),*) "SUM T_exp = ",TSUM 
     close(na_unit, status="delete")
     call get_DOS(Nd,Ns,DOS,omega,E_grid,.FALSE.)
     call get_sigma(Nd,Ns,Es,Ed,V,chem,omega,E_grid,MOM,Self_E,Self_Einf)
     !do ms1=1,Nd+Ns
     !   write(*,"('ms=',i2,3x,6(d12.6,1x,d12.6,3x))") ms1,MOM(ms1,ms1,0:5)
     !end do
     call kernel(m_max_gf,K_Jackson,"deallocate")
     deallocate(GF, stat=alloc_stat)
     call check_allocate(alloc_Stat,"GF IN CLS_L_MAIN",-1)
     !close(mlog_unit)
     close(lz_unit, status="keep")  
     !                ********* Revert ms-indexes *********                !
     allocate(RG(N,N), stat=alloc_stat)
     call check_allocate(alloc_stat,"RG IN CLS_L_MAIN",+1)
     if(.true.) then
     do iw=1,E_grid
        RG = Green1(:,:,iw)
        Green1(1:Ns,1:Ns,iw)     = RG(Nd+1:N,Nd+1:N)
        Green1(Ns+1:N,1:Ns,iw)   = RG(1:Nd,Nd+1:N)
        Green1(1:Ns,  Ns+1:N,iw) = RG(Nd+1:N,1:Nd)
        Green1(Ns+1:N,Ns+1:N,iw) = RG(1:Nd,1:Nd)
        RG = Green2(:,:,iw)
        Green2(1:Ns,1:Ns,iw)     = RG(Nd+1:N,Nd+1:N)
        Green2(Ns+1:N,1:Ns,iw)   = RG(1:Nd,Nd+1:N)
        Green2(1:Ns,  Ns+1:N,iw) = RG(Nd+1:N,1:Nd)
        Green2(Ns+1:N,Ns+1:N,iw) = RG(1:Nd,1:Nd)
        RG = Self_E(:,:,iw)
        Self_E(1:Ns,1:Ns,iw)     = RG(Nd+1:N,Nd+1:N)
        Self_E(Ns+1:N,1:Ns,iw)   = RG(1:Nd,Nd+1:N)
        Self_E(1:Ns,  Ns+1:N,iw) = RG(Nd+1:N,1:Nd)
        Self_E(Ns+1:N,Ns+1:N,iw) = RG(1:Nd,1:Nd)               
     end do
     do iw=-5,5
        RG = MOM(:,:,iw)
        MOM(1:Ns,1:Ns,iw)     = RG(Nd+1:N,Nd+1:N)
        MOM(Ns+1:N,1:Ns,iw)   = RG(1:Nd,Nd+1:N)
        MOM(1:Ns,  Ns+1:N,iw) = RG(Nd+1:N,1:Nd)
        MOM(Ns+1:N,Ns+1:N,iw) = RG(1:Nd,1:Nd) 
     end do
     do iw=-5,5
        RG = DEN(:,:,iw)
        DEN(1:Ns,1:Ns,iw)     = RG(Nd+1:N,Nd+1:N)
        DEN(Ns+1:N,1:Ns,iw)   = RG(1:Nd,Nd+1:N)
        DEN(1:Ns,  Ns+1:N,iw) = RG(Nd+1:N,1:Nd)
        DEN(Ns+1:N,Ns+1:N,iw) = RG(1:Nd,1:Nd) 
     end do     
     RG = Self_Einf
     Self_Einf(1:Ns,1:Ns)     = RG(Nd+1:N,Nd+1:N)
     Self_Einf(Ns+1:N,1:Ns)   = RG(1:Nd,Nd+1:N)
     Self_Einf(1:Ns,  Ns+1:N) = RG(Nd+1:N,1:Nd)
     Self_Einf(Ns+1:N,Ns+1:N) = RG(1:Nd,1:Nd)   
     end if
     deallocate(RG, stat=alloc_stat)
     call check_allocate(alloc_stat,"RG IN CLS_L_MAIN",-1)
     do ne=0,N
        if(locc(ne)==0) cycle
        deallocate(fockspace(ne)%phis, stat=alloc_stat)
        call check_allocate(alloc_stat,"FOCKSPACE%PHIS IN CLS_L_MAIN",-1) 
        deallocate(fockspace(ne)%phid, stat=alloc_stat)
        call check_allocate(alloc_stat,"FOCKSPACE%PHID IN CLS_L_MAIN",-1)
        deallocate(fockspace(ne)%phi,  stat=alloc_stat)
        call check_allocate(alloc_stat,"FOCKSPACE%PHI IN CLS_L_MAIN",-1)
        deallocate(fockspace(ne)%eigv, stat=alloc_stat)
        call check_allocate(alloc_stat,"FOCKSPACE%EIGV IN CLS_L_MAIN",-1)
        h_unit=free_io_unit()
        open(h_unit, file=fockspace(ne)%H_file, form="UNFORMATTED")
        close(h_unit, status="delete")
        colptr_unit=free_io_unit()
        open(colptr_unit, file=fockspace(ne)%ptr_file, form="UNFORMATTED")
        close(colptr_unit, status="delete")  
     end do
     deallocate(fockspace, stat=alloc_stat)
     call check_allocate(alloc_stat,"FOCKSPACE IN CLS_L_MAIN",-1)
     deallocate(row_ind, stat=alloc_stat)
     call check_allocate(alloc_stat,"ROW_IND IN CLS_L_MAIN",-1)
     deallocate(col_ptr, stat=alloc_stat)
     call check_allocate(alloc_stat,"COL_PTR IN CLS_L_MAIN",-1)
     call stop_timer(T_CLS_MAIN)
     call print_time(channel(5))
     
   end subroutine cls_l_main
 
   !------------------------------------------------------------------------!
   !  Generating Hamiltonian matrix                                         !
   !  optype = 1 - calculate and save Hamiltonian                           !
   !           0 - only read Hamiltonian                                    !
   !------------------------------------------------------------------------!
   subroutine Hamiltonian(ne,N,Ns,Nd,Es,Ed,U,V,B1,B2,B3,SOC,optype)
     integer(kind=4), intent(in) :: ne,N,Ns,Nd,optype
     complex(kind=8), intent(in) :: Es(Ns,Ns),Ed(Nd,Nd),V(Nd,Ns)
     real(kind=8),    intent(in) :: U(Nd/2,Nd/2,Nd/2,Nd/2)
     real(kind=8),    intent(in) :: B1,B2,B3,SOC
     !----------------------------------------------------!
     integer(kind=4), parameter  :: d_create=1,d_destroy=-1
     integer(kind=4)             :: i,j,ms1,ms2,ks1,ks2,m1,m2,k1,k2
     integer(kind=4)             :: d1_sign,d2_sign
     integer(kind=4)             :: d3_sign,d4_sign
     integer(kind=4)             :: alloc_stat,s_ks1,s_ks2,l
     integer(kind=kind_phi), pointer :: phi(:),phid(:),phis(:)
     integer(kind=kind_phi)          :: d1_phi,d2_phi,d3_phi,d4_phi
     integer(kind=4), pointer    :: dim
     integer(kind=4), pointer    :: n_rec
     integer(kind=4)             :: inv
     integer(kind=4)             :: n_nonzero
     integer(kind=4)             :: hcol_unit,colptr_unit
     integer(kind=4)             :: ir
     
     call start_timer(T_H)  
     dim =>  fockspace(ne)%dim
     phi =>  fockspace(ne)%phi
     phid => fockspace(ne)%phid
     phis => fockspace(ne)%phis
     n_rec => fockspace(ne)%n_rec
     if(allocated(col_ptr)) then
        deallocate(col_ptr, stat=alloc_stat)
        call check_allocate(alloc_stat,"COL_PTR IN HAMILTONIAN",-1)
     end if
     allocate(col_ptr(dim+1), stat=alloc_stat)
     call check_allocate(alloc_stat,"COL_PTR IN HAMILTONIAN",+1)               
     if(allocated(row_ind)) then
        deallocate(row_ind,stat=alloc_stat)
        call check_allocate(alloc_stat,"ROW_IND in HAMILTONIAN",-1)
     end if
     col_ptr(:)=0     
     ir=0
     hcol_unit=free_io_unit()
     open(hcol_unit, file=fockspace(ne)%H_file, form="UNFORMATTED")
     colptr_unit=free_io_unit()
     open(colptr_unit, file=fockspace(ne)%ptr_file, form="UNFORMATTED")     
     
     if(optype==1) then ! calculate Hamiltonian
     allocate(HCOL(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"HCOL IN HAMILTONIAN",+1)
     n_rec=0
     col_ptr(1)=1
     do J=1,dim
        HCOL=(0.0d0,0.0d0)
        ! ********** Noninteracting Impurity Hamiltonian ********** !  
        do ms2=1,Nd
           if(.NOT.allowed(0,d_destroy,ms2,N,phi(J),d1_sign,d1_phi) ) cycle
             do ms1=1,Nd
                if(.NOT.allowed(0,d_create,ms1,N,d1_phi,d2_sign,d2_phi) ) cycle
                I=inv(d2_phi,dim,ne)
                HCOL(I)=HCOL(I) + Ed(ms1,ms2)*d1_sign*d2_sign
             end do
        end do
        ! ********** Correlation Part of Impurity : U ********** !              
        do ks1=1,Nd
           if(.NOT.allowed(0,d_destroy,ks1,N,phi(J),d1_sign,d1_phi) ) cycle
           s_ks1=spin(ks1,Nd)
           do ks2=1,Nd
              if(.NOT.allowed(0,d_destroy,ks2,N,d1_phi,d2_sign,d2_phi) ) cycle
              s_ks2=spin(ks2,Nd)
              do ms2=1,Nd
                 if(.NOT.allowed(0,d_create,ms2,N,d2_phi,d3_sign,d3_phi) ) cycle
                 if(spin(ms2,Nd) /= s_ks2) cycle              
                 do ms1=1,Nd
                    if(.NOT.allowed(0,d_create,ms1,N,d3_phi,d4_sign,d4_phi) ) cycle
                    if(spin(ms1,Nd) /= s_ks1) cycle
                    I=inv(d4_phi,dim,ne)
                    m1=mod(ms1-1,Nd/2)+1 
                    m2=mod(ms2-1,Nd/2)+1
                    k1=mod(ks1-1,Nd/2)+1 
                    k2=mod(ks2-1,Nd/2)+1 
                    HCOL(I)=HCOL(I) + 0.5_8*U(m1,k1,m2,k2)*d1_sign*d2_sign*d3_sign*d4_sign                   
                 end do
              end do
           end do
        end do
        ! ********** Noninteracting Bath Hamiltonian ********** !
        do ms2=1,Ns
           if(.NOT.allowed(Nd,d_destroy,ms2,N,phi(J),d1_sign,d1_phi) ) cycle
           do ms1=1,Ns
              if(.NOT.allowed(Nd,d_create,ms1,N,d1_phi,d2_sign,d2_phi) ) cycle
              I=inv(d2_phi,dim,ne)
              if(I/=0) HCOL(I)=HCOL(I) + Es(ms1,ms2)*d1_sign*d2_sign  ! inserted if(I/=0) 
           end do
        end do        
        ! ********** Hybridization Hamiltonian ********** !
        do ks1=1,Ns
           if(allowed(Nd,d_destroy,ks1,N,phi(J),d1_sign,d1_phi) ) then ! 1st term
              do ms1=1,Nd
                 if(.NOT.allowed(0,d_create,ms1,N,d1_phi,d2_sign,d2_phi) ) cycle
                 I=inv(d2_phi,dim,ne)
                 if(I/=0) HCOL(I)=HCOL(I) + V(ms1,ks1)*d1_sign*d2_sign
              end do
           end if
        end do
        do ms1=1,Nd
           if(allowed(0,d_destroy,ms1,N,phi(J),d1_sign,d1_phi) ) then ! 2nd term
              do ks1=1,Ns
                 if(.NOT.allowed(Nd,d_create,ks1,N,d1_phi,d2_sign,d2_phi) ) cycle
                 I=inv(d2_phi,dim,ne)
                 if(I/=0) HCOL(I)=HCOL(I) + dconjg(V(ms1,ks1))*d1_sign*d2_sign
              end do
           end if
        end do
        ! ********** Impurity Magnetic Field Hamiltonian ********** !
        do ms1=1,Nd
           if(.NOT.occupied(0,ms1,phi(J),N)) cycle
           m1=mod(ms1-1,Nd/2)+1 - (Nd+2)/4     
           HCOL(J)=HCOL(J) - B1*spin(ms1,Nd) - B2*m1
        end do
        ! ********** Bath Magnetic Field Hamiltonian ********** !
        do ks1=1,Ns
           if(.NOT.occupied(Nd,ks1,phi(J),N)) cycle
           HCOL(J)=HCOL(J) - B3*spin(ks1,Ns)
        end do
        ! ********** Spin-Orbit Impurity Hamiltonian ********** !
        l = (Nd-2)/4                      
        do ms2=1,Nd
           if(.NOT.allowed(0,d_destroy,ms2,N,phi(J),d1_sign,d1_phi) ) cycle
           m2=mod(ms2-1,Nd/2)+1 - (Nd+2)/4
           do ms1=1,Nd
              if(.NOT.allowed(0,d_create,ms1,N,d1_phi,d2_sign,d2_phi) ) cycle
              I=inv(d2_phi,dim,ne)
              m1=mod(ms1-1,Nd/2)+1 - (Nd+2)/4
              if(I/=0) HCOL(I)=HCOL(I) + d1_sign*d2_sign*hso(l,m1,spin(ms1,Nd),m2,spin(ms2,Nd),SOC)
           end do
        end do
        !            ********* Pack Hamiltonian Column *********             !
        !            and always save diagonal to scale H later on            !
        col_ptr(J+1)=col_ptr(J)
        do I=1,dim
           if(cdabs(HCOL(I))>=1.d-9 .OR. I==J) then
              col_ptr(J+1)=col_ptr(J+1)+1
              n_rec=n_rec+1
              write(hcol_unit),I,HCOL(I)
           end if
        end do
     end do ! J
     write(colptr_unit) col_ptr
     deallocate(HCOL,stat=alloc_stat)
     call check_allocate(alloc_stat,"HCOL IN HAMILTONIAN",-1)
     rewind(hcol_unit)
     close(colptr_unit, status="keep")
     end if ! calculate Hamiltonian
     
     allocate(row_ind(n_rec), stat=alloc_stat)
     call check_allocate(alloc_stat,"ROW_IND IN HAMILTONIAN",+1)
     allocate(HP(n_rec), stat=alloc_stat)
     call check_allocate(alloc_stat,"HP IN HAMILTONIAN",+1)
     do ir=1,n_rec
        read(hcol_unit) row_ind(ir),HP(ir)      
     end do
     close(hcol_unit,status="keep")
     if(optype==0) then
       read(colptr_unit) col_ptr
       close(colptr_unit, status="keep")
     end if     

     nullify(dim)
     nullify(phi) 
     nullify(phid) 
     nullify(phis) 
     call stop_timer(T_H)
     
   end subroutine Hamiltonian
 
   !------------------------------------------------------------------------!
   !  Generating Fock space                                                 !
   !------------------------------------------------------------------------!
   subroutine get_fockbasis(ne,Nd,Ns,N,nd_min,nd_max,Es)
     integer(kind=4), intent(in) :: ne,N,Nd,Ns,nd_min,nd_max
     complex(kind=8), intent(in) :: Es(Ns,Ns)
     !-------------------------------------------------------!
     integer(kind=kind_phi)      :: Id,Is,itwo,phimin
     integer(kind=4)             :: kd,ks,occ_d,J,JJ,JMIN
     integer(kind=4)             :: ifoc,nfocm,nfocb
     integer(kind=4), pointer    :: dim
     integer(kind=4), allocatable:: ifbath(:,:),ifmem(:),iphis(:)
     real(kind=8), allocatable   :: spec(:)
    ! Generate bath states
     dim => fockspace(ne)%dim
     dim=0
     nfocb=0
     allocate(iphis(nfocb))
     do kd=nd_min,nd_max ! 0,Nd
        if( kd >= ne-Ns .and. kd<=ne ) then
           ks = ne-kd ! current bath occupancy
    ! Find out dimension of hilbert space
           nfocm = 0
           allocate(spec(nfocm))
           allocate(ifbath(0:Ns,nfocm))
           call RAT_CLSDIAG_BATH(spec,ifbath,ks,nfocm,ifoc)
           deallocate(spec,ifbath)
    ! Generate bath hilbert space
           nfocm = ifoc           
           allocate(spec(nfocm))
           allocate(ifbath(0:Ns,nfocm))
           call RAT_CLSDIAG_BATH(spec,ifbath,ks,nfocm,ifoc)
           dim = dim + bincoef(Nd,kd)*ifoc
    ! Record previous states
           allocate(ifmem(nfocb))
           do ks=1,nfocb
           ifmem(ks)=iphis(ks)
           end do
           deallocate(iphis)
    ! Reallocate array and record new states
           allocate(iphis(nfocb+ifoc))
           do ks=1,nfocb
           iphis(ks)=ifmem(ks)
           end do
           do ks=nfocb+1,nfocb+ifoc
    ! Alexei's numbering is reverse (from right to left), recompute binaries:
           Is=0
           itwo=1
           do j=1,Ns
           Is=Is+ifbath(Ns-j+1,ks-nfocb)*itwo
           itwo=itwo*2
           end do
           ifbath(0,ks-nfocb)=Is
           iphis(ks)=Is
           end do
    ! Advance total number of bath states
           nfocb=nfocb+ifoc
           deallocate(spec,ifbath,ifmem)
        end if
     end do
     write(*,*) "FOCKSPACE ", ne,dim
     ! Now we are back with Alexei's algorithm     
     allocate(fockspace(ne)%phi(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"fockspace(ne)%phi in get_fockspace",+1)
     allocate(fockspace(ne)%phis(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"fockspace(ne)%phi in get_fockspace",+1)
     allocate(fockspace(ne)%phid(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"fockspace(ne)%phi in get_fockspace",+1)
     fockspace(ne)%phi  = 0
     fockspace(ne)%phid = 0
     fockspace(ne)%phis = 0
     J=0
     do Id=0,2**Nd-1
        occ_d = occupation(Nd,Id)
          if( occ_d >= nd_min .AND. occ_d <= nd_max .AND. &
              occ_d >= ne-Ns  .AND. occ_d <=ne  ) then  ! if(
           do ifoc=1,nfocb
           Is = iphis(ifoc)
              if(occ_d+occupation(Ns,Is) == ne) then
                J=J+1
                fockspace(ne)%phid(J) = Id
                fockspace(ne)%phis(J) = Is 
                fockspace(ne)%phi(J)  = join(Id,Ns,Is)
              end if
           end do ! Is
        end if
     end do ! Id
     !             ********* Check number of states *********             !
     if( J /= fockspace(ne)%dim ) then
      write(*,"(/3x,'N_J = ',i8,'  =/=  FOCK_DIM = ',i8/)") J,dim 
      call FULLSTOP("GET_FOCKSPACE : STOP")
     end if
     ! erase memory
     deallocate(iphis) 
     !             ********* Order Fock basis (for sure) *********             !
   do J=1,dim
        phimin = fockspace(ne)%phi(J)
        JMIN=J
        do JJ=J+1,dim
           if(fockspace(ne)%phi(JJ) < phimin)  then
              phimin = fockspace(ne)%phi(JJ)
              Id     = fockspace(ne)%phid(JJ)
              Is     = fockspace(ne)%phis(JJ)
              JMIN = JJ
           end if
        end do
        fockspace(ne)%phi(JMIN) = fockspace(ne)%phi(J)
        fockspace(ne)%phi(J) = phimin
        fockspace(ne)%phis(JMIN) = fockspace(ne)%phis(J)
        fockspace(ne)%phis(J) = Is
        fockspace(ne)%phid(JMIN) = fockspace(ne)%phid(J)
        fockspace(ne)%phid(J) = Id
     end do

   end subroutine get_fockbasis




   !------------------------------------------------------------------------!
   !  Generating Fock space (Alexei's version                               !
   !------------------------------------------------------------------------!
   subroutine get_fockbasis_ag(ne,Nd,Ns,N,nd_min,nd_max,Es)
     integer(kind=4), intent(in) :: ne,N,Nd,Ns,nd_min,nd_max
     complex(kind=8), intent(in) :: Es(Ns,Ns)
     !----------------------------------------------------------------!
     integer(kind=kind_phi)      :: Id,Is
     integer(kind=4)             :: occ_d,J,JJ,JMIN,phimin,Ns_occ
     integer(kind=4)             :: ms,N_exc
     integer(kind=4), pointer    :: dim
     real(kind=8)                :: Efb
     
     Efb=0.0_8
     dim => fockspace(ne)%dim
     dim=0
     !         ********* Empty cycle to define dimension of the Fockspace *********         !
     do Id=0,2**Nd-1
        occ_d = occupation(Nd,Id)
          if( occ_d >= nd_min .AND. occ_d <= nd_max .AND. &
              occ_d >= ne-Ns  .AND. occ_d <=ne  ) then  ! if(
           do Is=0,2**Ns-1
              Ns_occ = occupation(Ns,Is)
              if(occ_d+Ns_occ == ne) then
                N_exc=0
                do ms=1,Ns
                   if(occupied(0,ms,Is,Ns) .AND. real(Es(ms,ms))>Efb  ) N_exc=N_exc+1
                end do
                if(N_exc <= 1) then ! Yet 3-particle excitation explicitly !
                dim=dim+1
                end if
              end if
           end do ! Is
        end if
     end do ! Id
     write(*,*) "FOCKSPACE ", ne,dim
     allocate(fockspace(ne)%phi(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"fockspace(ne)%phi in get_fockspace",+1)
     allocate(fockspace(ne)%phis(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"fockspace(ne)%phi in get_fockspace",+1)
     allocate(fockspace(ne)%phid(dim), stat=alloc_stat)
     call check_allocate(alloc_stat,"fockspace(ne)%phi in get_fockspace",+1)
     fockspace(ne)%phi  = 0
     fockspace(ne)%phid = 0
     fockspace(ne)%phis = 0
     J=0
     do Id=0,2**Nd-1
        occ_d = occupation(Nd,Id)
          if( occ_d >= nd_min .AND. occ_d <= nd_max .AND. &
              occ_d >= ne-Ns  .AND. occ_d <=ne  ) then  ! if(
           do Is=0,2**Ns-1
              Ns_occ = occupation(Ns,Is)
              if(occ_d+Ns_occ == ne) then
                N_exc=0
                do ms=1,Ns
                   if(occupied(0,ms,Is,Ns) .AND. real(Es(ms,ms))>Efb  ) N_exc=N_exc+1
                end do
                if(N_exc <= 1) then ! Yet 3-particle excitation explicitly !
                J=J+1
                fockspace(ne)%phid(J) = Id
                fockspace(ne)%phis(J) = Is 
                fockspace(ne)%phi(J)  = join(Id,Ns,Is)
                end if
              end if
           end do ! Is
        end if
     end do ! Id
     !call FULLSTOP("GET_FOCKSPACE : !!! STOP")
     !             ********* Check number of states *********             !
     if( J /= fockspace(ne)%dim ) then
      write(*,"(/3x,'N_J = ',i8,'  =/=  FOCK_DIM = ',i8/)") J,dim 
      call FULLSTOP("GET_FOCKSPACE : STOP")
     end if
     !             ********* Order Fock basis (for sure) *********             !
     do J=1,dim
        phimin = fockspace(ne)%phi(J)
        JMIN=J
        do JJ=J+1,dim
           if(fockspace(ne)%phi(JJ) < phimin)  then
              phimin = fockspace(ne)%phi(JJ)
              JMIN = JJ
           end if
        end do
        fockspace(ne)%phi(JMIN) = fockspace(ne)%phi(J)
        fockspace(ne)%phi(J) = phimin
     end do
 
   end subroutine get_fockbasis_ag

   
   !------------------------------------------------------------------------!
   !  Reverse table (Binary search)                                         !
   !------------------------------------------------------------------------!
   function inv(phi,dim,ne)
   integer                     :: inv
   integer(kind=kind_phi), intent(in) :: phi
   integer(kind=4), intent(in) :: dim,ne
   !---------------------------------------!
   integer(kind=4)             :: lo,hi,mid
 
     inv=0
     lo=1
     hi=dim+1
     do
       if(lo >= hi-1) exit
       mid=(lo+hi)/2
       if(phi >= fockspace(ne)%phi(mid) ) then
          lo = mid
       else
          hi = mid
       end if
     end do
     if(fockspace(ne)%phi(lo)==phi) inv = lo

   end function inv
   
   !------------------------------------------------------------------------!
   !  Spin-Spin part of Spin-Orbit Impurity Hamiltonian                     !
   !------------------------------------------------------------------------!
   function hso(l,m1,s1,m2,s2,a_SO)
     real(kind=8)                :: hso
     real(kind=8),    intent(in) :: a_SO
     integer(kind=4), intent(in) :: l,m1,s1,m2,s2
     !------------------------------------------!
     
     select case(s1)
     case(+1)
       select case(s2)
       case(+1)
         hso = delta_k(m1,m2)*m2
       case(-1)
         hso = delta_k(m1,m2-1)*sqrt(real((l+m2)*(l-m2+1),8))
       end select
     case(-1)
       select case(s2)
       case(+1)
         hso = delta_k(m1,m2+1)*sqrt(real((l-m2)*(l+m2+1),8))
       case(-1)
         hso =-delta_k(m1,m2)*m2
       end select     
     end select
     hso = a_SO*hso/2  
   end function hso  

   !------------------------------------------------------------------------!
   ! Calculation of Angular Momentum Data for eigenstate |vna>              !
   ! !!! state by state to reduce memory requirements for large "fdim"    !!!    
   !------------------------------------------------------------------------!
   subroutine ang(Nd,Ns,ne,fdim,ML,MS,MJ,S,L,J,OCC,MSC,SC)
     integer(kind=4), intent(in)    :: Nd,Ns,ne,fdim
     real(kind=8),    intent(inout) :: OCC(Ns+Nd)
     real(kind=8),    intent(inout) :: ML,MS,MJ,S,L,J,MSC,SC
     !-----------------------------------------------------!
     complex(kind=8), allocatable   :: Lna(:),Sna(:)
     complex(kind=8)                :: ddvna
     real(kind=8),    allocatable   :: l_matrix(:,:,:)
     real(kind=8),    allocatable   :: s_matrix(:,:,:)
     real(kind=8)                   :: m1_L_m2,s1_S_s2
     integer(kind=4), parameter     :: d_create=1,d_destroy=-1
     integer(kind=kind_phi), pointer :: phi(:)
     integer(kind=kind_phi)          :: d1_phi,d2_phi
     integer(kind=4)                :: N,Ld,mu
     integer(kind=4)                :: d1_sign,d2_sign
     integer(kind=4)                :: ms1,ms2,i,jj,m,ss,m1,s1,m2,s2
     integer(kind=4)                :: alloc_stat

     call start_timer(T_ANG)     
     N=Nd+Ns
     phi =>  fockspace(ne)%phi
     OCC = 0.d0
     ML  = 0.d0
     MS  = 0.d0     
     L=0.d0
     S=0.d0
     J=0.d0
     MSC = 0.d0
     SC  = 0.d0 
     Ld=(Nd-2)/4 
     allocate(Lna(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"LNA IN ANG",+1)
     allocate(Sna(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"SNA IN ANG",+1)     
     allocate(l_matrix(-Ld:Ld,-1:1,-Ld:Ld), stat=alloc_stat)
     call check_allocate(alloc_stat,"L_MATRIX IN ANG",+1)
     allocate(s_matrix(-1:1,-1:1,-1:1), stat=alloc_stat)
     call check_allocate(alloc_stat,"S_MATRIX IN ANG",+1)     
     do ms1=1,Nd+Ns
        do i=1,fdim
           if(.NOT.occupied(0,ms1,phi(i),N)) cycle
           OCC(ms1) = OCC(ms1) + cdabs(vna(i))**2
        end do
        if(ms1<=Nd) then
           m=mod(ms1-1,Nd/2)+1 - (Nd+2)/4
           ss=spin(ms1,Nd)
           ML = ML + m*OCC(ms1)
           MS = MS + 0.5d0*ss*OCC(ms1)
        else
           ss=spin(ms1-Nd,Ns)
        end if
        MSC = MSC +  0.5d0*ss*OCC(ms1) 
     end do
     MJ  = ML + MS       
     call get_l_matrix(Ld,l_matrix) 
     call get_s_matrix(s_matrix)
     do mu=-1,1
        !                  ********* {S_mu,L_mu}|na> (Impurity)*********                  !    
        Lna = dcmplx(0.d0,0.d0)
        Sna = dcmplx(0.d0,0.d0)    
        do ms1=1,Nd
           m1=mod(ms1-1,Nd/2)+1 - (Nd+2)/4
           s1=spin(ms1,Nd)
           do ms2=1,Nd
              m2=mod(ms2-1,Nd/2)+1 - (Nd+2)/4
              s2=spin(ms2,Nd)
              if(m2 /= m1 .AND. s2 /= s1) cycle ! ms2
              m1_L_m2 = l_matrix(m1,mu,m2)
              s1_S_s2 = s_matrix(s1,mu,s2)
              do i=1,fdim
                 if(.NOT.allowed(0,d_destroy,ms2,Nd+Ns,phi(i),d2_sign,d2_phi)) cycle
                 if(.NOT.allowed(0, d_create,ms1,Nd+Ns,d2_phi,d1_sign,d1_phi)) cycle 
                 JJ=inv(d1_phi,fdim,ne)
                 if(JJ==0) cycle
                 ddvna = d1_sign*d2_sign*vna(i)
                 if(s2==s1) Lna(JJ) = Lna(JJ) + m1_L_m2*ddvna
                 if(m2==m1) Sna(JJ) = Sna(JJ) + s1_S_s2*ddvna
              end do
           end do ! ms2
        end do ! ms1
        L = L + dot_product(Lna,Lna)
        S = S + dot_product(Sna,Sna)
        J = J + dot_product(Lna+Sna,Lna+Sna)
        !                  ********* {S_mu}|na> (Bath)*********                  !        
        Lna = dcmplx(0.d0,0.d0)
        do ms1=1,Ns
           m1=mod(ms1,Ns/2)+1 - (Ns+2)/4
           s1=spin(ms1,Ns)
           do ms2=1,Ns
              m2=mod(ms2,Ns/2)+1 - (Ns+2)/4
              s2=spin(ms2,Ns)
              s1_S_s2 = s_matrix(s1,mu,s2)
              if(m1/=m2) cycle
              do i=1,fdim
                 if(.NOT.allowed(Nd,d_destroy,ms2,Nd+Ns,phi(i),d2_sign,d2_phi)) cycle
                 if(.NOT.allowed(Nd, d_create,ms1,Nd+Ns,d2_phi,d1_sign,d1_phi)) cycle 
                 JJ=inv(d1_phi,fdim,ne)
                 if(JJ==0) cycle
                 Lna(JJ) = Lna(JJ) + s1_S_s2*d1_sign*d2_sign*vna(i)
              end do 
           end do ! ms2
        end do    ! ms1
        SC  = SC  + dot_product(Sna+Lna,Sna+Lna)        
     end do ! mu
     L  = -0.5d0 + 0.5d0*dsqrt(1.d0+4*L)
     S  = -0.5d0 + 0.5d0*dsqrt(1.d0+4*S)
     J  = -0.5d0 + 0.5d0*dsqrt(1.d0+4*J)
     SC = -0.5d0 + 0.5d0*dsqrt(1.d0+4*SC)     
     deallocate(Lna, stat=alloc_stat)
     call check_allocate(alloc_stat,"LNA IN ANG",-1)
     deallocate(Sna, stat=alloc_stat)
     call check_allocate(alloc_stat,"SNA IN ANG",-1)      
     deallocate(l_matrix, stat=alloc_stat)
     call check_allocate(alloc_stat,"L_MATRIX IN ANG",+1)
     deallocate(s_matrix, stat=alloc_stat)
     call check_allocate(alloc_stat,"S_MATRIX IN ANG",-1)
     call stop_timer(T_ANG)
         
   end subroutine ang

   !------------------------------------------------------------------------!
   ! Calculation of <m'|l_mu|m> matrix                                      !
   !------------------------------------------------------------------------!   
   subroutine get_l_matrix(L,matrix)
     integer(kind=4), intent(in)    :: L
     real(kind=8),    intent(inout) :: matrix(-L:L,-1:1,-L:L)
     !------------------------------------------------------!
     integer(kind=4)                :: mu,m1,m2
     
       do m1=-L,L
       do m2=-L,L
         matrix(m1,-1,m2) = delta_k(m1,m2-1)*dsqrt(dfloat((L+m2)*(L-m2+1)))/dsqrt(2.d0)
         matrix(m1, 0,m2) = delta_k(m1,m2)*dfloat(m2)
         matrix(m1, 1,m2) =-delta_k(m1,m2+1)*dsqrt(dfloat((L-m2)*(L+m2+1)))/dsqrt(2.d0) 
       end do
       end do
   
   end subroutine get_l_matrix

   !------------------------------------------------------------------------!
   ! Calculation of <s'|s_mu|s> spin-matrix                                 !
   !------------------------------------------------------------------------!    
   subroutine get_s_matrix(s_matrix)
      real(kind=8), intent(inout) :: s_matrix(-1:1,-1:1,-1:1)
      !-----------------------------------------------------!
 
      s_matrix = 0.d0           
      s_matrix(-1,-1, 1) = 1.d0/dsqrt(2.d0)           
      s_matrix( 1, 0, 1) = 0.5d0
      s_matrix(-1, 0,-1) =-0.5d0                            
      s_matrix( 1, 1,-1) =-1.d0/dsqrt(2.d0)            
              
   end subroutine get_s_matrix
   
   !------------------------------------------------------------------------!
   !  Calculation of the partition function, mean filling at "ch" and       !
   !  scaling energy using N_exact eigenstates plus KPM-correction from the !
   !  of H(n) spectrum                                                      !
   !------------------------------------------------------------------------!      
   subroutine get_ZFill(N,locc,beta,ch,N_exact,avn,Z,enmin)
     integer(kind=4), intent(in)  :: N,N_exact,locc(0:N)
     real(kind=8),    intent(in)  :: beta,ch 
     real(kind=8),    intent(out) :: Z,avn,enmin
     !-----------------------------------------! 
     real(kind=8)                 :: ab,Z1,Ea
     integer(kind=4)              :: ne,ieigv
     
     Z=0.0_8
     enmin=10.d10
     do ne=0,N
        if(locc(ne)==0) cycle
          ab = fockspace(ne)%emin- ch*ne
          if(ab <= enmin) enmin=ab
     end do
     do ne=0,N
        if(locc(ne)==0) cycle
        do ieigv=1,N_exact
           Ea=fockspace(ne)%eigv(ieigv)%eigval
           Z = Z + safedexp(-beta*(Ea-ch*ne-enmin) )
        end do
        ab = fockspace(ne)%emin- ch*ne
        Z = Z + fockspace(ne)%Z0 * safedexp(-beta*(ab-enmin) )
     end do
     avn=0.0_8
     do ne=0,N
        if(locc(ne)==0) cycle
        Z1=0.d0
        do ieigv=1,N_exact
           Ea=fockspace(ne)%eigv(ieigv)%eigval
           Z1 = Z1 + safedexp(-beta*(Ea-ch*ne-enmin) )
        end do        
        ab = fockspace(ne)%emin- ch*ne
        Z1 = Z1 + fockspace(ne)%Z0 * safedexp(-beta*(ab-enmin) )
        avn = avn + real(ne)*Z1/Z
     end do

   end subroutine get_ZFill

   !---------------------------------------------------------------------------!
   !  Modified Explicitly Restarted Lanczos (MERL) Method                      !
   !  fdim        => H-dimension                                               !
   !  emin,emax   => spectrum boundaries E_min, E_max                          !
   !  ibound      => 1,2 enforce (lower,upper) spectrum bound                  !
   !  save_vector => if .TRUE. save eigenvector                                !
   !  spectrum    => if .TRUE. iterate H with converged eigenstates removed    !
   !  etop        => top eigenvalue to move lower states in case of "spectrum" !
   !  na_unit     => unit to keep eigenstates                                  !
   !---------------------------------------------------------------------------!   
   subroutine Lanczos(fdim,emin,emax,ibound,spectrum,n_states,etop,na_unit,lz_unit)
     integer(kind=4), intent(in)  :: fdim                  ! Fock space (ne) dimension
     logical,         intent(in)  :: spectrum              ! Find a part of spectrum
     real(kind=8),    intent(out) :: emin,emax             ! H-spectrum boundaries
     real(kind=8),    intent(in)  :: etop                  ! Top energy to replace states
     integer(kind=4), intent(in)  :: ibound                ! Spectrum bound
     integer(kind=4), intent(in)  :: n_states              ! Number of states calculated
     integer(kind=4), intent(in)  :: na_unit               ! The unit to keep spectrum
     integer(kind=4), intent(in)  :: lz_unit               ! Lanczos output unit
     !-----------------------------------------------------!
     complex(kind=8), allocatable :: q(:)                  ! Initial vector
     complex(kind=8)              :: scal                  ! <i|j>
     logical                      :: orth_u                ! Lanczos orthogonalization flag
     logical                      :: stop_cycle            ! Exit flag
     real(kind=8),    parameter   :: eps_e = 1.d-15        ! Eigenvalue convergence parameter
     complex(kind=8), allocatable :: ht(:,:)               ! Subspace Hamiltonian 
     real(kind=8),    allocatable :: et(:)                 ! Subspace Spectrum
     integer(kind=4), parameter   :: N_L_min=3, N_L_max=10 ! 
     integer(kind=4)              :: il,jl,iv,nv,N_L1,NB   !
     integer(kind=4)              :: it,it_0,ii,jj,i       !  
     real(kind=8)                 :: ee,pi                 !
     real(kind=8),   parameter    :: zero=0.0_8            !
     
     call start_timer(T_LANCZOS)            
     pi=dacos(-1.0_8)
     N_L=N_L_min
     N_L1=N_L
     allocate(q(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"Q IN LANCZOS",+1)
     orth_u=.true.
     stop_cycle=.false.
     allocate(alp(n_L_max),stat=alloc_stat)
     call check_allocate(alloc_stat,"ALP IN LANCZOS",1)
     allocate(bet(N_L_max+1),stat=alloc_stat)
     call check_allocate(alloc_stat,"BET IN LANCZOS",1)
     allocate(ul(fdim,N_L_max+1),stat=alloc_stat)
     call check_allocate(alloc_stat,"UL IN LANCZOS",1)
     !                 ********* Generate initial random-vector *********                 !
     allocate(x(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"X IN LANCZOS",1)
     call random_seed()
     call random_number(x)
     x=2*pi*x-pi
     q(:) = cmplx(dcos(x(:)),dsin(x(:)),kind=8)/dsqrt(real(fdim,8)) 
     q(:)=q(:)/dsqrt(dreal(dot_product(q(:),q(:))))
     deallocate(x, stat=alloc_stat)
     call check_allocate(alloc_stat,"X IN LANCZOS",-1)
     it_0=0
     iter_0 : do
        it_0=it_0+1
           allocate(ht(N_L,N_L),stat=alloc_stat)
           call check_allocate(alloc_stat,"HT IN LANCZOS",1)
           allocate(et(N_L),stat=alloc_stat)
           call check_allocate(alloc_stat,"ET IN LANCZOS",1)
           if(it_0==1) then
              if(allocated(al)) then
                deallocate(al, stat=alloc_stat)
                call check_allocate(alloc_stat,"AL 2 IN LANCZOS",-1)
              end if
              allocate(al(N_L,N_L), stat=alloc_stat)
              call check_allocate(alloc_stat,"AL 1 IN LANCZOS",+1)
           end if        
        iter_1 : do it=1,2
           if(it_0==1.and.it==1) then ! Use random vector
              ul(:,1) = q(:)
           else ! Build Davidson's initial vector
              q = dcmplx(zero,zero)
              select case(ibound)
              case(1) ! lower bound
                 NB=1
              case(2) ! upper bound
                 NB=N_L1
              end select
              do i=1,N_L1
                 q(:) = q(:) + al(i,NB)*ul(:,i)
              end do
              q = q/dsqrt(real(dot_product(q,q)))
              ul(:,1)=q(:)
              if(it_0>1 .and. it==1) then
                 deallocate(al, stat=alloc_stat)
                 call check_allocate(alloc_stat,"AL 1 IN LANCZOS",-1)
                 allocate(al(N_L,N_L), stat=alloc_stat)
                 call check_allocate(alloc_stat,"AL 2 IN LANCZOS",+1)
                 N_L1=N_L
              end if                  
           end if
           alp=zero
           bet=zero
           bet(1)=zero
           do il=1,n_L
           ul(:,il+1)=dcmplx(zero,zero)
              do jj=1,fdim  !--> H|u> 
                 do ii=col_ptr(jj),col_ptr(jj+1)-1
                    ul(row_ind(ii),il+1) = ul(row_ind(ii),il+1) + HP(ii)*ul(jj,il) 
                 end do
              end do
              if(spectrum) then ! Remove all states already convergegd
                 rewind(na_unit)
                 do iv=1,n_states
                    read(na_unit) ee,vna
                    ul(:,il+1)=ul(:,il+1) + 2*(etop-ee)*vna*dot_product(vna,ul(:,il))
                 end do 
              end if                                       
              alp(il)    = dot_product(ul(:,il),ul(:,il+1))  
              ul(:,il+1) = ul(:,il+1) - alp(il)*ul(:,il)
              if(il>1) ul(:,il+1)=ul(:,il+1) - bet(il)*ul(:,il-1)
              bet(il+1) = dsqrt(dreal(dot_product(ul(:,il+1),ul(:,il+1))))
              ul(:,il+1) = ul(:,il+1)/bet(il+1)
              if(orth_u) then
                do iv=1,il
                   ul(:,il+1)=ul(:,il+1) - ul(:,iv)*dot_product(ul(:,iv),ul(:,il+1))
                end do
                ul(:,il+1)=ul(:,il+1)/dsqrt(dreal(dot_product(ul(:,il+1),ul(:,il+1))))
              end if ! orth
            end do ! il
               ht=dcmplx(zero,zero)
               et=zero
               ht(1,1)=alp(1)
               ht(1,2)=bet(2)
               do jl=2,N_L
                  ht(jl,jl-1)=bet(jl)
                  ht(jl,jl)=alp(jl)
                  if(jl<N_L) ht(jl,jl+1)=bet(jl+1)
               end do
               call cls_l_diag(ht,et,N_L,al)
               select case(ibound)
               case(1)
                 if(it>1) then
                   if(abs(emin-et(1))  <= eps_e) stop_cycle=.true.
                 end if
                 emin = et(1)
               case(2)
                 if(it>1) then
                   if(abs(emax-et(N_L))<= eps_e) stop_cycle=.true.
                 end if
                 emax = et(N_L)
               end select
               write(lz_unit,&
               "('NB nL=',2i4,3x,'Emin=',f14.10,5x,'Emax=',f14.10,i5)") ibound,N_L,emin,emax,n_states
               if(stop_cycle) then
                  deallocate(ht,stat=alloc_stat)
                  call check_allocate(alloc_stat,"HT IN LANCZOS",-1)
                  deallocate(et,stat=alloc_stat)
                  call check_allocate(alloc_stat,"HT IN LANCZOS",-1)
                  exit iter_0
               end if
         end do iter_1
         !            ********* Increase/Decrease Krylov's space *********            !
         N_L1=N_L 
         N_L=N_L+1
         if(N_L > N_L_max) N_L=N_L_min
         deallocate(ht,stat=alloc_stat)
         call check_allocate(alloc_stat,"HT IN LANCZOS",-1)
         deallocate(et,stat=alloc_stat)
         call check_allocate(alloc_stat,"HT IN LANCZOS",-1)          
      end do iter_0
      if(spectrum) then
         if(.NOT.allocated(vna)) then 
            allocate(vna(fdim),stat=alloc_stat)
            call check_allocate(alloc_stat,"VNA IN LANCZOS",+1) 
         end if
         vna=dcmplx(zero,zero)
         vna(:) = matmul(ul(:,1:N_L),al(1:N_L,1))        
         deallocate(al, stat=alloc_stat)
         call check_allocate(alloc_stat,"AL IN LANCZOS",-1)       
      end if 
      if(.NOT.spectrum .AND. ibound==2) &
         write(lz_unit,"(3x,'E_min=',f10.6/3x,'E_max=',f10.6)") emin,emax      
      deallocate(alp,stat=alloc_stat)
      call check_allocate(alloc_stat,"ALP IN LANCZOS",-1)
      deallocate(bet,stat=alloc_stat)
      call check_allocate(alloc_stat,"BET IN LANCZOS",-1)
      deallocate(q, stat=alloc_stat)
      call check_allocate(alloc_stat,"Q IN LANCZOS",-1)
      deallocate(ul, stat=alloc_stat)
      call check_allocate(alloc_stat,"UL IN LANCZOS",-1)
      call stop_timer(T_LANCZOS)      

   end subroutine Lanczos

   !------------------------------------------------------------------------!
   !  Finds max number of moments needed for Z0 calculations                !
   !  Exponentially scaled Bessel Functions are used                        !
   !------------------------------------------------------------------------!
   function max_mu_number(bb,eps_i)
     integer(kind=4)          :: max_mu_number
     real(kind=8), intent(in) :: bb,eps_i
     !----------------------------------!
     integer(kind=4)          :: m
     m=0
     do
       !if(dabs(bessi(m,bb)*dexp(-bb)) <= eps_i) exit
       if(dabs(bessi(m,bb)) <= eps_i) exit
       m=m+1
     end do
     if(mod(m,2)/=0) m=m+1 ! make it even
     max_mu_number=m
   end function max_mu_number

   !------------------------------------------------------------------------!
   !  Generating scalar moments for KPM method                              !                                    
   !------------------------------------------------------------------------!   
   subroutine moments(m_max,ne,fdim,nrv,K_type,N_exact)
     integer(kind=4), intent(in)  :: m_max,ne,N_exact
     integer(kind=4), intent(in)  :: K_type,fdim,nrv
     !---------------------------------------------!
     complex(kind=8), allocatable :: t0(:),t1(:),tr(:)
     complex(kind=8), parameter   :: czero=(0.0_8,0.0_8)
     complex(kind=8), parameter   :: cone =(1.0_8,0.0_8)
     real(kind=8)                 :: pi
     real(kind=8),    parameter   :: zero=0.0_8
     real(kind=8),    parameter   :: lamda=1.0d0        
     integer(kind=4)              :: r,m,ii,jj,iv
 
     call start_timer(T_MOMENTS)          
     allocate(mu(0:m_max), stat=alloc_stat)
     call check_allocate(alloc_stat,"MU IN MOMENTS",1)
     allocate(g(0:m_max), stat=alloc_stat)
     call check_allocate(alloc_stat,"G IN MOMENTS",1)
     allocate(t0(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"T0 IN MOMENTS",1)
     allocate(t1(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"T1 IN MOMENTS",1)
     allocate(tr(fdim), stat=alloc_stat)
     call check_allocate(alloc_stat,"TR IN MOMENTS",1)
     !         ********* Project out exact eigenstates from all random vectors *********         !
     do r=1,nrv
        do iv=1,N_exact
           ksi(:,r) = ksi(:,r) - fockspace(ne)%eigv(iv)%coor * &
                                 dot_product(fockspace(ne)%eigv(iv)%coor,ksi(:,r))
        end do
        ksi(:,r) = ksi(:,r)/dsqrt(real(dot_product(ksi(:,r),ksi(:,r))))
     end do
     mu=czero  
     pi=dacos(-1.0_8)
     do r=1,nrv   
        m_cycle : do m=0,m_max
           if(m==0) then
              t0(:)=ksi(:,r)
              tr(:)=t0(:)
           else if(m==1) then
              t1(:)=dcmplx(zero,zero)
              do jj=1,fdim  !--> H|t0> 
                 do ii=col_ptr(jj),col_ptr(jj+1)-1
                    t1(row_ind(ii)) = t1(row_ind(ii)) + HP(ii)*t0(jj) 
                 end do
              end do
              tr=t1
           else if(m>1) then
              tr(:)=dcmplx(zero,zero)
              do jj=1,fdim  !--> H|t1> 
                 do ii=col_ptr(jj),col_ptr(jj+1)-1
                    tr(row_ind(ii)) = tr(row_ind(ii)) + HP(ii)*t1(jj) 
                 end do
              end do             
              tr = 2*tr - t0
              t0=t1
              t1=tr
           end if
           mu(m) = mu(m) + dot_product(ksi(:,r),tr)
        end do m_cycle ! m 
     end do ! r     
     mu=mu/nrv
     do m=0,m_max ! < Kernel >
        select case(K_type)
        case(K_Jackson)
          g(m)= ((m_max+1-m)*dcos(pi*m/(m_max+1)) + &
                 dsin(pi*m/(m_max+1))/dtan(pi/(m_max+1)) )/dfloat(m_max+1) 
          !write(*,*) "M MOMENT  ", m, mu(m), g(m)        
       case(K_Lorentz)
          g(m) = dsinh(lamda*(1.0_8-dfloat(m)/m_max))/dsinh(lamda)
       end select
     end do
     deallocate(t0,stat=alloc_stat)
     call check_allocate(alloc_stat,"T0 IN MOMENTS",-1)
     deallocate(t1,stat=alloc_stat)
     call check_allocate(alloc_stat,"T1 IN MOMENTS",-1)
     deallocate(tr,stat=alloc_stat)
     call check_allocate(alloc_stat,"TR IN MOMENTS",-1)
     call stop_timer(T_MOMENTS)

   end subroutine moments  
 
   !------------------------------------------------------------------------!
   ! Performs specified operation on Kernel weights                         !
   !------------------------------------------------------------------------!      
   subroutine kernel(m_max,K_type,optype)  
     integer(kind=4),  intent(in) :: m_max,K_type
     character(len=*), intent(in) :: optype
     !------------------------------------!
     integer(kind=4)              :: m
     real(kind=8),    parameter   :: lamda=1.0d0 
     real(kind=8)                 :: pi
     
     select case(trim(adjustl(optype)))
     case("allocate")
       allocate(g2(0:m_max), stat=alloc_stat)
       call check_allocate(alloc_stat,"G IN MOMENTS_RR_SET",1)
       return
     case("generate")
       pi=dacos(-1.d0)
       do m=0,m_max ! < Kernel >
          select case(K_type)
          case(K_Jackson)
            g2(m)= ((m_max+1-m)*dcos(pi*m/(m_max+1)) + &
                   dsin(pi*m/(m_max+1))/dtan(pi/(m_max+1)) )/dfloat(m_max+1)       
          case(K_Lorentz)
            g2(m) = dsinh(lamda*(1.0_8-dfloat(m)/m_max))/dsinh(lamda)
          end select
       end do
     case("deallocate")
       deallocate(g2, stat=alloc_stat)
       call check_allocate(alloc_stat,"G IN MOMENTS_RR_SET",-1)     
       return       
     case default
        call FULLSTOP("KERNEL : UNKNOWN TYPE ",trim(adjustl(optype)))
     end select

   end subroutine kernel
 
   !------------------------------------------------------------------------!
   !  Calculation of Green's Function contribution of (gf_sign) on Real or  !
   !  Imaginary axes                                                        !                                    
   !------------------------------------------------------------------------! 
   subroutine moments_GF(Nd,Ns,m_max_GF,nrv,ne,omega,nw,chem,T_exp,gf_sign,&
                         Green1,Green2,MOM,DEN,nmom)
      integer(kind=4),  intent(in)    :: Nd,Ns,nw,gf_sign
      integer(kind=4),  intent(in)    :: m_max_GF,ne,nrv,nmom
      real(kind=8),     intent(in)    :: chem,T_exp
      complex(kind=8),  intent(in)    :: omega(nw)
      complex(kind=8),  intent(inout) :: Green1(Ns+Nd,Ns+Nd,nw)
      complex(kind=8),  intent(inout) :: Green2(Ns+Nd,Ns+Nd,nw)
      complex(kind=8),  intent(inout) :: MOM(Ns+Nd,Ns+Nd, -nmom:+nmom)
      complex(kind=8),  intent(inout) :: DEN(Ns+Nd,Ns+Nd, -nmom:+nmom)
      !-------------------------------------------------------!
      complex(kind=8),  allocatable   :: d_na(:,:),t0(:),t1(:),tr(:)
      complex(kind=8),  allocatable   :: Tw(:,:)
      complex(kind=8)                 :: mu_nu,cw,w
      complex(kind=8),  parameter     :: ic    = dcmplx(0.0_8,1.0_8)
      complex(kind=8),  parameter     :: czero = dcmplx(0.0_8,0.0_8)
      complex(kind=8),  parameter     :: cone  = dcmplx(1.0_8,0.0_8)
      real(kind=8),     allocatable   :: FMOM(:,:)
      real(kind=8),     parameter     :: zero=0.0_8
      real(kind=8),     pointer       :: apm,bpm,E_min,E_max
      real(kind=8)                    :: ww,dnu,t,pi,a,norm,norm2
      real(kind=4)                    :: half,psum  
      integer(kind=4),  parameter     :: d_create=1,d_destroy=-1
      integer(kind=4),  pointer       :: dim,dim1
      integer(kind=kind_phi), pointer :: phi(:),phi1(:)
      integer(kind=kind_phi)          :: d_phi
      integer(kind=4)                 :: i,j,ii,jj,r,iw,ne1,alloc_stat
      integer(kind=4)                 :: ms,ims,mss,ms1,ms2,nu,w1,w2,sgnw
      integer(kind=4)                 :: d_type,d_sign
      integer(kind=4)                 :: imom,k,p
      logical                         :: real_axes
      character(len=64)               :: char_bits

      call start_timer(T_GF)
      write(*,"(' MOMENTS_GF : N N+/-1 ',2i5)"),ne,ne+gf_sign 
      dim    => fockspace(ne)%dim
      phi    => fockspace(ne)%phi     
      ne1    =  ne+gf_sign
      dim1   => fockspace(ne1)%dim      
      phi1   => fockspace(ne1)%phi
      apm    => fockspace(ne1)%a
      bpm    => fockspace(ne1)%b  
      allocate(d_na(dim1,Nd+Ns),stat=alloc_stat)
      call check_allocate(alloc_stat,"D_NA IN MOMENTS_GF",+1)
      allocate(t0(dim1),stat=alloc_stat)
      call check_allocate(alloc_stat,"T0 IN MOMENTS GF",+1)
      allocate(t1(dim1),stat=alloc_stat)
      call check_allocate(alloc_stat,"T1 IN MOMENTS GF",+1)
      allocate(tr(dim1),stat=alloc_stat)
      call check_allocate(alloc_stat,"Tr IN MOMENTS GF",+1)
      E_min  => fockspace(ne1)%emin
      E_max  => fockspace(ne1)%emax
      !write(*,*) -bpm-gf_sign*(EL-apm)-chem, "< Omega < ",bpm-gf_sign*(EL-apm)-chem
      !             ********* Generate all polinomials needed on the omega-mesh *********             !
      allocate(Tw(nw,0:m_max_GF-1), stat=alloc_stat)
      call check_allocate(alloc_stat,"TW IN MOMENTS_GF",+1)
      Tw(:,:)=dcmplx(zero,zero)
      do iw=1,nw
         cw = (omega(iw) + chem + gf_sign*(EL - apm))/bpm
         sgnw=1
         if(dimag(cw)<0.d0) sgnw=-1
         do nu=0,m_max_GF-1
            dnu=2.0_8
            if(nu==0) dnu=1.0_8
            cw = dreal(cw) + ic*(dimag(cw)+1.d-30)
             Tw(iw,nu) = dnu*g2(nu)*cdexp(-ic*sgnw*nu*carccos(cw))/(ic*sgnw*cdsqrt(1.d0-cw*cw)) /bpm
             Tw(iw,nu)=Tw(iw,nu)*T_exp
         end do
      end do
      !             ********* Generate power-moment function *********             !
      allocate(FMOM(0:nmom,0:m_max_GF-1),stat=alloc_stat)
      call check_allocate(alloc_stat,"FMOM IN MOMENTS_GF",+1)
      a =-(chem + gf_sign*(EL - apm))
      do imom=0,nmom
      do nu=0,m_max_GF-1
            dnu=2.0_8
            if(nu==0) dnu=1.0_8
         FMOM(imom,nu)=zero
         do k=0,nmom
            psum=0
            do p=0,k/2
               half=1.d0
               if(2*(k/2)==k .and. k-2*p==0) half=0.5d0
               psum=psum + half*bincoef(k,p)*delta_k(k-2*p,nu)
            end do
            FMOM(imom,nu) = FMOM(imom,nu) + & 
                            bincoef(imom,k) * a**(imom-k) * (bpm/2.d0)**k * psum
         end do ! k
         FMOM(imom,nu)=g2(nu)*FMOM(imom,nu)*2*T_exp
      end do
      end do

      select case(gf_sign)
      case(-1) ! d
        d_type=d_destroy
      case(+1) ! d+
        d_type=d_create
      end select
      norm=0.0_8
      norm2=0.0_8
      pi=dacos(-1.d0)
      !               ********* d(+/-)_ms|na> *********               !      
      do ms=1,Nd+Ns
         ims=0
         mss=ms
         if(ms>Nd) then
            ims=Nd
            mss=mss-Nd
         end if
         d_na(:,ms)=dcmplx(zero,zero)
         do i=1,dim
            if(.NOT.allowed(ims,d_type,mss,Nd+Ns,phi(i),d_sign,d_phi)) cycle
            j=inv(d_phi,dim1,ne+gf_sign)
            if(j/=0) d_na(j,ms)=d_na(j,ms) + d_sign*vna(i)
         end do
         norm=norm + dot_product(d_na(:,ms),d_na(:,ms))
      end do
      !write(*,*) "SUM <d+-|I(nne)|d+->", norm, gf_sign
      norm=0.d0
      do ms1=1,Nd+Ns  
         do nu=0,m_max_gf-1
            dnu=2.0_8
            if(nu==0) dnu=1.0_8
            !---------- Calculate nu-th moment ----------!
            call start_timer(T_MS_MOMENT)
            if(nu==0) then
               t0(:)=d_na(:,ms1)
               tr(:)=t0(:)
            else if(nu==1) then
               t1(:)=dcmplx(zero,zero)
               do jj=1,dim1  !--> H|t0> 
                  do ii=col_ptr(jj),col_ptr(jj+1)-1
                     t1(row_ind(ii)) = t1(row_ind(ii)) + HP(ii)*t0(jj) 
                  end do
               end do
               tr(:)=t1(:)    
            else if(nu>1) then
               tr(:)=dcmplx(zero,zero)
               do jj=1,dim1  !--> H|t1> 
                  do ii=col_ptr(jj),col_ptr(jj+1)-1
                     tr(row_ind(ii)) = tr(row_ind(ii)) + HP(ii)*t1(jj) 
                  end do
               end do             
               tr = 2*tr - t0
               t0=t1
               t1=tr  !-> T_nu|d_na>  
            end if
            call stop_timer(T_MS_MOMENT)
            !--------------------------------------------! 
            do ms2=1,Nd+Ns
               mu_nu = dot_product(d_na(:,ms2),tr) 
               if(abs(mu_nu) > 1.d-12)then
               GF(1:nw,ms2,ms1) = GF(1:nw,ms2,ms1) + mu_nu*Tw(1:nw,nu)
               MOM(ms2,ms1,0:nmom)   = MOM(ms2,ms1,0:nmom)   + mu_nu * FMOM(0:nmom,nu)
               if(gf_sign==-1) then
                  DEN(ms2,ms1,0:nmom)   = DEN(ms2,ms1,0:nmom)   + mu_nu * FMOM(0:nmom,nu)
               end if
               end if
            end do ! ms2
         end do ! nu
      end do    ! ms1
      !-->GF = Green1 + ic*Green2
      do ms1=1,Nd+Ns
         do ms2=1,Nd+Ns
            Green1(ms1,ms2,:) = dreal(GF(:,ms1,ms2))
            Green2(ms1,ms2,:) = dimag(GF(:,ms1,ms2))
         end do
      end do
      deallocate(d_na,stat=alloc_stat)
      call check_allocate(alloc_stat,"D_NA IN MOMENTS_GF",-1)
      deallocate(t0,stat=alloc_stat)
      call check_allocate(alloc_stat,"T0 IN MOMENTS GF",-1)
      deallocate(t1,stat=alloc_stat)
      call check_allocate(alloc_stat,"T1 IN MOMENTS GF",-1)
      deallocate(tr,stat=alloc_stat)
      call check_allocate(alloc_stat,"Tr IN MOMENTS GF",-1)
      deallocate(Tw, stat=alloc_stat)
      call check_allocate(alloc_stat,"TW IN MOMENTS_GF",-1) 
      deallocate(FMOM,stat=alloc_stat)
      call check_allocate(alloc_stat,"FMOM IN MOMENTS_GF",-1)
      call stop_timer(T_GF)           
 
   end subroutine moments_GF

   !------------------------------------------------------------------------!
   !  Density Of States calculations                                        !
   !------------------------------------------------------------------------! 
   subroutine get_DOS(Nd,Ns,DOS,omega,nw,DOS_to_file)
      integer(kind=4), intent(in)    :: Nd,Ns,nw
      real(kind=8),    intent(inout) :: DOS(nw)
      complex(kind=8), intent(in)    :: omega(nw)
      logical,         intent(in)    :: DOS_to_file
      !-------------------------------------------!
      real(kind=8)                   :: pi
      !integer(kind=4)                :: free_io_unit
      integer(kind=4)                :: DOS_unit,iw,ms
      
      if(DOS_to_file) then
         DOS_unit = free_io_unit()
         fname=trim(adjustl(PROJECT))//"_DOS.out"
         open(DOS_unit, file=fname)
      end if
      pi=dacos(-1.d0)
      DOS=0.0_8
      do iw=1,nw
         do ms=1,Nd+Ns
            DOS(iw) = DOS(iw) + (-dimag(GF(iw,ms,ms))/pi)
         end do
         if(DOS_to_file) write(DOS_unit,"(f12.6,3x,f12.6)") dreal(omega(iw)),DOS(iw)
      end do
      if(DOS_to_file) close(DOS_unit)
   
   end subroutine get_DOS 

   !------------------------------------------------------------------------!
   !  Atomic self-energy calculations                                       !
   !------------------------------------------------------------------------! 
   subroutine get_sigma(Nd,Ns,Es,Ed,V,chem,omega,nw,MOM,Self_E,Self_Einf)
     integer(kind=4), intent(in)    :: Nd,Ns,nw
     real(kind=8),    intent(in)    :: chem
     complex(kind=8), intent(in)    :: omega(nw)
     complex(kind=8), intent(in)    :: MOM(Ns+Nd, Ns+Nd, -5:5)
     complex(kind=8), intent(in)    :: Es(Ns,Ns),Ed(Nd,Nd),V(Nd,Ns)     
     complex(kind=8), intent(inout) :: Self_E(Ns+Nd,Ns+Nd,nw)
     complex(kind=8), intent(inout) :: Self_Einf(Ns+Nd,Ns+Nd)           
     !------------------------------------------------------!
     complex(kind=8), allocatable   :: EF(:,:)
     real(kind=8),    allocatable   :: R(:)
     integer(kind=4)                :: N,ms1,ms2,iw
     integer(kind=4)                :: alloc_stat,ifail
  
     N=Nd+Ns
     allocate(R(N),stat=alloc_stat)
     call check_allocate(alloc_stat,"R IN GET_SIGMA",+1)
     allocate(EF(N,N),stat=alloc_stat)
     call check_allocate(alloc_stat,"EF IN GET_SIGMA",+1)
     do ms1=1,N
        do ms2=1,N
           do iw=1,nw
              Self_E(ms1,ms2,iw)=GF(iw,ms1,ms2)
           end do
        end do
     end do
     EF(1:Nd,1:Nd)     = Ed
     EF(Nd+1:N,Nd+1:N) = Es
     EF(1:Nd,Nd+1:N)   = V 
     EF(Nd+1:N,1:Nd)   = dconjg(transpose(V))
     do iw=1,nw
        call LIB_CINV(N, Self_E(1:N,1:N,iw), N, R, ifail)
        if(ifail /= 0) then
           write(*,*) "IFAIL=",IFAIL
           CALL FULLSTOP("STOP BY LIB_CINV")
        end if
        do ms1=1,N
           do ms2=1,N
              Self_E(ms1,ms2,iw) =-Self_E(ms1,ms2,iw) - EF(ms1,ms2) 
           end do
           Self_E(ms1,ms1,iw) = Self_E(ms1,ms1,iw) + omega(iw)+chem  
        end do
     end do
     Self_Einf = dcmplx(0.d0,0.d0)
     do ms1=1,N
        do ms2=1,N
           Self_Einf(ms1,ms2) = MOM(ms1,ms2,1) - EF(ms1,ms2)
        end do
        Self_Einf(ms1,ms1) = Self_Einf(ms1,ms1) + chem
     end do
     deallocate(R,stat=alloc_stat)
     call check_allocate(alloc_stat,"R IN GET_SIGMA",-1)
     deallocate(EF,stat=alloc_stat)
     call check_allocate(alloc_stat,"EF IN GET_SIGMA",-1)     
   
   end subroutine get_sigma

   !------------------------------------------------------------------------!
   !  Performs specified operation on random vectors set                    !
   !------------------------------------------------------------------------!    
   subroutine ksi_set(n,dim,NR,optype)
     integer(kind=4),  intent(in) :: n,dim,NR
     character(len=*), intent(in) :: optype
     !-------------------------------------!
     integer(kind=4)              :: ksi_unit,r,ir
     integer(kind=4)              :: ioerr
     real(kind=8),     parameter  :: zero=0.0_8     
     real(kind=8)                 :: pi
     character(len=4)             :: chn
     character(len=20)            :: fname
     
     select case(trim(adjustl(optype)))
     case("allocate")
       allocate(ksi(dim,NR), stat=alloc_stat)
       call check_allocate(alloc_stat,"KSI IN KSI_SET",1)
       return
     case("deallocate")
       deallocate(ksi, stat=alloc_stat)
       call check_allocate(alloc_stat,"KSI IN KSI_SET",-1)
       return
     case("generate")
       allocate(x(dim), stat=alloc_stat)
       call check_allocate(alloc_stat,"X IN KSI_SET",1)
       allocate(sr(nr), stat=alloc_stat)
       call check_allocate(alloc_stat,"SR IN KSI_SET",1)
       allocate(xr(nr), stat=alloc_stat)
       call check_allocate(alloc_stat,"XR IN KSI_SET",1)     
       call random_seed()
       call random_number(xr)
       sr(:)=dim*xr(:)
       pi=dacos(-1.0_8)
       do r=1,nr
          x(:)=zero
          call random_seed(sr(r))
          call random_number(x)
          x(:)=2*pi*x(:)-pi
          ksi(:,r) = dcmplx(dcos(x(:)),dsin(x(:)))
       end do
       deallocate(x, stat=alloc_stat)
       call check_allocate(alloc_stat,"X  IN KSI_SET",-1)
       deallocate(sr, stat=alloc_stat)
       call check_allocate(alloc_stat,"SR IN KSI_SET",-1)
       deallocate(xr, stat=alloc_stat)
       call check_allocate(alloc_stat,"XR IN KSI_SET",-1)
       return     
     end select

   end subroutine ksi_set
   
 end module mod_cls_L