module mod_cls_L_lib
!---------------------------------------------------------------------------!
! MODULE CONTAINS SUBROUTINES AND FUNCTIONS USED IN "MOD_CLS_L"             !
!---------------------------------------------------------------------------!
public 
integer(kind=4), parameter :: kind_phi=8 ! lenght of |phi> representation
contains
   !------------------------------------------------------------------------!   
   ! Just checking if allocation is succefull                               !
   !------------------------------------------------------------------------!   
   subroutine check_allocate(alloc_stat,string,itype)
     integer(kind=4),  intent(in) :: alloc_stat,itype
     character(len=*), intent(in) :: string
     !------------------------------------!
     character(len=130) :: message
     
     select case(itype)
      case(+1) ! allocation
        if(alloc_stat /= 0) then
          message="Allocation of "//trim(adjustl(string))//" failed"
        end if  
      case(-1) ! deallocation
          message="Deallocation of "//trim(adjustl(string))//" failed"
     end select
     if(alloc_stat /= 0)  CALL FULLSTOP(trim(message))          
   end subroutine check_allocate

   !------------------------------------------------------------------------!
   !  Check if "n" is even                                                  !
   !------------------------------------------------------------------------!   
   function even(n)
     integer(kind=4), intent(in) :: n
     logical                     :: even
     !---------------------------------!
     even = 2*(n/2) == n
   end function even

   !------------------------------------------------------------------------! 
   ! Binomial coefficient C^n_m = n!/m!(n-m)!                               ! 
   !------------------------------------------------------------------------!
   function bincoef(n,m)
      implicit none
      integer(kind=4), intent(in) :: n,m
      integer(kind=4)             :: bincoef
      !------------------------------------!
      real(kind=8)    :: cmm,binr
      integer(kind=4) :: k
  
      if(n<m) then
         binr=real(0,8)
      else if(n==m .OR. m==0) then
         binr=real(1,8)
      else if(m==1) then
         binr=real(n,8)
      else
         cmm=real(1,8)
         do k=m+1,n
            binr = real(k,8)/real(k-m,8)*cmm
            cmm = binr
         end do
      end if
      bincoef = nint(binr)   
   end function bincoef

   !------------------------------------------------------------------------! 
   !  Returns character string with a bit-representation of state "i"       !
   !  N - total number of cells in state "i"                                !
   !------------------------------------------------------------------------!
   function char_bits(N,i)
      implicit none
      character(len=64)           :: char_bits
      integer(kind=4), intent(in) :: N
      integer(kind=kind_phi), intent(in) ::i
      !--------------------------------!
      integer(kind=4)             :: k
      write(char_bits,"(64i1)") (ibits(i,k,1),k=N-1,0,-1)
   end function char_bits
   
   !------------------------------------------------------------------------!
   !  Returns spin of ms-cell                                               !
   !------------------------------------------------------------------------!
   function spin(ms,N)
     integer(kind=4)             :: spin
     integer(kind=4), intent(in) :: ms,n
     spin=1
     if(ms>N/2) spin=-1 
   end function spin
   
   !------------------------------------------------------------------------!
   ! Checks if m-th cell is occupied                                        !
   !------------------------------------------------------------------------!   
   function occupied(Nl,m,phi,N)
     logical                     :: occupied
     integer(kind=4), intent(in) :: Nl,m,N
     integer(kind=kind_phi), intent(in) :: phi
     !---------------------------------------!
     occupied = btest(phi,N-Nl-m)
   end function occupied
   
   !------------------------------------------------------------------------!
   !  Returns the total number of occupied cells in N-bit state "i"         !
   !------------------------------------------------------------------------!
   function occupation(N,i)
     implicit none
     integer(kind=4), intent(in) :: N
     integer(kind=kind_phi), intent(in) :: i
     integer(kind=4)             :: occupation
     !---------------------------------------!
     integer(kind=4) :: k
     occupation =sum( (/(ibits(i,k,1), k=N-1,0,-1) /) )
   end function occupation
   
   !------------------------------------------------------------------------!
   ! Makes state |IdIs>=|Id>|Is>, dim(Is)=Ns                                !
   !------------------------------------------------------------------------!
   function join(Id,Ns,Is)
     implicit none
     integer(kind=kind_phi)             :: join
     integer(kind=4),        intent(in) :: Ns
     integer(kind=kind_phi), intent(in) :: Id,Is
     !-------------------------------------!
     join = Is + 2**Ns*Id
   end function join
   
   !------------------------------------------------------------------------!
   !  Trying to apply create [d+(m)] or destroy [d(m)] operators to         !
   !  to <right>-part of state |T>=|(1)...(Nl)...(N)> In case operation     !
   !  succesfull, allowed=.TRUE. and the new state  dsign*|new_phi>         !
   !  is  returned;                                                         !
   !  d_type = +1/-1                                                        !
   !  Nl=0 in case of apllying d's to the leftmost part of |T>              !
   !------------------------------------------------------------------------!   
   function allowed(Nl,d_type,m,N,phi,dsign,new_phi) 
     implicit none
     logical                       :: allowed
     integer(kind=kind_phi), intent(in)  :: phi
     integer(kind=kind_phi), intent(out) :: new_phi
     integer(kind=4),  intent(in)  :: Nl,N,m
     integer(kind=4),  intent(in)  :: d_type
     integer(kind=4),  intent(out) :: dsign
     !--------------------------------------------!
     integer(kind=4)               :: k,NNL
     
     allowed=.false.
     NNl=N-Nl
     select case(d_type)
       case(1) ! create state
         if(.NOT.btest(phi,NNl-m)) then 
            allowed = .true.
            dsign   = (-1)**sum( (/(ibits(phi,k,1), k=N-1,NNl-m+1,-1) /) ) 
            new_phi = ibset(phi,NNl-m)
         end if
         return
       case(-1)  ! destroy state
         if(btest(phi,NNl-m)) then
            allowed = .true.
            dsign   = (-1)**sum( (/(ibits(phi,k,1), k=N-1,NNl-m+1,-1) /) )
            new_phi = ibclr(phi,NNl-m)
         end if
         return
     end select
     write(*,"(/10x,'Unknown operator type : '/)") d_type
   
   end function allowed

   !------------------------------------------------------------------------!
   !  Delta_symbol                                                          !
   !------------------------------------------------------------------------!   
   function delta_k(A,B)
     implicit none
     integer(kind=4)             :: delta_k
     integer(kind=4), intent(in) :: A,B
     !----------------------------------!
     delta_k=1
     if(A/=B) delta_k=0
   end function delta_k   

   !------------------------------------------------------------------------!
   ! Calculation of Chebyshev order-n polinomial of "knd"-kind              !
   !------------------------------------------------------------------------!
   function t(n,x,knd)
      implicit none
      complex(kind=8), intent(in) :: x
      real(kind=8)                :: t
      integer(kind=4), intent(in) :: n,knd
      !------------------------------!
      integer(kind=4)             :: i
      real(kind=8)                :: t0,t1
         
      if(n==-1) then
         t=0.0d0
         return
      end if
      t0=1.0_8      
      if(n==0) then 
         t=t0
         return
      end if
      if(knd==1) t1=x
      if(knd==2) t1=2.d0*x
      if(n==1) then
         t=t1 
         return
      end if
      do i=2,n
        t=2*x*t1 - t0
        t0=t1
        t1=t
      end do
     
   end function t      

   !------------------------------------------------------------------------!
   ! Arccos of complex variable                                             !
   !------------------------------------------------------------------------!   
   function carccos(z)
     implicit none
     complex(kind=8)             :: carccos
     complex(kind=8), intent(in) :: z
     !------------------------------!
     complex(kind=8), parameter :: ic=dcmplx(0.d0,1.d0)
     
     carccos = ic*cdlog(z-ic*cdsqrt(1.d0-z*z))
   
   end function carccos

   !------------------------------------------------------------------------!
   !   This subroutine calculates the first kind modified bessel function   !
   !   of integer order n, for any real x. we use here the classical        !
   !   recursion formula, when x > n. for x < n, the miller's algorithm     !
   !   is used to avoid overflows.                                          !
   !   reference:                                                           !
   !   c.w.clenshaw, chebyshev series for mathematical functions,           !
   !   mathematical tables, vol.5, 1962.                                    !
   !   WARNING !!! Bessel Fucntions have been scaled as                     !
   !               I(m,x) -> I(m,x)*exp(-x)                                 !
   !------------------------------------------------------------------------! 
      function bessi(n,x)

      parameter (iacc = 40,bigno = 1.d10, bigni = 1.d-10)
      real *8 x,bessi,bessi0,bessi1,tox,bim,bi,bip
      if (n.eq.0) then
      bessi = bessi0(x)
      return
      endif
      if (n.eq.1) then
      bessi = bessi1(x)
      return
      endif
      if(x.eq.0.d0) then
      bessi=0.d0
      return
      endif
      tox = 2.d0/x
      bip = 0.d0
      bi  = 1.d0
      bessi = 0.d0
      m = 2*((n+int(dsqrt(dfloat(iacc*n)))))
      do 12 j = m,1,-1
      bim = bip+dfloat(j)*tox*bi
      bip = bi
      bi  = bim
      if (dabs(bi).gt.bigno) then
      bi  = bi*bigni
      bip = bip*bigni
      bessi = bessi*bigni
      endif
      if (j.eq.n) bessi = bip
   12 continue
      bessi = bessi*bessi0(x)/bi
      return
      end function bessi
! ----------------------------------------------------------------------
! auxiliary bessel functions for n=0, n=1
      function bessi0(x)
      real *8 x,bessi0,y,p1,p2,p3,p4,p5,p6,p7,  &
      q1,q2,q3,q4,q5,q6,q7,q8,q9,ax,bx
      data p1,p2,p3,p4,p5,p6,p7/1.d0,3.5156229d0,3.0899424d0,1.2067429d0,  &
      0.2659732d0,0.360768d-1,0.45813d-2/
      data q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1, &
      0.225319d-2,-0.157565d-2,0.916281d-2,-0.2057706d-1,  &
      0.2635537d-1,-0.1647633d-1,0.392377d-2/
      if(dabs(x).lt.3.75d0) then
      y=(x/3.75d0)**2
      bessi0=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
      bessi0=bessi0*dexp(-dabs(x))
      else
      ax=dabs(x)
      y=3.75d0/ax
      bx=1.d0/dsqrt(ax)   !dexp(ax)/dsqrt(ax)
      ax=q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*(q7+y*(q8+y*q9)))))))
      bessi0=ax*bx
      endif
      return
      end function bessi0
! ----------------------------------------------------------------------
      function bessi1(x)
      real *8 x,bessi1,y,p1,p2,p3,p4,p5,p6,p7,  &
      q1,q2,q3,q4,q5,q6,q7,q8,q9,ax,bx
      data p1,p2,p3,p4,p5,p6,p7/0.5d0,0.87890594d0,0.51498869d0,  &
      0.15084934d0,0.2658733d-1,0.301532d-2,0.32411d-3/
      data q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,-0.3988024d-1, &
      -0.362018d-2,0.163801d-2,-0.1031555d-1,0.2282967d-1, &
      -0.2895312d-1,0.1787654d-1,-0.420059d-2/
      if(dabs(x).lt.3.75d0) then
      y=(x/3.75d0)**2
      bessi1=x*(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7))))))
      bessi1=bessi1*dexp(-dabs(x))
      else
      ax=dabs(x)
      y=3.75d0/ax
      bx=1.d0/dsqrt(ax)   !dexp(ax)/dsqrt(ax)
      ax=q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*(q7+y*(q8+y*q9)))))))
      bessi1=ax*bx
      endif
      return
      end function bessi1
! ----------------------------------------------------------------------

	REAL*8 FUNCTION SAFEDEXP(POW)
!*******************************************************
!*  Safely evaluates expontent e(pow)                  *
!*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA POWMAX/1021/
	IF(POW.GT.POWMAX)THEN
	SAFEDEXP=DEXP(POWMAX)	
	ELSEIF(POW.LT.-POWMAX)THEN
	SAFEDEXP=DEXP(-POWMAX)
	ELSE
	SAFEDEXP=DEXP(POW)
	ENDIF
	END FUNCTION SAFEDEXP

   !------------------------------------------------------------------------!
   !                     Returns free IO-chanel(unit)                       !   
   !------------------------------------------------------------------------!
      function free_io_unit()
      integer(kind=4) :: free_io_unit
      !-----------------------------!
      integer(kind=4)            :: iunit
      integer(kind=4), parameter :: N_max_IO=100 
      logical                    :: unit_busy

      do iunit=6,N_max_IO
         inquire(unit=iunit, opened=unit_busy)
         if(.not.unit_busy) then
            free_io_unit = iunit
            exit
         end if
      end do

      return
      end function free_io_unit
   !------------------------------------------------------------------------!
   !  "print_io_units" prints out the list of units used and files          !
   !------------------------------------------------------------------------!
      subroutine print_io_units()
        integer(kind=4), parameter :: N_max_IO=100
        integer(kind=4)            :: iunit
        logical                    :: unit_busy
        character(len=150)         :: file_name

        write(*,"(/10x,'************ IO_units - Files ************')")
        do iunit = 6,N_max_IO
           inquire(unit=iunit, opened=unit_busy, name=file_name)
           if(unit_busy) then
              write(*,"(10x,'Unit = ',i2,3x,a)") iunit, trim(file_name)
           end if
        end do
        write(*,"(10x,'******************************************'/)")

        return
      end subroutine print_io_units  

   !------------------------------------------------------------------------!
   !  Printing out memory (to be) allocated                                 !
   !------------------------------------------------------------------------!
      subroutine print_memory(unit,array,bytes)
        integer(kind=4),  intent(in) :: unit
        real(kind=8),     intent(in) :: bytes
        character(len=*), intent(in) :: array
        !----------------------------------!
        real(kind=8)                 :: memory
        character(len=50)            :: chmem
        
        memory=bytes/1000000_8
        write(chmem,"(f12.3)") memory
        write(unit,*) "MEMORY REQUIRED TO SAVE "//&
                      &trim(adjustl(array))     //&
                      &"  "                     //&
                      &trim(adjustl(chmem))     //&
                      &" Mb"
        return      
      end subroutine print_memory
      
end module mod_cls_L_lib      