 !-----------------------------------------------------------------------------!
 !  Module contains timing subroutines and functions                           !
 !-----------------------------------------------------------------------------!
    module mod_cls_L_time
    use mod_cls_L_lib
	  implicit none
	  private

	  type t_timer
		integer(kind=4) :: nexec
	    real(kind=4)    :: start_usr
	    real(kind=4)    :: usr(2)
	    logical         :: started
	    logical         :: used
	  end type t_timer   

      integer(kind=4), parameter :: t_current=1, t_total=2
	  integer(kind=4), parameter :: T_CLS_MAIN  =  1, &    ! Total program time;
                                    T_H         =  2, &    ! Hamiltonian time;
                                    T_LANCZOS   =  3, &    ! Lanczos time;
                                    T_MOMENTS   =  4, &    ! Scalar moments time;
                                    T_GF        =  5, &    ! Green's Function time;
                                    T_MS_MOMENT =  6, &
                                    T_ANG       =  7       ! Force calculations;
	  integer(kind=4),  parameter :: N_max_timers = T_ANG ! Largest above
	  logical                     :: timers_initialized=.false.
	  type(t_timer), allocatable  :: timer(:)
      public  init_timers, start_timer, stop_timer,print_timer, print_time
	  public  T_CLS_MAIN, T_H, T_LANCZOS, T_MOMENTS, T_GF, T_MS_MOMENT, T_ANG
	  contains
      !-----------------------------------------------------------------------!
	  ! init_timers : timers initialization subroutine                        !
      !-----------------------------------------------------------------------!
      subroutine init_timers()
	    integer(kind=4) :: alloc_stat
        !---------------------------!
         
        if(.NOT.timers_initialized) then
           allocate(timer(N_max_timers), stat=alloc_stat)
		   call check_allocate(alloc_stat,"timer in init_timer",1)
		   timers_initialized=.true.
		end if
        timer(:)%start_usr      = 0.0
        timer(:)%usr(t_current) = 0.0
        timer(:)%usr(t_total)   = 0.0
        timer(:)%started        = .false.
        timer(:)%used           = .false.
        timer(:)%nexec          = 0
      
      end subroutine init_timers 

      subroutine start_timer(ID)
	    integer(kind=4), intent(in) :: ID
	    !-------------------------------!
        real(kind=4)                 :: t_cpu
        character(len=4)             :: CHT

        if(ID .gt. N_max_timers) then
           call FULLSTOP("Start_timer : Unknown timer")
        end if
        if(timer(ID)%started) then
           write(CHT,"(i4)") ID
           call FULLSTOP("Start timer: timer "//trim(adjustl(CHT))//" is already running")
        end if
        call cpu_time(t_cpu)
        timer(ID)%start_usr = t_cpu
        timer(ID)%nexec     = timer(ID).nexec + 1
        timer(ID)%started   = .true.
        timer(ID)%used      = .true.
      end subroutine start_timer    

      subroutine stop_timer(ID)
	    integer(kind=4), intent(in) :: ID
	    !-------------------------------!
        real(kind=4) :: t_cpu
 
        call cpu_time(t_cpu)
        timer(ID)%usr(t_current) = t_cpu - timer(ID)%start_usr
        timer(ID)%usr(t_total)   = timer(ID)%usr(t_total)  + &
                                   timer(ID)%usr(t_current)
        timer(ID)%started        = .false.
        end subroutine stop_timer

      subroutine print_time(iounit)
        integer(kind=4), intent(in) :: iounit    
        !-----------------------------------!
        
        write(iounit,"(/5x,23(1h=),' Time ',24(1h=)/)") 
        call print_timer(T_CLS_MAIN, iounit, "CLS_L_MAIN",        t_total)
        call print_timer(T_H,        iounit, "Hamiltonian",       t_total)
        call print_timer(T_LANCZOS,  iounit, "Lanczos",           t_total)
        call print_timer(T_MOMENTS,  iounit, "Scalar moments",    t_total)
        call print_timer(T_MS_MOMENT,iounit, "MS-moments",        t_total)        
        call print_timer(T_GF,       iounit, "Greens's Function", t_total)
        call print_timer(T_ANG,      iounit, "Angular Momentum",  t_total)
        write(iounit,"(/5x,53(1h=)/)")
      end subroutine print_time

      subroutine print_timer(ID, iounit, comment, ttype)
        integer(kind=4), intent(in) :: ID, iounit,ttype
        character(len=*), intent(in) :: comment
	    !-------------------------------------!
        character(len=8)             :: usr,sys
        character(len=100)           :: form
		character(len=7)             :: tstate
 
        if(.not.timer(ID)%used) return
		tstate=" "
        if(timer(ID)%started) tstate="Running" 
        call trans(timer(ID)%usr(ttype),usr) 
        form="(5x,a,t25,':  ',a8, 5x,'[ ',i5,' ]',3x,a)"
        if(iounit==0) then
           write(*,trim(form)) trim(adjustl(comment)),usr,timer(ID)%nexec,tstate
        else
           write(iounit,trim(form)) trim(comment),usr,timer(ID)%nexec,tstate
        end if
      end subroutine print_timer

      subroutine trans(T,hms)
        real(kind=4),     intent(in)    :: T
        character(len=8), intent(inout) :: hms
	    !----------------------------------!
        integer(kind=4)              :: i

        write(hms,"(i2,'.',i2,'.',i2)") int(T/3600),                   &
                                        int(mod(T,3600.0)/60),         & 
                                        int(mod(mod(T,3600.0),60.0))       
        do i=1,len_trim(hms)
           if(hms(i:i).eq.' ') hms(i:i) = '0'
        end do
     
      end subroutine trans

	end module mod_cls_L_time