! QMC based on Hirsch-Fye algorithm, input 1/G0(iw_n), output Sf(iw_n)
! cubic spline for fourier transformation, moments calculated from 1/G0
! Yi-feng Yang, May 2007, Davis
        
        subroutine qmc_run(nts,ndeg,nfs,nwu,nmc,nsweep,sym,u,beta,gw0,sf,gw,dens,mom1,mom2,Ldiag)
        implicit none

! input data:
! nts: integer, number of time slices
! ndeg: integer, orbital degeneracy
! nfs: integer, number of Matsubara frequency
! nwu: integer, number of qmc sweep for warm up
! nmc: integer, number of qmc sweep for sampling
! nsweep: integer, number of qmc sweep per sampling
! sym(ndeg): integer, symmetry matrix
! u(ndeg,ndeg): real, Coulomb interation
! beta: real, inverse temperature
! gw0(nfs,ndeg,ndeg): complex, initial Weiss field Gw0

! output data:
! sf(nfs,ndeg,ndeg): complex, selfenergy
! gw(nfs,ndeg,ndeg): complex, Green's function
! dens(ndeg,ndeg): real, density matrix
! mom1(ndeg): real, large frequency limit of self energy
! mom2(ndeg): real, first moment of self energy
! Ldiag: logical, matrix diagonal or nondiagonal

        integer nts,ndeg,nfs,nwu,nmc,nsweep,sym(ndeg)
        real*8 beta,u(ndeg,ndeg),dens(ndeg,ndeg)
        complex*16 gw0(nfs,ndeg,ndeg),gw(nfs,ndeg,ndeg),sf(nfs,ndeg,ndeg)
        logical Ldiag
             
        integer ndnt,nisC,nconfC,i,j,im,l1,l2,k,im1,im2,info,dseed
        real*8 sgn,tmp,pi,dt,x,y,mom1(ndeg),mom2(ndeg)
        complex*16 ii
        parameter (ii=cmplx(0.d0,1.d0))
        data dseed/123456789/
        
        real*8, allocatable :: gt(:,:),gt0(:,:),w(:),rn(:),nn(:,:),aa(:,:),ipiv(:)
        complex*16, allocatable :: gwinv(:,:,:),gg(:,:,:),a(:),b(:),c(:,:),c0(:,:)
        
        allocate(gt(ndeg*nts,ndeg*nts),gt0(ndeg*nts,ndeg*nts),w(nfs))
        allocate(rn(ndeg),nn(ndeg,ndeg),aa(nts,nts),ipiv(ndeg))
        allocate(gwinv(nfs,ndeg,ndeg),gg(nts,ndeg,ndeg))
        allocate(a(nfs),b(nts),c(ndeg,ndeg),c0(ndeg,ndeg))
        
        pi=acos(-1.d0)
        nisC=ndeg*(ndeg-1)/2
        nconfC=2**nisC
        ndnt=nts*ndeg

! Matsubara frequency
        
        do i=1,nfs
            w(i)=pi*(2*i-nfs-1)/beta
        end do 

! diagonal or non-diagonal

        if(.not.Ldiag) write(*,*)' input not diagonal, program to be tested'

! Force symmetry

        call lib_forcesym3l(gw0,sym,ndeg,nfs,1)        

! inverse Weiss field
        
        if(Ldiag) then
            gwinv(:,:,:)=0.d0
            do im=1,ndeg
                gwinv(:,im,im)=1.d0/gw0(:,im,im)
            end do
        else 
            do i=1,nfs
                c(:,:)=gw0(i,:,:)
                call LIB_CINV(ndeg,c,ndeg,ipiv,info) 
                gwinv(i,:,:)=c(:,:)
            end do 
        end if

! Fourier transformation of Weiss field

        gg(:,:,:)=0.d0
        if(Ldiag) then
            do im=1,ndeg
                a(:)=gw0(:,im,im)
                info=1
                call qmc_w2t(nts,nfs,beta,a,b,info) 
                gg(:,im,im)=b(:) 
            end do 
         else
            do im1=1,ndeg
                do im2=1,ndeg
                    a(:)=gw0(:,im1,im2)
                    info=0
                    if(im1.eq.im2) info=1
                    call qmc_w2t(nts,nfs,beta,a,b,info)
                    gg(:,im1,im2)=b(:)
                end do
            end do
        end if

! G(tau) --> G(i,j)
        
        gt0(:,:)=0.d0
        do i=1,nts
            do j=1,nts
                l1=nts-j+i+1
                if(i.ge.j) l1=i-j+1
                do im1=1,ndeg
                    do im2=1,ndeg  
                        gt0(ndeg*(i-1)+im1,ndeg*(j-1)+im2)=ISIGN(1,i-j)*gg(l1,im1,im2)
                    end do
                end do
            end do
        end do   

!  QMC sampling

        call qmc_sampling(nts,ndeg,ndnt,nwu,nmc,nsweep,dseed,beta,u,gt0,sgn,gt,dens,nn)

! writeout occupation number

        x=0.d0
        do i=1,ndeg
            rn(i)=dens(i,i)
            write(*,*)'orbital',i,':', rn(i)
            x=x+rn(i) 
        end do
        write(*,*)'total occupation  :',x

        
! G(i,j)--->G(tau)

       gg(:,:,:)=0.d0
       do im1=1,ndeg
           do im2=1,ndeg 
                do j=1,nts
                    do i=1,j-1
                        l1 = j-i+1
                        l2 = nts-j+i+1
                        gg(l2,im1,im2)=gg(l2,im1,im2)-gt(ndeg*(i-1)+im1,ndeg*(j-1)+im2)
                        gg(l1,im1,im2)=gg(l1,im1,im2)+gt(ndeg*(j-1)+im1,ndeg*(i-1)+im2)
                    end do
                    gg(1,im1,im2)=gg(1,im1,im2)+gt(ndeg*(j-1)+im1,ndeg*(j-1)+im2)
                 end do
             end do
        end do
        gg(:,:,:)=gg(:,:,:)/dble(nts)

! Force symmetry

       call lib_forcesymr1(rn,sym,ndeg)
       call lib_forcesymr2(nn,sym,ndeg)
       call lib_forcesymr2(dens,sym,ndeg)
       call lib_forcesym3l(gg,sym,ndeg,nts,1)        
       
! Fourier transformation G(tau)---> G(iw)

        if(Ldiag) then
            gw(:,:,:)=0.d0
            do im=1,ndeg
                b(:)=gg(:,im,im)
                a(:)=gwinv(:,im,im)
                info=1 
                call qmc_getmoment(ndeg,nfs,a,u,rn,nn,w,mom1(im),mom2(im),im)
                call qmc_t2w(nts,nfs,beta,b,a,info,mom1(im),mom2(im))
                gw(:,im,im)=a(:)
            end do
        else                                              
            do im1=1,ndeg
                do im2=1,ndeg
                    a(:)=gwinv(:,im1,im2)
                    b(:)=gg(:,im1,im2)    
                    if(im1.eq.im2) then 
                        call qmc_getmoment(ndeg,nfs,a,u,rn,nn,w,mom1(im1),mom2(im1),im1)
                    else
                        call qmc_t2w(nts,nfs,beta,b,a,0,0.d0,0.d0)
                    end if 
                    gw(:,im1,im2)=a(:)
                end do
            end do     
        end if
       call lib_forcesymr1(mom1,sym,ndeg)
       call lib_forcesymr1(mom2,sym,ndeg) 
       call lib_forcesym3l(gw,sym,ndeg,nfs,1)         
       
 ! calculating self energy
 
        sf(:,:,:)=0.d0
        do i=1,nfs
            if(Ldiag) then
                do im=1,ndeg
                    sf(i,im,im)=gwinv(i,im,im)-1.d0/gw(i,im,im)
                end do
            else 
                c(:,:)=gw(i,:,:)
                call LIB_CINV(ndeg,c,ndeg,ipiv,info) 
                sf(i,:,:)=gwinv(i,:,:)-c(:,:)
            end if 
        end do 

! calculating moments for the self energy

        mom1(:)=0.d0
        mom2(:)=0.d0
        do im=1,ndeg
            do im1=1,ndeg
                mom1(im)=mom1(im)+u(im,im1)*(rn(im1)-0.5d0)
                do im2=1,ndeg
                    mom2(im)=mom2(im)+u(im,im1)*u(im,im2)*(nn(im1,im2)-rn(im1)*rn(im2))
                end do  
            end do
        end do

! delocate

        deallocate(gt,gt0,w,rn,nn,aa,ipiv)
        deallocate(gwinv,gg,a,b,c,c0)
        
    end subroutine qmc_run
