#!/usr/bin/env python
# Author: Kristjan Haule, March 2007

import os,sys,errno
ROOT = os.environ.get('LMTO_DMFT_ROOT')
if ROOT is not None:
    sys.path.append( ROOT )
    sys.path.append( ROOT + '/fort')
else:
    print >> sys.stderr, "Environment variable LMTO_DMFT_ROOT must be set!"
    print "Environment variable LMTO_DMFT_ROOT must be set!"
    sys.exit(1)

#from ld_base import *
from ldap import *

# specific imports to this module
import shutil


if __name__ == '__main__':
    # -------------- Parameters which are set through input files sparams.dat and params.dat -----------
    # These parameters will be set if found in sparams.dat file
    sparams = {'inpdir':'"."', 
               'scratch':False,
               'cix':None,
               'start':0} 
    # These parameters will be set if found in params.dat file
    params = {'U':          0.0,
              'J':          0.0,
              'Niter':      1,
              'finish':     100,
              'admixDM':    0.5,
              'Dinterval':  10.,
              'Ninter':     1,
              'admix_large':0.3,
              'broyden':    True,
              'admix0':     0.3,
              'admix_rho':  1.,
              'admix_mag':  1.,
              'restBrdn':   15,
              'cmpDOS':     False,
              'om_a':       -4,
              'om_b':       4,
              'Nom':        200,
              'small_dmu':  1e-5,
              'mix_mu':     1.,
              'mix_ham':    1.,
              'admix_frozen': 1.,
              'test_complex_eigenvalues' : False,
              'a' : 0.0}

    p = Params('sparams.dat', sparams, 'params.dat', params)
    p.refresh()

    fh_info = open(os.path.join(p.inpdir, 'phi.info'), 'w')
    print >> fh_info, '-'*80
    print >> fh_info, '*'*20, 'Iteration', 1, '*'*20
    print >> fh_info, '-'*80, '\n'

    # Prepares for lda execution
    ldaexe = LDApU(p.inpdir, fh_info, p.scratch)


    # Runs the starting lda
    ldaexe.RunFirstLDA()
    
    # allocated k-sum memory and reads the lmtart data
    lmt = Lmtart(os.path.join(p.inpdir, ldaexe.ksum), fh_info) #, embed='Savrasov')
    lmt.check_cix(p.cix)
    lmt.read_cix()
    
    lmt_corr_index = lmt.Correlated_LS_index()


    # Checks if there is an old density matrix available
    (StartExist, mocc) = ReadMocc(os.path.join(p.inpdir,"mocc.out"), lmt.ndim)
    
    if (StartExist): # Starts from the old run
        # Construct Sigma from occupancy
        Sigma =  ldau.hartree(p.U, p.J, mocc, lmt.bndind, lmt_corr_index, p.a)

        WriteMatrix('occup.dat', mocc)
        WriteMatrix('sigm.dat', Sigma)

        # to cubic
        la.ztransform(Sigma, lmt.T2C,'N') # From spheric -> cubic
    else:
        # Otherwise first step is LDA-like
        Sigma = zeros((lmt.ndim, lmt.ndim), dtype=complex, order='F')

    # Computes new chemical potential
    mu = lmt.cmp_chem_potential_LDAU(lmt.mu, p.Dinterval, Sigma, p.test_complex_eigenvalues)
    #mu=lmt.mu

    # Computes new density matrix coefficients
    nxx = lmt.DensityMatrixCoeff(mu, Sigma)
    # new density matrix
    mocc_new = crho.cmp_occupancy(nxx[0], nxx[1], nxx[2], nxx[3], lmt.moo, lmt.bndind, lmt.isrt, lmt.llmtind)

    if (StartExist): 
        # mixing of density matrix
        mocc = array(matrix(mocc)*(1-p.admixDM) + matrix(mocc_new)*p.admixDM, order='F')
    else:
        mocc = mocc_new


    WriteMatrix('1occup.dat', mocc)
    # correlated occupancy
    nf = lmt.N_correlated(mocc)
    
    for itt in range(p.start,p.finish):
        print '-'*10, 'iteration #', itt, ' ', '-'*10
        
        p.refresh()
        
        for i in range(p.Niter):

            # Sigma is here in spheric's
            Sigma =  ldau.hartree(p.U, p.J, mocc, lmt.bndind, lmt_corr_index, p.a)
            WriteMatrix('1sigm.dat', Sigma)
            # to cubic
            la.ztransform(Sigma, lmt.T2C,'N') # From spheric -> cubic

            # Computes new chemical potential
            new_mu = lmt.cmp_chem_potential_LDAU(mu, p.Dinterval, Sigma, p.test_complex_eigenvalues)
            
            if (abs(new_mu-mu)<p.small_dmu): break
            mu = mu*(1-p.mix_mu) + new_mu*p.mix_mu
            
            # Computes new density matrix coefficients
            nxx = lmt.DensityMatrixCoeff(mu, Sigma)

            # new density matrix
            mocc_new = crho.cmp_occupancy(nxx[0], nxx[1], nxx[2], nxx[3], lmt.moo, lmt.bndind, lmt.isrt, lmt.llmtind)

            WriteMatrix('1occup.dat', mocc_new)
            
            # mixing of density matrix
            mocc = array(matrix(mocc)*(1-p.admixDM) + matrix(mocc_new)*p.admixDM, order='F')

            # correlated occupancy
            nf = lmt.N_correlated(mocc)

        # Print occupancy in case we need to restart
        WriteMocc(os.path.join(p.inpdir,"ksum/mocc.out"), mocc)

        # New density is computed
        nrho = lmt.recompute_density(nxx)

        # Reading old scs file
        scs_f = LmtScsf()
        (rho_s, rho_cor_s, pot_frz_s, efmu_s, Enu_s) = scs_f.read(lmt.project_name+'.scs')

        # Computes difference between steps
        diff = lmt.RhoDiff(rho_s, nrho)

        if (diff[0]<1e-5 and diff[1]<1e-5): break
        
        # Computing energy on output density
        scs_f.write(lmt.project_name+'.scse', nrho, rho_cor_s, pot_frz_s, mu, Enu_s)
        Enes = ldaexe.RunLDAforEnergy(lmt.project_name+'.scse')
        print >> fh_info, '[EPot, ECoul, EExch]=', Enes
        print '[EPot, ECoul, EExch]=', Enes

        # Mixes total density
        if (p.broyden):
            rho_mix = lmt.BroydenMix(rho_s, nrho, p.admix_large, p.admix0, p.admix_rho, p.admix_mag)
        else:
            rho_mix = smix(rho_s, nrho, p.admix_rho)

        # Reads scf file to get new frozen potential
        scf_f = LmtScsf()
        (rho_f, rho_cor_f, pot_frz_f, efmu_f, Enu_f) = scf_f.read(lmt.project_name+'.scf')

        # Every restBrnd steps we restart Broyden mixing
        if (itt % p.restBrdn == 0):
            tmp_files = glob.glob(os.path.join(ldaexe.ksmdir, '*.tmp'))
            for fu in tmp_files: os.remove(fu)
        
        # Mixes the rest
        rho_cor_mix = smix(rho_cor_s, lmt.rho_cor, p.admix_frozen)
        pot_frz_mix = smix(pot_frz_s, pot_frz_f, p.admix_frozen)
        
        Enu_mix=[]
        for ispin in range(len(Enu_s)):
            tmp=[]
            for isort in range(len(Enu_s[ispin])):
                tmp.append((array(Enu_s[ispin][isort])*(1-p.admix_frozen) + array(Enu_f[ispin][isort])*p.admix_frozen).tolist())
            Enu_mix.append(tmp)
        
        # Writes new density to file
        
        scf_f.write(lmt.project_name+'.scsn', rho_mix, rho_cor_f, pot_frz_f, mu, Enu_mix)
        
        shutil.move(lmt.project_name+'.scsn', lmt.project_name+'.scs')

        print >> fh_info, '-'*80
        print >> fh_info, '*'*20, 'Iteration', itt+2, '*'*20
        print >> fh_info, '-'*80, '\n'

        # LDA computed new Hamiltonian ond Overlap
        ldaexe.RunLDA()

        # Reds new hamiltonian and computes new DMFT base
        lmt.read_ham_mix(p.mix_ham)
        lmt.cmpTransformation()

        # Print occupancy in case we need to restart
        WriteMocc(os.path.join(p.inpdir,"ksum/mocc.out"), mocc)

    
    if p.cmpDOS:
        fsig = os.path.join(p.inpdir,"sig.inp")
        if (os.path.exists(fsig)):
            (om,Sigc) = ReadSelfEnergyColumns(fsig)
        else:
            om = linspace(p.om_a, p.om_b, p.Nom)

        # Here we compute DOS
        fh_dos = open(os.path.join(p.inpdir,'dos.out'),'w')
        print "Computing dos... ", len(om)
        for i in range(len(om)):
            (TDOS,pDOS) = lmt.Partial_DOS(om[i] + mu, Sigma)
            fh_dos.write("%f \t %f \t " % (om[i], TDOS))
            for t in range(len(pDOS)):
                fh_dos.write("%f \t " % pDOS[t])
            fh_dos.write("\n")
        fh_dos.close()

