
import os,sys,errno,glob,re
# specific imports to this module
import ldau

ROOT = os.environ.get('LMTO_DMFT_ROOT')
if ROOT is not None:
    sys.path.append( ROOT )
    sys.path.append( ROOT + '/fort')
else:
    print >> sys.stderr, "Environment variable LMTO_DMFT_ROOT must be set!"
    print "Environment variable LMTO_DMFT_ROOT must be set!"
    sys.exit(1)

from ld_base import *

def force_symlink(file1, file2):
    try:
        os.symlink(file1, file2)
    except OSError, e:
        if e.errno == errno.EEXIST:
            os.remove(file2)
            os.symlink(file1, file2)

    
class LDApU(object):
    """The main class for LDA+U execution
    """
    def __init__(self, inpdir, kdir, fh_info=sys.stdout, scratch=False, lda_exe=ROOT+'/lda/lda'):
        """Checks the presense of mandatory files and sets-up input for LDA+U execution
        Copies mandatory files to subdirectories ./ksum and ./ksum/cener

        Input:
           lda_exe              -- the path and name of the Lmtart executable
           kdir                 -- directory for ksum - usualy 'ksum'
        Here we initialize:
           project_name         -- name of this project as used in LmtArt
           ksmdir               -- directory for ksum execution
           cendir               -- directory where total energy on outpu density is computed
           oini                 -- filename of ini file on root
           ostr                 -- filename of str file on root
           oscs                 -- filename of scs file on root
           kini                 -- filename of ini file in ksmdir
           kstr                 -- filename of str file in ksmdir
           kscs                 -- filename of scs file in ksmdir
           kpot                 -- filename of pot file in ksmdir
           
        """
        self.lda_exe = lda_exe
        self.fh_info = fh_info
        
        ini_files = glob.glob( os.path.join(inpdir, "*.ini") )

        if (len(ini_files)==0):
            print >> sys.stderr, "Could not find *.ini file therefore could not start calculation"
            sys.exit(1)
        else:
            if (len(ini_files)>1):
                print >> sys.stderr, "Found more than one ini file. Will use ", ini_files[0], "!"
                print >> sys.stderr, "All found files are=", ini_files

        m = re.match(".*\/(.*)\.ini", ini_files[0])
        self.project_name = m.group(1)        # It remembers name of the project and path to it

        print "Project name is ", self.project_name

        self.ksum = kdir
        self.ksmdir = os.path.join(inpdir, self.ksum)
        if len(glob.glob(self.ksmdir))==0 : os.mkdir( self.ksmdir )
        
        self.cener = 'cener'
        self.cendir = os.path.join(inpdir, self.ksum, self.cener)
        if len(glob.glob(self.cendir))==0 : os.mkdir( self.cendir )

        rinp = os.path.realpath(inpdir)
        self.oini = os.path.join(rinp, self.project_name + ".ini")
        self.ostr = os.path.join(rinp, self.project_name + ".str")
        self.oscs = os.path.join(rinp, self.project_name + ".scs")
        self.kini = os.path.join(rinp, self.ksum, self.project_name + ".ini")
        self.kstr = os.path.join(rinp, self.ksum, self.project_name + ".str")
        self.kscs = os.path.join(rinp, self.ksum, self.project_name + ".scs")
        self.kpot = os.path.join(rinp, self.ksum, self.project_name + ".pot")

        if not os.path.exists(self.oini):
            print >> sys.stderr, "Could not find ini file at ", oini, "exiting..."
            sys.exit(1)
        if not os.path.exists(self.ostr):
            print >> sys.stderr, "Could not find str file at ", ostr, "exiting..."
            sys.exit(1)

        force_symlink(self.ostr, self.kstr)
        
        self.scratch = scratch
        if os.path.exists(self.oscs) and (not self.scratch):
            force_symlink(self.oscs, self.kscs)
            self.IniFixEnus(self.oscs, self.oini, self.kini)
            
        else:
            self.scratch = True
            force_symlink(self.oini, self.kini)


    def __LDA_exe(self, dir):
        # Below we execute lda
        lda_cmd = 'cd '+dir+'; ' + self.lda_exe + ' <<EOF\n' + self.project_name + '\nini+str+scs\nscf+pot+out\nEOF 2>&1\n'
        print 'Running ---- LDA -----'
        lda_out = os.popen(lda_cmd).read()
        print >> self.fh_info, lda_out


    def RunFirstLDA(self):
        
        if (self.scratch): Niter1 = 200
        else: Niter1 = 1

        # Below we change Niter1 value in the pn.ini file
        fini = open(self.kini, "r")
        data = fini.read()
        fini.close()
        fini = open(self.kini, "w")
        data = re.sub(r'(Niter1\s*=)\s*(\d+)', '\\1 '+str(Niter1), data)
        fini.write(data)
        fini.close()

        # Below we execute lda
        self.__LDA_exe(self.ksmdir)

        # Her we modify ini file appropriately
        if (Niter1 != 1): # In future, we will run lda only for a single step
            # ini file is removed and replaced by one with fixed Enu's
            if os.path.exists(self.kini): os.remove(self.kini)
            # Enu's are fixed to there LDA value
            self.IniFixEnus(self.oscs, self.oini, self.kini)
            # Below we change Niter1 value in the pn.ini file
            fini = open(self.kini, "r")
            data = fini.read()
            fini.close()
            fini = open(self.kini, "w")
            data = re.sub(r'(Niter1\s*=)\s*(\d+)', '\\1 '+str(1), data)
            fini.write(data)
            fini.close()
        
        # move pot file to cener to compute total energy on output density
        # copy of ini and str files to cener
        shutil.move(self.kpot, self.cendir)
        shutil.copy2(self.kini, self.cendir)
        shutil.copy2(self.kstr, self.cendir)
        shutil.copy2(self.kscs, self.kscs+'.0')
        
    def RunLDAforEnergy(self, scsfile):
        # Prepares necessary input files
        shutil.move(scsfile, os.path.join(self.cendir, self.project_name+'.scs'))

        # Below we execute lda
        self.__LDA_exe(self.cendir)
        # And read what was produced in the output file with lda run
        fene = open(os.path.join(self.cendir , 'iener.dat'), 'r')
        fene.next()
        return [float(x) for x in fene.next().split()]

    def RunLDA(self):
        # Below we execute lda
        self.__LDA_exe(self.ksmdir)
        # move pot file to cener to compute total energy on output density
        shutil.move(self.kpot, self.cendir)

    def IniFixEnus(self, self_oscs, self_oini, self_kini):
        # Reading scs file to get Enu's
        scs_f = LmtScsf()
        (rho_s, rho_cor_s, pot_frz_s, efmu_s, Enus) = scs_f.read(self_oscs)
        ns = len(Enus)
        # Enus averaged over spins
        Enu = []
        for ip in range(len(Enus[0])):
            Enu.append(0.5*(array(Enus[0][ip])+array(Enus[ns-1][ip])))

        # Will fix Enu's to value in scs file
        sep = r'<SECTION='
        fini = open(self_oini)
        data = fini.readlines()
        fini.close()
        # Full ini file
        tdata = ''.join(data)
        # splitted at SECTION
        spl = tdata.split(sep)
        #
        fout = open(self_kini, 'w')
        k=0
        for i in range(len(spl)):
            if (i==0):
                print >> fout, spl[0],
            else:
        
                if (spl[i][0:4]=='SORT'):
                    if (k<len(Enu)):
        
                        str_Mnu = r' Mnu(:)='
                        for s in range(len(Enu[k])):
                            str_Mnu += '0  '
                        
                        str_Enu = r' Enu(:)='
                        for s in range(len(Enu[k])-1):
                            str_Enu += str(Enu[k][s]) + ' '
                        str_Enu += str(Enu[k][-1])
                        
                        m = re.search('Enu', spl[i])
                        if m is not None:
                            print 'found Enu'
                            p = re.compile( 'Mnu\(:\)\s*=(.*)\\n')
                            spli = p.sub(str_Mnu + '\n', spl[i])
                            p = re.compile( 'Enu\(:\)\s*=(.*)\\n')
                            splj = p.sub(str_Enu + '\n', spli)
        
                            print >> fout, (sep+splj),
                        else:
                            print >> fout, "%s%s" % (sep, spl[i]),
                            print >> fout, '<subsection=lmto>'
                            print >> fout, str_Mnu
                            print >> fout, str_Enu
                    else:
                        print >> fout, "%s%s" % (sep, spl[i]),
                        
                    k+=1
                else:
                    print >> fout, "%s%s" % (sep, spl[i]),
                    
        fout.close()



def ReadMocc(fmocc, ndim):
    if (not os.path.exists(fmocc)): return (False, None)
    fhn = open(fmocc, 'r')
    mocc = zeros((ndim, ndim), dtype=complex, order='F')
    for p in range(ndim):
        line = fhn.next().split()
        if (len(line) != ndim):
            print 'Could not read in', fmocc, '. Check input!'
            return (False, None)
        for q in range(ndim):
            mocc[p,q] = eval(line[q])
    fhn.close()
    return (True, mocc)

def WriteMocc(fmocc, mocc):
    fhn = open(fmocc, 'w')
    for p in range(shape(mocc)[0]):
        for q in range(shape(mocc)[1]):
            m = mocc[p,q].real
            if (abs(m)<1e-10): m=0
            print >> fhn, m,
        print >> fhn
    fhn.close()
    

def WriteMatrix(filename, Data):
    fm = open(filename, 'w')

    print >> fm, "%6d " % 0,
    for p in range(len(Data)): print >> fm, "%6d " % p,
    print >> fm

    for p in range(len(Data)):

        print >> fm, "%6d  " % p,

        for q in range(len(Data[p])):
            print >> fm, "%6.3f " % Data[p,q],
        print >> fm, sum(Data[p,:])
        
    fm.close()


def pars_params(param_file, params):
    "Parses file and evaluates variables which appear in both input file and list of params."
    fh_parm = open(param_file, 'r')
    print "Setting variables from ---",  param_file, "---"
    # Each variable from params is evaluated if found
    commands=[]
    while True:
        try:
            #com = read_next(fh_parm, 2)
            line = re.sub(r'(.*)\#.*', r"\1", fh_parm.next().strip()) # get line and remove comments
            m = re.match(r'\s*(\w+)\s*(.*)', line)
            if m is not None:
                com = m.groups()
                if len(com)>1 and com[0] in params: commands.append([com[0], com[1].strip()])
                
        except StopIteration:
            break
    fh_parm.close()
    return commands

class Params(object):
    """Class to store various control paramers which control the main flow of the program
       The class is constructed such that it can handle two types of parameters:
         - starting parameters which are read only at the instantiation
         - on fly parameters which can be refreshed (reread from the file) at any time
           by a member function refresh

       Input:
          f_sparams    -- filename of the parameters read only at the beginning
          sparams      -- a dictionary of pairs {name of the variable: default value} which are looked for in the f_sparams file
          f_params     -- filename of the parameters which can be refreshed several times by refresh member function
          params       -- a dictionary of {variable: default value} pairs which can be updated at runtime
       Output:
          None
       Variables which become members of the class are those listed in params and sparams dictionaries
    """
    def __init__(self, f_sparams, f_params, params):

        # executes the parameters at startup
        execfile(f_sparams)
        sp = locals()    # stores parameters
        del sp['self']   # deletes automatic locals
        
        self.p ={}
        self.p.update(params) # some default values
        self.p.update(sp)     # new or updates variables
        
        # remember params and its data file
        self.f_params = f_params
        
                
    def refresh(self, fh_info):

        # executes the parameters file
        execfile(self.f_params)
        sp = locals()   # stores locals
        del sp['self']  # deletes automatics

        # prints what was changed
        for c in sp.keys():
            if (c not in self.p.keys()) or (sp[c] != self.p[c]):
                print >>fh_info, '********------ parameter change ------***********'
                print >>fh_info, c, "=", sp[c]

        # updates the dictonary
        self.p.update(sp)
        
    def __getitem__(self, x):
        return self.p[x]

    def keys(self):
        return self.p.keys()

           
