# Author: Kristjan Haule, Jan 2007

"""This module allows reading of the data outputed by LmtArt program
such as Hamitonian, Overlap, Brillouin zone information, ....

  Available classes:
    Lmtart               -- contains data and methods to read data:
      Methods are:
        Lmtart.__init__            -- constructor 
        Lmtart.__del__             -- destructor
        Lmtart.read_ham            -- reads hamiltonian and overlap
        Lmtart.read_bz             -- reads Brillouin zone information (tetrahedrons)
        Lmtart.read_grp            -- reads Group operation data
        Lmtart.read_idx            -- read the index convention of hamiltonian, overlap,...
        Lmtart.cmpTransformation   -- computes transformation to the DMFT base
        Lmtart.read_cix            -- reads correlated index array named cix
        Lmtart.check_cix           -- checks if cix exists and creates default cix if not
        Lmtart.Correlated_LS_index -- gives a simple 1D array of correlated bands to be used in LDA+U part
        Lmtart.cmp_Gloc_coeff      -- computes the LMTO matrix elements which can be used to construct Gloc or density
        Lmtart.symmetrize          -- symmetrizes the above mentioned coefficients
"""

# Standard library imports.
import sys as s
import os.path
import shutil
import re
import glob
from scipy import *

# My python imports
import tutils as utl

# My fortran imports
import fort_read
from mod_Sk import msk
import symmetr as sym
import transb as trn
import strns as sts
import cmp_olocef as col
import linalg as la
import tetra
import cmp_rho as crho
import auxil as aux
import mix
import mtoverlap

# Other imports
import copy as cpy
from gdmft import *

ren = "[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?" # real number pattern

class Lmtart(object):
    """Class stores large amount of LmtArt data
    Available methods are:
    
    Deta stored in the class:
    
        integers: ndim, nkp, nspin, lmax, norb, norbs, natom, nsort, lmdim, ntet, nktot, ndiv[0:3], ngroup
        
        floats  : zvaltot, latcon, unit_celL_volume, mtet

        arrays  :
             isrt[natom]                             -- sort of each atom -- equivalent to is in fortran
             ilsb[nsort]                             -- maximum l in base
             ilsv[nsort]                             -- maximum l for the potential and charge density
             ilst[nsort]                             -- maximum for trial function expansion
             ltpan[lmax+1,natom]                     -- l-presence array for atoms
             ltsort[lmax+1,nsort]                    -- l-presence array for sorts
             lmtind[ilsbmax**2,natom]                -- orbital index
             llmtind[ilsmax**2,natom]                -- long orbital index
             znuc[nsort], zval[nsort]                -- nucleous and valence charge
             zsem[nsort], zcor[nsort]                -- semicore and core charge
             bndind[ndim,4] = (iatom,l,m,s)          -- from orbital index gives atom,l,m,s
             lbndind[lndmim,4] = (iatom,l,m,s)       -- the same for the long dimension
             bndindJ[ndim,4] = (iatom,l,j,mj)        -- from index in j-base gives atom,l,j,mj
             hamf[ndim,ndim,nkp], olap[ndim,ndim,nkp]-- hamiltonian and overlap
             wk[nkp]                                 -- reducible k-points weights
             bk[3,nkp],                              -- k-point coordinate iside 1BZ
             itt[5,ntet]                             -- (tetrahedron-multiplicity, corner-index-to-irr-k, ..)
             kirrptr[ndiv[0],ndiv[1],ndiv[2]]        -- 
             gropptr[ndiv[0],ndiv[1],ndiv[2]]        --
             ikind[ndiv[0],ndiv[1],ndiv[2]]          --
             rtet[ntet]                              --
             subtet[ntet,maxmult,4,3]                --
             ikov(nroup)                             -- ir-ig group correspondance (group index)
             rdc(3,natom,ngroup)                     -- coordinate of the atom after rotation
             igatom(natom,ngroup)                    -- index of the atom after rotation
             SJ(2,2,ngroup)                          -- Wigner matrix for spin(1/2)
             wigmat(imax,ng)                         -- Wigner matrices for rotation
             Utk[ndim,ndim,nkp]                      -- the transformation  U^+ H U rotates to DMFT base
             moo(4,0:lmax,nsort,nspin)               -- overlap numbers
             hh(ndim)                                -- the minimized parameters for transformation
             jj(ndim)                                -- such that  O_k = (j . S_k - h)^{+} . (j . S_k - h)
             
     """
    
    def __init__(self, inpdir_, fh_info=s.stdout, embed='DMFT', dioloc_file=None):
        """ Checks if ham.dat and olp.dat exist in input directory
        Inputs:
                Input directory as string
                Optionally, the name of the info file (otherwise stdout is used)
        Outputs:
                integers: ndim, nkp, nspin, lmax, norb, norbs
        """
#        if (fh_info!=s.stdout): fh_info = open(fh_info, 'w')
        self.fh_info = fh_info
        self.embed = embed
        #------------------------- info -----------------------------------------
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("INIT: Constructor\n")
        #------------------------- info -----------------------------------------
        
        file_ham  = inpdir_ + "/ham.dat"
        self.fh_info.write("Reading "+file_ham+"\n")
        dimh = utl.ReadFirstLine(file_ham) # Read dimensions from hamiltonian

        file_olap = inpdir_ + "/olp.dat"
        self.fh_info.write("Reading "+file_olap+"\n")
        dimo = utl.ReadFirstLine(file_olap) # Read dimensions from overlap

        # Check compatibility between Hamitonian and overlap
        if (dimo!=dimh): s.stderr.write("Dimensions in hamiltonian do not match overlap"); s.stderr.write(dimh,dimo); s.stderr.write("!\n"); s.exit(1)
        # ----------------- set class data so far ------------------------
        self.inpdir = inpdir_
        (self.nkp,self.norb,self.norbs,self.nspin,self.lmax) = dimh
        self.ndim = self.norb*self.nspin
        # Initialize some variables used below
        self.rtm_initialize = 0

        # Tries to find scs file and tried to find chemical potential
        scs_files = glob.glob(self.inpdir + "/*.scs")
        self.mu = None
        if (len(scs_files)==0):
            s.stderr.write("Could not find *.scs file therefore could not determin chemical potential\n")
        else:
            if (len(scs_files)>1):
                s.stderr.write("Found more than one scs file. Will use " + scs_files[0] + "\n")
                print >> s.stderr, 'Files are=', scs_files
            
            m = re.match("(.*)\.scs", scs_files[0])
            self.project_name = m.group(1)        # It remembers name of the project and path to it

            scs_cont = open(scs_files[0]).readlines() # Reads whole file in memory to find chemical potential
            
            Ry2eV = 13.6058
            self.mu = float(scs_cont[-1].split()[0])*Ry2eV # Chemical potential is the first number in the last line of scs.file

        print >> self.fh_info, 'mu from scs file is ', self.mu
        # Some printing
        #------------------------- info -----------------------------------------
        self.fh_info.write("Dimensions: \n")
        self.fh_info.write("          Size of the Ham matrix          : %d\n" % self.ndim)
        self.fh_info.write("          Number of irreducible k-points  : %d\n" % self.nkp)
        self.fh_info.write("          Number of spins                 : %d\n" % self.nspin)
        self.fh_info.write("          Maximum angular momentum in base: %d\n" % self.lmax)
        self.fh_info.write("          Number of orbits (norb)         : %d\n" % self.norb)
        self.fh_info.write("          Number of orbits (norbs)        : %d\n" % self.norbs)
        if (self.mu):
            self.fh_info.write("          project name found              : %s\n" % self.project_name)
            self.fh_info.write("          Chemical potential is           : %f\n" % self.mu)
        else:
            self.fh_info.write("          project name not found! Chemical potential not set!")
        self.fh_info.write("\n")

        self.read_idx() # reads dimensions and orbital information
        self.read_ham() # reads hamiltonian and overlap
        if dioloc_file==None:
            self.read_bz()  # reads Brillouin zone information
        self.read_grp() # reads group operations and Wigner matrices

        # Initializes the rotation matrices which are used for symmetrization of quantities over the Brillouin zobe
        if dioloc_file==None:
            sym.rtm.init(self.rdc, self.bk, self.wigmat, self.ltpan, self.ilsb, self.ilst, self.igatom, self.lmtind, self.llmtind, self.isrt, self.ikov, self.SJ, self.norb, self.lnorb, self.ndim, self.lndim, self.nspin, self.kirrptr, self.gropptr)

        # Creates additional transformation to cubic/relativistic harmonics
        if self.nspin > 1 and self.norbs > 1: 
            self.T2C = sts.cmp_t2j(self.bndindJ, self.bndind)
            self.fh_info.write("   * Adding rotation from (l,m,s) base to relativistic (l,j,mj) base\n")
        else:
            self.T2C = sts.cmp_t2c(self.isrt, self.ilsb, self.ltpan, self.lmtind, self.ndim, self.norb)
            self.fh_info.write("   * Adding rotation from spheric to cubic harmonics\n")

        self.cmpTransformation(dioloc_file) # computes transformation to DMFT base
        
        # Computes Overlap within MT spehere needed for some LDA+U calculation and Savrasov's embedding
        self.OlapMT = mtoverlap.cmp_mtoverlap(self.inpdir, self.bndind, self.isrt, self.nkp)
        self.add_sqrt2 = True

        self.worbs = array([[1, 2], [2, 1]], dtype=int, order='F') # diagonal -> orbs=1, off-diagonal -> orbs=2
        self.wupdn = array([[1, 1], [2, 2]], dtype=int, order='F') # for diagonal 1,1 and 1,2 -> 1
        

    def Embed(self, ikp, sig_DMFT_base):
        if (self.embed != 'Savrasov'):
            sig2 = array(cpy.deepcopy(sig_DMFT_base),order='F')
            la.ztransform(sig2, self.Utk[:,:,ikp],'N') # From DMFT base to LMTO basis
#            sym.symmetrize_spheric(sig2)               # Symmetrization of self-energy in LMTO basis
            # Checks causality of self-energy. Imaginary part should be negative
            la.check_causality2(sig2,1e-12)
            return sig2
        else:
            sig2 = array(cpy.deepcopy(sig_DMFT_base),order='F')
            la.ztransform(sig2, self.T2C,'C') # From cubic -> spheric
            sig2 = array(matrix(self.OlapMT[:,:,ikp])*matrix(sig2),order='F') # Embedding according to Sergej
#            sym.symmetrize_spheric(sig2)               # Symmetrization of self-energy in LMTO basis
            la.check_causality2(sig2,1e-12)
            return sig2

        
    def read_idx(self):
        """ Reads information about lmto orbitals and their order
        
        Inputs:
        Outputs:
                integers: natom, nsort, lmdim
                
                arrays  : isrt[natom]  -- sort of each atom -- equivalent to is in fortran
                          ilsb[nsort]  -- maximum l in base
                          ilsv[nsort]  -- maximum l for the potential and charge density
                          ilst[nsort]  -- maximum for trial function expansion
                          ltpan[lmax+1,natom] 
                          lmtind[ilsbmax**2,natom] -- orbital index
                          llmtind[ilsmax**2,natom] -- long orbital index
                          znuc[nsort], zval[nsort], zsem[nsort], zcor[nsort]
                          bndind[ndim,4] = (iatom,l,m,s)     -- from orbital index gives atom,l,m,s
                          lbndind[lndmim,4] = (iatom,l,m,s)  -- the same for the long dimension
                          bndindJ[ndim,4] = (iatom,l,j,mj)   -- from index in j-base gives atom,l,j,mj
                          bndindx is either bndind or bndindJ 
                floats  : zvaltot
        """
        #------------------- info ---------------------------------------
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("READ_IDX: Dimensions\n")
        #------------------- info ---------------------------------------
        #---------------- Reading idx file ---------------------------------
        file_idx = self.inpdir + "/idx.dat"
        self.fh_info.write("Reading "+file_idx+"\n")
        fh_idx = open(file_idx, 'r')
        fh_idx.readline() # comment
        (natom, nsort) = map(int,(fh_idx.readline()).split())
        fh_idx.readline() # comment
        #---------- Read atom to sort index and max l for each sort ----------
        self.isrt = zeros((natom),dtype=int)
        self.ilsb = zeros((nsort),dtype=int)
        self.ilst = zeros((nsort),dtype=int)
        self.ilsv = zeros((nsort),dtype=int)
        for iatom in range(natom):
            (it,isort,t_ilsb,t_ilst,t_ilsv) = map(int,(fh_idx.readline()).split())
            if (it != iatom+1): s.stderr.write("Error[1] in reading idx from %s!\n" % file_idx); s.exit(1)
            isort = isort-1
            self.isrt[iatom] = isort+1
            self.ilsb[isort] = t_ilsb
            self.ilst[isort] = t_ilst
            self.ilsv[isort] = t_ilsv
        ilsb_max = max(self.ilsb)
        ils_max  = max(map(max,[self.ilsb,self.ilst,self.ilsv]))
        fh_idx.readline() # comment
        fh_idx.readline() # comment
        # ---------- Angular momentum presence array ----------
        self.ltpan = array(zeros((ils_max+1,natom),dtype=int),order='F')
        self.ltsort =array(zeros((ils_max+1,nsort),dtype=int),order='F')
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            for l in range(self.ilsb[isort]+1):
                field = (fh_idx.readline()).split()
                if (int(field[0]) != iatom+1) : s.stderr.write("Error[2] in reading idx from %s!\n" % file_idx); s.exit(1)
                if (int(field[1]) != l) : s.stderr.write("Error[3] in reading idx from %s!\n" % file_idx); s.exit(1)
                if (field[2] == 'T'):
                    self.ltpan[l][iatom] = 1
                    self.ltsort[l][isort] = 1
        
        fh_idx.readline() # comment
        fh_idx.readline() # comment
        
        self.lmtind  = array(zeros(((ilsb_max+1)**2,natom),dtype=int),order='F')
        itmp=1
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            for l in range(self.ilsb[isort]+1):
                for m in range(-l,l+1):
                    fields = map(int,(fh_idx.readline()).split())
                    if fields[0] != iatom+1 : s.stderr.write("Error[4] in reading idx from %s!\n" % file_idx); s.exit(1)
                    if fields[1] != l : s.stderr.write("Error[5] in reading idx from %s!\n" % file_idx); s.exit(1)
                    if fields[2] != m : s.stderr.write("Error[6] in reading idx from %s!\n" % file_idx); s.exit(1)
                    lm = l*(l+1)+m
                    if (not self.ltpan[l,iatom]): continue
                    self.lmtind[lm,iatom] = itmp
                    itmp = itmp+1
        # Reading charge information at the end of the file (znuc, zval, zsem, zcor, zvaltot)
        fh_idx.readline() # comment
        rest=""
        for line in fh_idx: rest += line.strip() + " " # reading all the rest
        data = map(float,rest.split())                 # all data become float
        fh_idx.close() # Reading finished and file is closed
        # set the data
        self.znuc = data[0*nsort : 1*nsort]
        self.zval = data[1*nsort : 2*nsort]
        self.zsem = data[2*nsort : 3*nsort]
        self.zcor = data[3*nsort : 4*nsort]
        
        self.zvaltot = 0
        for iatom in range(natom):
            self.zvaltot += self.zval[self.isrt[iatom]-1]
        
        self.llmtind = array(zeros(((ils_max +1)**2,natom),dtype=int),order='F')
        itmp=1
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            for l in range(self.ilst[isort]+1):
                for m in range(-l,l+1):
                    lm = l*(l+1)+m
                    self.llmtind[lm,iatom] = itmp
                    itmp = itmp+1
        self.lnorb = itmp-1
        # Here need to create bndind and bndindJ
        self.lndim = self.lnorb * self.nspin
        self.natom = natom
        self.nsort = nsort
        # Set up index array (bndind) = (iatom,l,m,s)
        self.bndind = array(zeros((self.ndim,4),dtype=int),order='F')
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            for l in range(self.ilsb[isort]+1):
                if (not self.ltpan[l,iatom]): continue
                for m in range(-l,l+1):
                    lm = l*(l+1)+m
                    itmp = self.lmtind[lm,iatom]-1
                    if (itmp >= self.ndim): continue
                    self.bndind[itmp]      = [iatom+1,l,m,1]
                    if (self.nspin>1): self.bndind[itmp+self.ndim/2] = [iatom+1,l,m,2]
        # Set up the long index array (lbndind) = (iatom,l,m,s)
        self.lbndind = array(zeros((self.lndim,4),dtype=int),order='F')
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            for l in range(self.ilst[isort]+1):
                for m in range(-l,l+1):
                    lm = l*(l+1)+m
                    itmp = self.llmtind[lm,iatom]-1
                    if (itmp >= self.lndim): continue
                    self.lbndind[itmp]      = [iatom+1,l,m,1]
                    if (self.nspin>1): self.lbndind[itmp+self.lndim/2] = [iatom+1,l,m,2]

        # Set up j-base index array (bndindJ) = (iatom,l,j,mj)
        if (self.nspin > 1 and self.norbs>1):
            self.bndindJ = array(zeros((self.ndim,4),dtype=float),order='F')
            iw=0
            for iatom in range(natom):
                isort = self.isrt[iatom]-1
                for l in range(self.ilsb[isort]+1):
                    if (not self.ltpan[l,iatom]): continue
                    for j in arange(l-0.5,l+0.5+1):
                        if (j<0): continue
                        for mj in arange(-j,j+1):
                            self.bndindJ[iw] = [iatom+1,l,j,mj]
                            iw += 1

        if (self.nspin > 1 and self.norbs>1):
            self.bndindx = self.bndindJ
        else:
            self.bndindx = self.bndind
            
        self.fh_info.write("          Number of atoms in unit cel     : %d\n" % self.natom)
        self.fh_info.write("          Number of different atoms       : %d\n" % self.nsort)
        self.fh_info.write("          Large dimension lndim           : %d\n" % self.lndim)
    
        for isort in range(nsort):
            self.fh_info.write("          Mmaximum L(isort=%d): base=%d, psi_k=%d and potential=%d\n" % (isort,self.ilsb[isort],self.ilst[isort],self.ilsv[isort]))
            self.fh_info.write("          Nuclear charge: total=%f, valence=%f, semicore=%f, core=%f\n" % (self.znuc[isort],self.zval[isort],self.zsem[isort],self.zcor[isort]))
        self.fh_info.write("\n")
    
        
    def __del__(self):
        """ Destructor cleans up memory in fortran modules """
        if (self.rtm_initialize): sym.rtm.clean_up()
        
    
    def read_ham(self):
        """ Reads Hamitonian and Overlap matrix outputed by LmtArt.
        
        Output:
                integers: ndim, nkp, norb, norbs, nspin, lmax
                matrices: hamf[ndim,ndim,nkp], olap[ndim,ndim,nkp]

        """
        #------------------- info ---------------------------------------
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("READ_HAM:   Hamiltonian and Overlap\n\n")
        #------------------- info ---------------------------------------
        file_ham  = self.inpdir + "/ham.dat"
        file_olap = self.inpdir + "/olp.dat"
        #-------------------- Reading Hamiltonian ---------------
        self.fh_info.write("Reading hamf from "+file_ham+"\n")
        # fortran read
        self.hamf = fort_read.read_c(file_ham, self.ndim, self.ndim, self.nkp, 1, 1)
        # from Rydbergs to eV
        self.fh_info.write("Assuming Hamiltonian is in Rydberg->Changing to electron volts\n")
        Ry2eV = 13.6058
#        self.hamf = self.hamf*Ry2eV
        self.hamf *= Ry2eV
        #----------------------- Reading Overlap ----------------
        self.fh_info.write("Reading olap from "+file_olap+"\n\n")
        # fortran read
        self.olap = fort_read.read_c(file_olap, self.ndim, self.ndim, self.nkp, 1, 1)
        # export the rest of variables

    def read_ham_mix(self, mix):
        """ Reads Hamitonian and Overlap matrix outputed by LmtArt.
        
        Output:
                integers: ndim, nkp, norb, norbs, nspin, lmax
                matrices: hamf[ndim,ndim,nkp], olap[ndim,ndim,nkp]

        """
        #------------------- info ---------------------------------------
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("READ_HAM:   Hamiltonian and Overlap\n\n")
        #------------------- info ---------------------------------------
        file_ham  = self.inpdir + "/ham.dat"
        file_olap = self.inpdir + "/olp.dat"
        #-------------------- Reading Hamiltonian ---------------
        self.fh_info.write("Reading and mixing hamf from "+file_ham+" with mixing "+str(mix)+"\n")
        # fortran read
        ham = fort_read.read_c(file_ham, self.ndim, self.ndim, self.nkp, 1, 1)
        # from Rydbergs to eV
        self.fh_info.write("Assuming Hamiltonian is in Rydberg->Changing to electron volts\n")
        Ry2eV = 13.6058
        ham *= Ry2eV
        
        if (mix == 1.0):
            self.hamf = ham
        else:
            self.hamf = array(self.hamf*(1-mix) + ham*mix, order='F')
            
        #----------------------- Reading Overlap ----------------
        self.fh_info.write("Reading olap from "+file_olap+"\n\n")
        # fortran read
        olap = fort_read.read_c(file_olap, self.ndim, self.ndim, self.nkp, 1, 1)

        if (mix == 1.0):
            self.olap = olap
        else:
            self.olap = array(self.olap*(1-mix) + olap*mix, order='F')


    def read_bz(self):
        """ Reads Brillouin zone information outputed by LmtArt.
        
        Output:
                integers: nkp, ntet, nktot, ndiv[0:3]
                floats  : latcon, unit_celL_volume, mtet
                arrays  : wk[nkp],
                          bk[3,nkp],     (for rotmat)    --- k-point coordinate iside 1BZ
                          itt[5,ntet],   
                          kirrptr[ndiv[0],ndiv[1],ndiv[2]],
                          gropptr[ndiv[0],ndiv[1],ndiv[2]],
                          ikind[ndiv[0],ndiv[1],ndiv[2]],
                          rtet[ntet]
                          subtet[ntet,maxmult,4,3]

        Variables defined iside module:
                          maxmult

        Arrays defined inside module:                  
                          kcut[3,6,4],
                          nbr[nkp,4,3],
                          grop[nkp,4,3],
                          rvec[3,3],
                          mvec[3,3],
                          irrcor_array[ntet,4],
                          tetidx[ndiv[2],ndiv[1],ndiv[0],6],
                          coridx[ndiv[2],ndiv[1],ndiv[0],6,4],
        """
        #--------- info printing ----------------------
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("READ_BZ:   Brillouin zone information\n")
        #--------- info printing ----------------------
        #********************* Reading Brillouin zone information ******************
        file_bz = self.inpdir + "/bz.dat"
        self.fh_info.write("Reading "+file_bz+"\n")
        fh_bz = open(file_bz,'r')
        # Start reading
        line = fh_bz.readline() # First line is comment
        line = fh_bz.readline()
        fields = line.split()
        self.latcon = float(fields[0])
        (nkp,ntet,nktot) = map(int, fields[1:4])
        ndiv = map(int, fields[4:7])

        if (nkp!=self.nkp): s.stderr.write("Incompatible files BZ and Ham (nkp,nkp)=(%i,%i)\n"%(nkp,self.nkp)); s.exit(1)
        #---------- Reading inequivalent k-points ----------  
        self.wk=[]
        bk=[]
        line = fh_bz.readline() #  comment
        for i in range(nkp):
            line = fh_bz.readline()
            fields = line.split()
            if (int(fields[0])!=i+1): s.stderr.write("Something wrong reading Brillouin zone information (%s,%i)\n"%(fields[0],i+1)); s.exit(1)
            bk.append(map(float,fields[1:4]))
            self.wk.append(float(fields[4]))
        self.wk = self.wk/sum(self.wk)
        self.bk = array(transpose(bk),order='F')
        #---------- Reading inequivalent tetrahedra ----------
        itt=[]
        line = fh_bz.readline() #  comment
        for i in range(ntet):
            line = fh_bz.readline()
            fields = line.split()
            if (int(fields[0])!=i+1): s.stderr.write("Something wrong reading Brillouin zone information (%s,%i)\n"%(fields[0],i+1)); s.exit(1)
            itt.append(map(int,fields[1:-1])) # we do not read the last entry - type of tetrahedra. It is not necessary
        #---------- Reading kcut matrices ----------
        kcut=[] # be carefull, dimensions are (3,6,4)
        line = fh_bz.readline() #  comment
        for i in range(3):
            line = fh_bz.readline()
            field = map(int,line.split())
            kcut.append(reshape(field,(6,4)))   
        #---------- Reading neigbour information ----------
        nbr=[]  # dim=(nkp,4,3)
        grop=[] # dim=(nkp,4,3)
        line = fh_bz.readline() #  comment
        for i in range(nkp):
            line = fh_bz.readline()
            field = map(int,line.split())
            if (field[0]!=i+1): s.stderr.write("Something wrong reading Brillouin zone information (%s,%i)\n"%(field[0],i+1)); s.exit(1)
            nbr.append(reshape(field[1:13],(4,3)))
            line = fh_bz.readline()
            field = map(int,line.split())
            grop.append(reshape(field[1:13],(4,3)))
        #---------- Reading real space A-matrix ----------
        rvec=[]
        line = fh_bz.readline() #  comment
        for i in range(3):
            line = fh_bz.readline()
            field = map(float,line.split())
            rvec.append(field)
        self.rvec = [[rvec[r][i]/ndiv[i] for i in range(3)] for r in range(3)]
        mvec=[]
        line = fh_bz.readline() #  comment
        #---------- Reading momentum space minicell vectors ----------
        for i in range(3):
            line = fh_bz.readline()
            field = map(float,line.split())
            mvec.append(field)
        self.mvec = [[rvec[r][i]*ndiv[i]*2*pi for i in range(3)] for r in range(3)]
        #---------- Computing unit cell volume ----------
        self.unit_cell_volume = abs(linalg.det(matrix(rvec)))/(ndiv[0]*ndiv[1]*ndiv[2])
        #---- Reading all k-point coordinates, reference to irreducible partner and the group operation
        line = fh_bz.readline() #  comment

        kirrptr=[]
        gropptr=[]
        ikind=[]
        ij=0
        for i2 in range(ndiv[2]):
            for i1 in range(ndiv[1]):
                for i0 in range(ndiv[0]):
                    line = fh_bz.readline()
                    field = map(int,line.split())
                    if (field[0]!=i0 or field[1]!=i1 or field[2]!=i2): s.stderr.write("Something wrong reading Brillouin zone information all points"); s.exit(1)
                    kirrptr.append(field[3])
                    gropptr.append(field[4])
                    ikind.append(ij)
                    ij = ij + 1

        fh_bz.close()
        
        kirrptr = reshape(kirrptr, (ndiv[2],ndiv[1],ndiv[0]));
        gropptr = reshape(gropptr, (ndiv[2],ndiv[1],ndiv[0]));
        ikind   = reshape(ikind,   (ndiv[2],ndiv[1],ndiv[0]));

        self.kirrptr = array(transpose(kirrptr),order='F')
        self.gropptr = array(transpose(gropptr),order='F')
        self.ikind   = array(transpose(ikind),order='F')
        self.itt     = array(transpose(itt),order='F')
        # total tetrahedron mass
        self.mtet = sum(array(self.itt)[0,:])

        #---------- Set up sorted tetrahedron corner array ----------
        irrcor_array=[]
        for itet in range(ntet):
            tc = itt[itet][1:5]; tc.sort()
            irrcor_array.append(tc)

        #---------- Set up the reference pointers for all tetrahedra ----------
        tetidx=[]
        coridx=[]
        for i2 in range(ndiv[2]):
            for i1 in range(ndiv[1]):
                for i0 in range(ndiv[0]):
                    for ittr in range(6): # there are six tetrahedrons
                        # First find irreducible k-point for all corners
                        regcor=[]
                        for ic in range(4): # the reducible k-point is obtained
                            j0 = (i0+kcut[0][ittr][ic])%ndiv[0]
                            j1 = (i1+kcut[1][ittr][ic])%ndiv[1]
                            j2 = (i2+kcut[2][ittr][ic])%ndiv[2]
                            regcor.append(self.kirrptr[j0][j1][j2]) # gives corners in terms of irreducible point
                        creg = regcor[:] # save regcor for later use
                        regcor.sort()    # sorts corners
                        itet = irrcor_array.index(regcor) # Which irreducible tetrahedron?
                        tetidx.append(itet)
                        # Gather the corners up for matching
                        orig_reg = itt[itet][1:5]
                        # Match up the corners
                        for i in range(len(creg)): 
                            j = orig_reg.index(creg[i])
                            coridx.append(j)
                            orig_reg[j]=-1  # if same index appears twice, we need to find unique index
        tetidx = reshape(tetidx,(ndiv[2],ndiv[1],ndiv[0],6))
        coridx = reshape(coridx,(ndiv[2],ndiv[1],ndiv[0],6,4))

        #------------------- This is for debugging! Could be commented out if necessary---------------
        flat_tetidx = utl.flatten(tetidx.tolist())
        # Checking that multiplicity of each tetrahedron is te same as occurence in tetidx
        for itet in range(ntet):
            if (flat_tetidx.count(itet) != itt[itet][0]): s.stderr.write("tetidx array seems to be wrong! Check it!"); s.exit(1)
        #------------------- This is for debugging! Could be commented out if necessary---------------


        #---------- Set up the subtet index array ----------
        self.rtet = zeros(ntet,dtype=int)
        # biggest tetrahedron multiplicity
        maxmult = max(array(itt)[:,0])
        # subtetrahedrons of size: (ntet,multiplicity,4,3)
        self.subtet = array(zeros((ntet,maxmult,4,3),dtype=int),order='F')
        # Sum over all k-point coordinates
        for i2 in range(ndiv[2]):
            for i1 in range(ndiv[1]):
                for i0 in range(ndiv[0]):
                    for ittr in range(6): # there are six tetrahedrons
                        itet = tetidx[i2][i1][i0][ittr] # Find the irreducible representative
                        subt = zeros((4,3),dtype=int)
                        # Loop over all corners
                        for ic in range(4):
                            # Find the coordinates
                            j0 = (i0+kcut[0][ittr][ic])%ndiv[0]
                            j1 = (i1+kcut[1][ittr][ic])%ndiv[1]
                            j2 = (i2+kcut[2][ittr][ic])%ndiv[2]
                            # Find which corner this is in the representative
                            cor = coridx[i2][i1][i0][ittr][ic]
                            # Save the coordinates
                            subt[cor] = array([j0,j1,j2])

                        self.subtet[itet][self.rtet[itet]] = subt;
                        # Increment the subtetrahedra counter
                        self.rtet[itet] += 1
                        
        self.ndiv = ndiv
        self.ntet = ntet
        self.nktot = nktot
        
        self.fh_info.write("          Number of k-points      : %d\n" % self.nkp)
        self.fh_info.write("          Total # k-points        : %d\n" % self.nktot)
        self.fh_info.write("          Number of tetrahedra    : %d\n" % self.ntet)
        self.fh_info.write("          Total tetrahedral mass  : %d\n" % self.mtet)
        self.fh_info.write("          Divisions along axis 1  : %d\n" % self.ndiv[0])
        self.fh_info.write("          Divisions along axis 2  : %d\n" % self.ndiv[1])
        self.fh_info.write("          Divisions along axis 3  : %d\n" % self.ndiv[2])
        self.fh_info.write("          Number of orbitals:     : %d\n" % self.norb)
        self.fh_info.write("          Dimension of Hamiltonian: %d\n" % self.ndim)
        self.fh_info.write("          Lattice constant        : %f\n" % self.latcon)
        self.fh_info.write("          Unit cell volume        : %f\n" % self.unit_cell_volume)
        self.fh_info.write("\n");

    def read_grp(self):
        """ Reads group operations outputed by LmtArt..
        
        Output:
              integers:
                         ngroup
              arrays:    
                         ikov(nroup)          --- ir-ig group correspondance (group index)
                         rdc(3,natom,ngroup)  --- coordinate of the atom after rotation
                         igatom(natom,ngroup) --- index of the atom after rotation
                         SJ(2,2,ngroup)       --- Wigner matrix for spin(1/2)
                         wigmat(imax,ng)      --- Wigner matrices for rotation
        """
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("INI_GRP:  Group operations")
        file_grp  = self.inpdir + "/grp.dat"
        #-------------------- Reading cix file ---------------
        self.fh_info.write("Reading grp file from "+file_grp+"\n")
        fh = open(file_grp,'r')
        fh.readline() # comment
        fh.readline() # comment
        (ngroup, natom) = map(int, (fh.readline()).split())
        if (natom != self.natom): s.stderr.write("Number of atoms in idx and wig are not compatible (%d,%d)" % (natom,self.natom))
        fh.readline() # comment
        fh.readline() # comment

        self.ikov=[]
        for ir in range(ngroup):
            fields = map(int, (fh.readline()).split())
            if (fields[0] != ir+1): s.stderr.write("Error[1] in reading groups from %s!\n" % file_grp); s.exit(1)
            self.ikov.append(fields[1]);
        fh.readline() # comment
        fh.readline() # comment
        rdc=[]
        igatom=[]
        for ir in range(ngroup):
            for iatom in range(natom):
                fields = (fh.readline().split())
                if (int(fields[0]) != ir+1): s.stderr.write("Error[2] in reading groups from (%s,%d)!\n" % (fields[0],ir+1)); s.exit(1)
                if (int(fields[1]) != iatom+1): s.stderr.write("Error[3] in reading groups from (%s,%d)!\n" % (fields[1],iatom+1)); s.exit(1)
                igatom.append(int(fields[2]))
                rdc.append(map(float,fields[3:6]))
        fh.close()
        
        igatom = reshape(igatom, (ngroup,natom))
        rdc = reshape(rdc, (ngroup,natom,3))
        self.igatom = array(transpose(igatom),order='F')
        self.rdc = array(transpose(rdc),order='F')
        #--------- Read Wigner matrix for spin 1/2
        # fortran read
        beg_skip = 2+1+2+ngroup+2+ngroup*natom+1;
        self.SJ = fort_read.read_c(file_grp, 2, 2, ngroup, beg_skip, 0)
        self.ngroup = ngroup
        #---------- Open Wigner matrix data file ----------
        file_wig  = self.inpdir + "/wig.dat"
        self.fh_info.write("Reading Wigner matrices from file "+file_wig+"\n")
        dimw = utl.ReadFirstLine(file_wig) # Read dimensions from wigner data
        (ng, lmax) = dimw
        #---------- Check parameters ----------
        if (ng == ngroup):
            self.fh_info.write("\n          Maximal number of group operations (wig,out)=(%d,%d)\n" % (ng,ngroup))
        elif (ng > ngroup):
            self.fh_info.write("\n          Reduced number of group operations (wig,out)=(%d,%d)\n" % (ng,ngroup))
        else:
            self.fh_info.write("\n          Enhanced number of group operations (wig,out)=(%d,%d)\n" % (ng,ngroup))
        #---------- info printing ----------------------------
        self.fh_info.write("          Number of group operations             : %d\n" % self.ngroup)
        self.fh_info.write("          Number of atoms in unit cell (wig,out) : (%d,%d)\n" % (natom,self.natom))
        self.fh_info.write("          Number of angular momenta    (wig,out) : (%d,%d)\n" % (lmax,self.lmax))
        #---------- Compute maximum combined index ----------
        imax = int(lmax*(2*lmax-1)*(2*lmax+1)/3 + (2*lmax+1)**2)
        #---------- Read in the Wigner matrices ----------
        # fortran read
        self.wigmat = fort_read.read_sparse(file_wig, imax, ng, 1)
        
    def cmpTransformation(self, dioloc_file=None):
        """ Computes transformation from LMTO base to DMFT base
        
        Output:
              arrays:    
                     Utk(ndim,ndim)            --- the transformation  U^+ H U rotates to DMFT base
                     dioloc(ndim)              --- transformation is Utk = dioloc*(h-j*Sk)
                     moo(4,0:lmax,nsort,nspin) --- overlap numbers
                     hh(ndim)                  --- the minimized parameters for transformation
                     jj(ndim)                  --- such that  O_k = (j . S_k - h)^{+} . (j . S_k - h)
                                               --- or equivalently o(1)=h^{*}h  ==  OHH, o(2)=j^{*}h  ==  OJH
                                               ---   o(3)=h^{*}j  ==  OHJ, o(4)=j^{*}j  ==  OJJ 
        Input:
              It is necessary to run the following routines before:
                                                                 read_ham(), read_bz(), read_grp()
              
        """
        #------------------- info ---------------------------
        self.fh_info.write("-"*80+"\n")
        self.fh_info.write("CMP_TRANSFORMATION:  Computes DMFT base\n")
        #------------------- info ---------------------------
        # For printing fortran info 
        p=re.compile(r"\\n")
        # Gets overlap numbers from the opar.dat file
        inf_buf = array(" "*10000)
        (self.hh, self.jj, self.moo) = trn.get_overlap_numbers(self.inpdir, self.bndind, self.isrt, self.ltsort, self.lmax, self.nspin, inf_buf)
        self.fh_info.write(p.sub("\n", inf_buf.tolist()).rstrip(" \r\n")+"\n")

        # Comptutes the transformation. It uses symmetrize_spheric which is defined in include symmetr
        inf_buf = array(" "*1000)
        if dioloc_file==None:
            (self.Utk, self.dioloc) = trn.cmp_transformation1(sym.symmetrize_spheric, self.inpdir, self.olap, self.wk, self.isrt, self.ilsb, self.lmtind, self.ltpan, self.bndind, self.norb, self.nspin, self.lmax, self.hh, self.jj, inf_buf)
            self.fh_info.write(p.sub("\n", inf_buf.tolist()).rstrip(" \r\n")+"\n")
        else:
            execfile(dioloc_file)
            self.dioloc=array(locals()['dioloc'])
            self.Utk = trn.cmp_transformation1_short(self.inpdir, self.bndind, self.hh, self.jj, self.dioloc, self.natom, self.nkp)
            print 'Dioloc was successfully read!'

        # Transformation Utk will rotate from SS base directly to DMFT base (not to spheric but relativistic/cubic harmonics)
        for ikp in range(self.nkp):
            self.Utk[:,:,ikp] = la.zproduct_cn(self.T2C, self.Utk[:,:,ikp]) 

        if dioloc_file==None:
            # Checks local overlap matrix how far from unity it is
            inf_buf = array(" "*10000)
            col.cmp_olocef(self.olap, self.Utk, self.wk, self.T2C, sym.symmetrize_cubic, inf_buf)
            self.fh_info.write(p.sub("\n", inf_buf.tolist()).rstrip(" \r\n")+"\n")

        
    def check_cix(self, dcix):
        """ It checks if kcix.dat exists in the ksum directory. If it does, it does not do anything.
        If it does not exists, it creates a new kcix.dat according to dcix input from 'sparams.dat'.
        It tries to guess how kcix should be arranged. However, fine tuning is left to the user.
        The dcix (is a dictionary) can take the following values:
          {(atom1,atom2,...):(l1,...), (atom5,atom6,...):(l3,...),...}
          Here (atom1,atom2,...) is a tuple of correlated and equivalent atoms. Each of these atoms has a correlated
          subshel (l1,...).
          There can be more correlated atoms, which are not equivalent. Therefore (atom5,atom6,..) stands for the second
          grooup of equivalent atoms.

          
        """
        filename = self.inpdir + "/kcix.dat"

        if (os.path.isfile(filename)):
            self.fh_info.write("CHECK_CIX: success: kcix.dat exists!\n")
            return True

        #print 'dcix=', dcix
        
        if dcix==None:
            print >> s.stderr, 'You did not give me any hint of how to create the cix file! Boilling out!'
            s.exit(0)
            
        s.stderr.write("The kcix file "+filename+" does not exist.\nWill try to guess it.\nYou should edit it and set correlated index!\n")
        self.fh_info.write("fail: kcix.dat does not exists!\nWill try to guess it.\nYou should edit it and set correlated index!\n")

        fh_cix = open(filename, 'w')
        
        #new------------------------------------
        ilcix={}; tii=0; tii1=0; tjj=-1; lstruct={}
        dcixSpecified=False
        for c in dcix.keys():
            if type(dcix[c])==dict :
                ils=dcix[c].keys()
                dcixSpecified=True
            elif type(dcix[c]) in (tuple,list) :
                ils=dcix[c]
            else:
                ils=(dcix[c],)
        
            if type(c)==tuple : atoms = c
            else: atoms = (c,)
        
            for il in ils:
                
                if type(dcix[c])==dict :
                    sequence = (array(dcix[c][il])+tjj).tolist()
                    tjj += max(dcix[c][il])
                else:
                    sequence = [tjj+1]*(2*il+1)
                    tjj += 1
                sequence.reverse()
                
                for iatom in atoms:
                    ilcix[(iatom,il)] = (tii1,tii)
                    lstruct[tii1]= (sequence * self.nspin)
                    tii1+=1  # counts correlated blocks. When G_{loc} needs to be invertex, each block needs to be inverted separately
                tii+=1       # count only non-equivalent blocks.
                    
        #print 'ilcix=', ilcix
        #print 'lstruct=', lstruct
        #print 'sp=', dcixSpecified
        
        kilcix = ilcix.keys()
        cind = [[] for i in range(tii1)]
        
        if self.nspin > 1 and self.norbs > 1:
            fh_cix.write('#%2s  %3s    %3s  %3s  %3s  %4s  %4s\n' % ('i', 'ci', 'atm', 'l', 'j', 'mj', 'Esh'))

            tj=0
            pil=-1; pij=0.5
            for i in range(self.ndim):
                iatom = self.bndindJ[i,0]
                il    = self.bndindJ[i,1]
                ij    = self.bndindJ[i,2]
                ijz   = self.bndindJ[i,3]
            
                if il!=pil: iij = 0; pil=il; pij=ij
                if ij!=pij: iij += 1; pij=ij
                
                
                if (int(iatom),int(il)) in kilcix: # this atom and l is correlated
                    ii = ilcix[(iatom,il)]
                    tii1 = ii[0]
                    tii = ii[1]
                    
                    if not dcixSpecified:
                        cind[tii1].append((i, 2*tii+iij))
                    else:
                        if lstruct.has_key(tii1):
                            #print i, tii1, lstruct[tii1],
                            v = lstruct[tii1].pop()
                            #print v
                            cind[tii1].append((i, v))
                        else:
                            cind[tii1].append((i, tii))
                        
                    fh_cix.write('%3d  %3d    %3d  %3d  %3.1f  %4.1f  %4.1f\n' % (i+1, tii1+1, iatom, il, ij, ijz, 0.0))
                else:
                    fh_cix.write('%3d  %3d    %3d  %3d  %3.1f  %4.1f  %4.1f\n' % (i+1, 0, iatom, il, ij, ijz, 0.0))

        else:
            fh_cix.write('#%2s  %3s    %3s  %3s %10s  %3s\n' % ('i', 'ci', 'atm', 'l', 'orb', 'Esh'))
            
            tcubc = reshape(sts.get_tcubc().split(),(4,7))

            for i in range(self.ndim):
                iatom = self.bndind[i,0]
                il    = self.bndind[i,1]
                charorb = tcubc[il, self.bndind[i,2]+3]
                if (iatom,il) in kilcix: # this atom and l is correlated
                    ii = ilcix[(iatom,il)]
                    tii1 = ii[0]
                    tii = ii[1]
                    
                    if lstruct.has_key(tii1):
                        #print i, tii1, lstruct[tii1],
                        v = lstruct[tii1].pop()
                        #print v
                        cind[tii1].append((i, v))
                    else:
                        cind[tii1].append((i, tii))
                    fh_cix.write('%3d  %3d    %3d  %3d %10s  %3.1f\n' % (i+1, tii1+1, iatom, il, charorb, 0.0))
                else:
                    fh_cix.write('%3d  %3d    %3d  %3d %10s  %3.1f\n' % (i+1, 0, iatom, il, charorb, 0.0))

        fh_cix.write('# matrices of correlated indeces\n')
        
        for cin in cind:
            fh_cix.write('# ')
            for j in cin: fh_cix.write("%3d  " % (j[0]+1))
            fh_cix.write('\n')
            for ij,j in enumerate(cin):
                fh_cix.write(" ");
                for ik,k in enumerate(cin):
                    if ij==ik : ii = j[1]+1
                    else : ii=0
                    fh_cix.write("%3d  " % ii)
                fh_cix.write(" # %3d\n" % (j[0]+1))

        #new------------------------------------
                    
        fh_cix.write('# iatom  #rotations [vector angle [vector angle] ...]\n')
        for iatom in range(self.natom):
            print >> fh_cix, "%3d " % (iatom+1)

        fh_cix.close()
        
        return False
    
    def read_cix(self, dcix=None, filename=None):
        """ Reads the cix-file which contains the information about the correlated orbitals.
        The information is stored in the following arrays:
           Sigind[ndim,ndim]      -- contains the information how to constract a matrix of self-energy from the coulmns in the file of self-energy
                                     Sigma[i,j] = Sig_column[Sigind[i,j]]
           vSigind[ncb][nv]       -- enumerates blocks of correlates sets of bands. ncb is number of correlated blocks and nv is number of orbitals within a correlated block
           Imps[nimp][]           -- knows how many impurity problems needs to be solved
                                     Imps[i]
                                     vSigind[Imps[0][0]], vSigind[Imps[0][1]],... are columns obtained by solving the impurity problem number 0
                                     vSigind[Imps[1][0]], vSigind[Imps[1][1]],... are columns obtained by solving the impurity problem number 1
                                     ...
           
        Few examples should demonstrate the concept:
           1) One correlated atom with correlated index between 1...7 and 21..27.
              The indexes 1...6 have the same self-energy (like 5/2 band) and 7,21..27 have the same self-energy (like 7/2 band)
              
              vSigind=[[1,2,3,4,5,6,7,21,22,23,24,25,26,27]]
              Sigind=[[1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      ................
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
              Imps=[[1]]
              
           2) Two correlated atoms, but we need to solve only one impurity problem because the two atoms are equivalent.
              The correlated index of the first atom is 1...7 and the second atom 11...17. All self-energies are the same.
              
              vSigind=[[1,2,3,4,5,6,7],[11,12,13,14,15,16,17]]
              Sigind=[[1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      ................
                      [0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
              Imps=[[0,1]]
              
           3) Two inequivalent correlated atoms.
              The correlated index of the first atom is 1...7 and the second atom 11...17.
              All self-energies within the atom are the same.
              
              vSigind=[[1,2,3,4,5,6,7],[11,12,13,14,15,16,17]]
              Sigind=[[1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      ................
                      [0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0...],
                      [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0...],
              Imps=[[0],[1]]
              
        """
        if filename==None:
            filename = self.inpdir + "/kcix.dat"
            
        if (not os.path.isfile(filename)): self.check_cix(dcix)

        fh_cix = open(filename, 'r')

        self.Sigind = zeros((self.ndim,self.ndim),dtype=int)

        cind=[]
        for i in range(self.ndim): cind.append([])
            
        if self.nspin > 1 and self.norbs > 1 :
            it=0
            while (it<self.ndim):
                line = fh_cix.readline()
                while (re.match('^\s*#', line) is not None): line = fh_cix.readline() # reading comments
                it += 1   # found line
                field = line.split()
                (ind, ci, atm, l) = map(int,field[0:4])
                (j,mj,Esh) = map(float,field[4:7])
                if (it != ind) : s.stderr.write("Cix file seems to be corrupted (%d,%d)"%(it,ind)); s.exit(1)
                if (l != self.bndindJ[it-1,1]): s.stderr.write("Cix file error: l does not match %d:(%d,%d)\n"%(it,l,self.bndindJ[it-1,1])); s.exit(1)
                if (atm != self.bndindJ[it-1,0]): s.stderr.write("Cix file error: atom does not match %d:(%d,%d)\n"%(it,atm,self.bndindJ[it-1,0])); s.exit(1)
                if (ci>0): cind[ci-1].append(it-1)
                
                #self.Sigind[it-1,it-1] = ci
        else:
            it=0            
            while (it<self.ndim):
                line = fh_cix.readline()
                while (re.match('^\s*#', line) is not None):  line = fh_cix.readline() # reading comments
                it += 1   # found line
                field = line.split()
                (ind, ci, atm, l) = map(int,field[0:4])
                Esh = float(field[5])
                if (it != ind) : s.stderr.write("Cix file seems to be corrupted"); s.exit(1)
                if (l != self.bndind[it-1,1]): s.stderr.write("Cix file error: l does not match"); s.exit(1)
                if (atm != self.bndind[it-1,0]): s.stderr.write("Cix file error: atom does not match"); s.exit(1)
                if (ci>0): cind[ci-1].append(it-1)
                
                #self.Sigind[it-1,it-1] = ci

        self.vSigind=[]
        for cin in cind:
            if len(cin)>0:
                self.vSigind.append(cin)

        
        for cin in cind:
            it=0
            for j in range(len(cin)):
                line = fh_cix.readline()
                while (re.match('^\s*#', line) is not None):  line = fh_cix.readline() # reading comments
                it += 1 # found line
                line1 = re.sub(r'(.*)\#.*', r"\1", line.strip())  # remove comments at the end
                field = map(int, line1.split())                             # have one line of correlated matrix
                if (len(field)!=len(cin)) : s.stderr.write("Somethig wrong reading kcix! matrix seems to have too large dimension")
                for k in range(min(len(cin),len(field))):
                    if field[k]!=0:
                        if field[k]>self.ndim: s.stderr.write("Somethig wrong reading kcix! Correlated index > ndim")
                        self.Sigind[cin[j],cin[k]] = field[k]
                


        # Below it creates self.Imps which contains information of how many impurity solvers we need
        self.Impc=[]
        self.Impr=[]
        self.Impq=[]
        imps = zeros(len(self.vSigind),dtype=int)
        nv=1
        mtrs=[]
        for icin,cin in enumerate(self.vSigind): # over all correlated blocks
            if len(cin)==0: pass
            
            mtr = zeros((len(cin),len(cin)),dtype=int) # index for this correlated block
            for j in range(len(cin)):
                for k in range(len(cin)):
                    mtr[j,k] = self.Sigind[cin[j],cin[k]]
                    
            # check if this correlated block is equivalent to some other correlated block met before
            for m in mtrs:  # over all earlier blocks
                if shape(mtr)==shape(m[1]) and (sum(m[1])-sum(mtr)==0): # maybe they are the same?
                    imps[icin] = imps[m[0]]  # yes, the are. Then they have the same imps>0
                    break
                
            if imps[icin]==0:                # we checked all previous blocks and could not find equivalent block. Must be new type! 
                mtrs.append([icin,mtr])      # save the new type of block
                imps[icin] = nv              # and give it new index
                nv+=1
                cols = utl.union(mtr.flatten())
                cols.remove(0)
                cols = (array(cols)-1).tolist()
                self.Impc.append([min(cols),max(cols)+1])
                dia_cols = [mtr[r,r]-1 for r in range(len(mtr))]

                print 'dia_cols', dia_cols
                
                if self.nspin==2 and self.norbs!=2: dia_cols = dia_cols[:len(dia_cols)/2]
                self.Impr.append(dia_cols)
                self.Impq.append(mtr)

        self.Impw={}
        for icin,cin in enumerate(self.vSigind):
            ii = imps[icin]-1
            if self.Impw.has_key(ii): self.Impw[ii] += cin
            else: self.Impw[ii] = cin

        print 'Impw = ', self.Impw
        
        #print 'imps=', imps
        #print 'Impc=', self.Impc
        #print 'Impr=', self.Impr
        #print 'Impq=', self.Impq
        
        # Rotation
        pat_vector = "\(\s*(" + ren + ")\s*\,\s*(" + ren + ")\s*\,\s*(" + ren + ")\s*\)"
        pat_angle = "(" + ren + ")"

        # In next few lines, rotation of atoms is implemented
        # in case the local coordinate system is different from
        # the global coordinate system
        Rrot = identity(self.ndim) # Will create rotation matrix for all orbitals
        # Need to go over all atoms and their (l,m,s)
        for iatom in range(self.natom):
            line = fh_cix.readline()
            while (re.match('^\s*#', line) is not None): line = fh_cix.readline() # reading comments

            field = line.split()

            iat = int(field[0])
            if (iatom+1 != iat): s.stderr.write("Cix file error: atom does not match (%d,%d)\n"%(iatom+1,iat)); #s.exit(1)

            mz = re.search('z\s*=\s*'+pat_vector, line)
            mx = re.search('x\s*=\s*'+pat_vector, line)
            if mz is not None and mx is not None:
                rz = map(float,[mz.group(1),mz.group(3),mz.group(5)])
                rx = map(float,[mx.group(1),mx.group(3),mx.group(5)])
                ry = cross(rz,rx)

                if abs(dot(rz,rx))>1e-4:
                    print >> s.stderr, ("Could not rotate the atom %d because r*z!=0.0" % iatom), "z=", ("%f,"*3 % tuple(rz)), "x=", ("%f,"*3 % tuple(rx))
                    s.exit(1)
                if abs(dot(rz,rz))<1e-4:
                    print >> s.stderr, ("Could not rotate the atom %d because |z|==0.0" % iatom), "z=", ("%f,"*3 % tuple(rz)), "x=", ("%f,"*3 % tuple(rx))
                    s.exit(1)
                if abs(dot(rx,rx))<1e-4:
                    print >> s.stderr, ("Could not rotate the atom %d because |x|==0.0" % iatom), "z=", ("%f,"*3 % tuple(rz)), "x=", ("%f,"*3 % tuple(rx))
                    s.exit(1)
                    
                rx = array(rx)/sqrt(dot(rx,rx))
                ry = array(ry)/sqrt(dot(ry,ry))
                rz = array(rz)/sqrt(dot(rz,rz))
                
                R0 = array([rx, ry, rz])

                # Here we fill in the (ndim x ndim) matrix
                for l in range(self.ilsb[self.isrt[iatom]-1]+1):
                    if (not self.ltpan[l,iatom]): continue
                    inds = self.lmtind[l**2,iatom]-1 # start index of this block in Ham matrix 
                    inde = inds + 2*l+1              # end index of this block in Ham matrix
                    #print iatom, l, inds, inde
                    if l == 1 : Rrot[inds:inde,inds:inde] = R0
                    elif l==2 : Rrot[inds:inde,inds:inde] = sts.rotate_d(R0)
                    elif l>2  : s.stderr("Did not yet implement rotation for non s,p,d atoms!")

                
                
        if (self.ndim==self.norb*2):
            Rrot[self.norb:2*self.norb,self.norb:2*self.norb] = Rrot[0:self.norb,0:self.norb]
                
        # Transformation T2C is updates
        self.T2C = array( matrix(self.T2C) * matrix(Rrot).T, order='F')
        # Transformation Utk is updates
        for ikp in range(self.nkp):
            self.Utk[:,:,ikp] = la.zproduct_nn(Rrot, self.Utk[:,:,ikp]) 

    def Correlated_LS_index(self):
        " Creates correlated index in LS-base"
        corind = zeros(self.ndim,dtype=int)
        
        if self.nspin > 1 and self.norbs > 1: # relativistic harmonics
            
            for i in range(self.ndim):
                if self.Sigind[i,i] == 0 : continue # non-correlated oribital
                
                at_jl = self.bndindJ[i,0] # correlated atom
                l_jl  = self.bndindJ[i,1] # correlated l
                
                for j in range(self.ndim):
                    at_lm = self.bndind[j,0]
                    l_lm  = self.bndind[j,1]
                    
                    if (at_lm == at_jl and l_lm == l_jl): # this must also be correlated
                        corind[j] = 1
        else:
            for i in range(self.ndim):
                corind[i] = self.Sigind[i,i]
                
        return corind
    
    def cmp_Gloc_coeff(self, z, sig_DMFT_base, cmp_weights, lbndind, max_metropolis_steps=0):
        """
        Computes local Green's function in LMTO base
        Tetrahedron integration is used to sum over the Brillouin zone.
        Input:
           z                       -- omega+mu
           sig_DMFT_base[ndim,ndim]-- local self-energy
           cmp_weights             -- fortran subroutine to compute tetrahedron weight
                 can be one of
                     tetra.tetra_weights
                     tetra.tetra_weights_int
           lbndind[lndmim,4] = (iatom,l,m,s)  -- from orbital index gives atom,l,m,s
           max_metropolis_steps=0  -- metropolis steps to sort eigenvalues
        Output:
           ghh[ndim,ndim]          -- matrix element of Gloc - A^r.w.A^l
           ghj[ndim,lndim]         -- matrix element of Gloc - A^r.w.A^l.Sk^+
           gjh[lndim,ndim]         -- matrix element of Gloc - Sk.A^r.w.A^l
           gjj[lndim,lndim]        -- matrix element of Gloc - Sk.A^r.w.A^l.Sk^+
        Used in this subroutine:
           olap[ndim,ndim,nkp]      -- overlap matrix
           hamf[ndim,ndim,nkp]      -- hamiltonian
           itt[5,ntet]             -- tetrahedron index pointer
                 where
                     itt[0,itet]   -- multiplicity of a tetrahedron
                     itt[1:4,itet] -- corner indexes of a tetrahedron
           bndind[lndmim,4] = (iatom,l,m,s)  -- from orbital index gives atom,l,m,s
        """
        (ndim,nkp) = (self.ndim, self.nkp)
        # Computes eigensystem only
        Et = array(zeros((ndim,nkp), dtype=complex), order='F')
        Al = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
        Ar = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
        for ikp in range(nkp):
            # Rotates Sigma to this base
            sig2 = self.Embed(ikp, sig_DMFT_base)
            (Et[:,ikp], Al[:,:,ikp], Ar[:,:,ikp]) = Eigensystemf(array(self.hamf[:,:,ikp]+sig2,order='F'), self.olap[:,:,ikp])

        # Computes tetrahedron weights
        wgh = cmp_weights(z, Et, self.itt, max_metropolis_steps)
        
        # Computes matrix elements for the local green's function using
        # above computed tetrahedron weights
#        (ghh,ghj,gjh,gjj) = crho.cmp_rho_coefficients(self.inpdir, wgh, Al, Ar, self.bndind, lbndind)
        return crho.cmp_rho_coefficients(self.inpdir, wgh, Al, Ar, self.bndind, lbndind)
    
    def symmetrize(self, gxx): #ghh, ghj, gjh, gjj):
        """ symmetrization to simulate summation over the entire Brillouin zone
        The matrix elements of the valence density and Gloc can be symmetrized with this function.
        """
        if (shape(gxx[3]) != shape(gxx[0])):
            sym.symmetrize_spheric(gxx[0])
            sym.symmetrize_hj(gxx[1])
            sym.symmetrize_jh(gxx[2])
            sym.symmetrize_jj(gxx[3])
        else:
            sym.symmetrize_spheric(gxx[0])
            sym.symmetrize_spheric(gxx[1])
            sym.symmetrize_spheric(gxx[2])
            sym.symmetrize_spheric(gxx[3])
        return gxx

    def give_density_coeff(self, gxx): #ghh, ghj, gjh, gjj):
        return (crho.give_density(gxx[0], gxx[0], self.nspin),
                crho.give_density(gxx[1], gxx[2], self.nspin),
                crho.give_density(gxx[2], gxx[1], self.nspin),
                crho.give_density(gxx[3], gxx[3], self.nspin))

    def renorm_density(self, rho_tot, zvalt, SASA, SMT, rweight, dh):
        """  Renormalizes valence+semicore density to the ion charge """
        zvtot = 0
        for iatom in range(self.natom):
            isort = self.isrt[iatom]-1
            for ispin in range(self.nspin):
                drho = real(rho_tot[iatom][:,0,ispin,0])
                work = drho * rweight[isort]
                zvtot += integrate.simps(work)*dh[isort]/sqrt(4*pi)
#                zvtot += stupid_simps(work)*dh[isort]/sqrt(4*pi)
                # if atomic radius is larger than mt-sphere, the additional shell is filled with charge equal to last RHO value
                zvtot += (4*pi/3.)*(SASA[isort]**3-SMT[isort]**3)*drho[-1]/sqrt(4*pi)

        if (zvtot < 1e-10):
            s.stderr("rho_renorm: Charge density within MT sphere is %f. This must be wrong" % zvtot)
            s.exit(1)
        self.fh_info.write("Total charge found to be %f, charge must be %f, -> renormalizing by %f\n" %(zvtot, zvalt, zvalt/zvtot))
        print "Total charge found to be %f, charge must be %f, -> renormalizing by %f" %(zvtot, zvalt, zvalt/zvtot)
        return zvalt/zvtot

    def recompute_density(self, nxx, old_version=True):
        """
        Arrays allocated here:
           rho_val[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- valence charge density
           rho_sem[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- semicore density
           rho_tot[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- valence+semicore density
           rho_cor[nsort][nrad_isort+1,nspin]                        -- core charge
           nrad[nsort]                                               -- size of radial mesh for each sort
           LmtSym[limvmem,natommem]                                  -- index of nonzero expansion coefficients for the charge density
           Rm[nrad_max+1,nsort]                                      -- radial mesh 
           Cf[lmax0,lmax1,lmax1]                                     -- Gaunt coefficients
           Fxx[2,nrad_sort+1,ilst_sort+1,nspi]                       -- radial wave functions Fih and Fij
           gam[ilst_isort+2]                                         -- gam coefficients
           rweight[nsort][nrad_isort+1]                              -- weights for the radial integration
           dh[nsort]                                                 -- radial mesh dh
           SASA[nsort]                                               -- atomic sphere
           SMT[nsort]                                                -- muffin-tin sphere
        """
        fh_func = open(self.inpdir + '/func.dat', 'r')
        # reading information about radial mesh
        nrad = aux.read_nrad(fh_func, self.nsort)
        # reading information about nonzero components of charge density
        LmtSym = aux.read_LmtSym(fh_func, self.isrt, self.ilsv)
        # reading radial mesh for charge density
        Rm = aux.read_RadialMesh(fh_func, nrad, self.nsort)
        # reading gaunt coefficients
        Cf = aux.read_GauntCoeff(fh_func)

        # Computing new valence charge
        rho_val = []
        for iat in range(self.natom):
            
            isort = self.isrt[iat]-1
            
            Fxx = aux.read_RadialFunctions(fh_func, isort, nrad, self.ilst, self.nspin)

            gam = aux.read_Gam(fh_func, self.ilst[isort])

            if old_version:
                trhoval = crho.cmp_rho_valence(max(LmtSym[:,iat]), self.norbs, nxx[0], nxx[1], nxx[2], nxx[3], LmtSym[:,iat], self.ltpan[:,iat], self.lmtind[:,iat], self.llmtind[:,iat], self.ilsv[isort], Cf, gam, Fxx[0], Fxx[1])
            else:
                trhoval = crho.cmp_rho_valence2(iat+1, max(LmtSym[:,iat]), self.norbs, nxx[0], nxx[1], nxx[2], nxx[3], LmtSym[:,iat], self.bndind, self.lbndind, self.ilsv[isort], Cf, gam, Fxx[0], Fxx[1], self.worbs, self.wupdn)
            
            rho_val.append(trhoval)

        # reading new semicore density
        self.rho_sem = aux.read_RhoSemicore(fh_func, nrad, LmtSym, self.isrt, self.ilsv, self.nspin, self.norbs)

        # valence + semicore density
        rho_tot = []
        for iat in range(self.natom):
            rho_tot.append(rho_val[iat] + self.rho_sem[iat])

        # need some more details of the radial mesh for renormalizing charge density
        (self.rweight, self.dh) = aux.read_RadialMeshDetails(fh_func, nrad, self.nsort)

        (zvalt, self.SASA, self.SMT) = aux.read_ChargeAndSphere(fh_func, self.nsort)

        # reading core density
        self.rho_cor = aux.read_CoreDensity(fh_func, nrad, self.nspin)

        # renormalizing charge density
        nrm = self.renorm_density(rho_tot, zvalt, self.SASA, self.SMT, self.rweight, self.dh)

        # renormalizing all
        for iat in range(self.natom):
            rho_tot[iat] *= nrm
            rho_val[iat] *= nrm
            self.rho_sem[iat] *= nrm

        # computing total charge
        for iat in range(self.natom):
            isort = self.isrt[iat]-1
            for ispin in range(self.nspin):
                re_rho = real(rho_tot[iat][:,0,ispin,0]) +  self.rho_cor[isort][:,ispin]
                rho_tot[iat][:,0,ispin,0] = re_rho

        #fh = open('rho_out.dat', 'w')
        #for iatom in range(self.natom):
        #    isort = self.isrt[iatom]-1
        #    for iorbs in range(self.norbs):
        #        for ispin in range(self.nspin):
        #            for isym in range(shape(rho_tot[iatom])[1]):
        #                fh.write('unknown\n')
        #                for irad in range(shape(rho_tot[iatom])[0]):
        #                    r0 = Rm[irad,isort]
        #                    rt = rho_tot[iatom][irad, isym, ispin, iorbs]
        #                    rv = rho_val[iatom][irad, isym, ispin, iorbs]
        #                    rs = self.rho_sem[iatom][irad, isym, ispin, iorbs]
        #                    fh.write(' %20.12f  %30.16f %30.16f %30.16f %30.16f %30.16f %30.16f ' % (r0,rt.real,rt.imag,rv.real,rv.imag,rs.real,rs.imag))
        #                    rc=0
        #                    if (isym==0 and iorbs==0):
        #                        rc = self.rho_cor[isort][irad,ispin]
        #                    fh.write('%30.16f\n' % (rc))

        return rho_tot
    
    def cmp_chem_potential_LDAU(self, mu_old, Dinterval, sig_DMFT_base, test_complex_eigenvalues=False):
        """
        Computes chemical potential if self-energy is constant and real (LDA+U)
        Tetrahedron integration is used to sum over the Brillouin zone.
        Input:
           mu_old                  -- previous mu
           sig_DMFT_base[ndim,ndim]-- local self-energy
           
        Output:
           mu_new                  -- new chemical potential
           
        Used in this subroutine:
           olap[ndim,ndim,nkp]      -- overlap matrix
           hamf[ndim,ndim,nkp]      -- hamiltonian
           itt[5,ntet]             -- tetrahedron index pointer
                 where
                     itt[0,itet]   -- multiplicity of a tetrahedron
                     itt[1:4,itet] -- corner indexes of a tetrahedron
           zvaltot                 -- valence electronic charge
        """
        (ndim,nkp) = (self.ndim, self.nkp)
        # Computes eigenvalues only
        Et = array(zeros((ndim,nkp), dtype=complex), order='F')
        for ikp in range(nkp):
            # Rotates Sigma to this base
            sig2 = self.Embed(ikp, sig_DMFT_base)
            Et[:,ikp] = Eigenvaluesf(array(self.hamf[:,:,ikp]+sig2, order='F'), self.olap[:,:,ikp])

        maxE=[]
        minE=[]
        for p in range(ndim):
            maxE.append(max(Et[p,:]))
            minE.append(min(Et[p,:]))
            
        noccb = int(self.zvaltot*self.nspin/2.+1e-5)

        # first find fully filled bands for which tetrahedron is not needed
        filled=0
        for i in range(noccb):
            if (test_complex_eigenvalues):
                ww = zcmp_valence_charge(maxE[i], tetra.tetra_zweights_int_only, Et, self.itt, 0, noccb)
            else:
                ww = cmp_valence_charge(maxE[i], tetra.tetra_weights_int, Et, self.itt, 0, noccb)
            if ww>1e-6 : break
            filled=i+1

        # recognize insulator with gap            
        if (maxE[noccb-1] < minE[noccb]):  
            mu_new = 0.5*(maxE[noccb-1]+minE[noccb])
        else:   # metal
            # only partially filled bands considered
            a = mu_old-Dinterval
            b = mu_old+Dinterval

            if (test_complex_eigenvalues):
                fa = zcmp_valence_charge(a, tetra.tetra_zweights_int_only, Et[filled:,:], self.itt, 0, noccb-filled)
                fb = zcmp_valence_charge(b, tetra.tetra_zweights_int_only, Et[filled:,:], self.itt, 0, noccb-filled)
                mu_new = optimize.brentq(zcmp_valence_charge, a, b, args=(tetra.tetra_zweights_int_only, Et, self.itt, 0, noccb))
            else:
                fa = cmp_valence_charge(a, tetra.tetra_weights_int, Et[filled:,:], self.itt, 0, noccb-filled)
                fb = cmp_valence_charge(b, tetra.tetra_weights_int, Et[filled:,:], self.itt, 0, noccb-filled)
                mu_new = optimize.brentq(cmp_valence_charge, a, b, args=(tetra.tetra_weights_int, Et, self.itt, 0, noccb))
            
        print 'new chemical potential found at =', mu_new-mu_old, ' chemical potential becomes=', mu_new
        print >> self.fh_info, 'new chemical potential found at =', mu_new-mu_old, ' chemical potential becomes=', mu_new
        
        return mu_new
    

    def cmp_DOS(self, om_mu, Sigma, max_metropolis_steps=0):
        """
        Computes Density of states (calculations done in LMTO base)
        Input:
          om_mu                -- omega+mu
          Sigma                -- local self-energy in DMFT base
          max_metropolis_steps --

        Output:
          dDOS                  -- a matrix (ndim x ndim) of Density of state

        Used in this subroutine:
          cmp_Gloc_coeff        -- function to compute green's function in LMTO base
          symmetrize            -- symmetrization to mimic summation over the entire Brillouin zone
          give_density_coeff    -- computed rho=(G-G+)/(2pi*i) out of G
          cmp_occupancy         -- from matrix elements of G, computes density
        """
        gxx = self.cmp_Gloc_coeff(om_mu+0j, Sigma, tetra.tetra_weights, self.bndind, max_metropolis_steps)
        self.symmetrize(gxx)
        nxx = self.give_density_coeff(gxx)
        dDOS = crho.cmp_occupancy(nxx[0], nxx[1], nxx[2], nxx[3], self.moo, self.bndind, self.isrt, self.lmtind)
        return dDOS

    def Partial_DOS(self, om_mu, Sigma, max_metropolis_steps=0):
        """
        Computes Density of states (calculations done in LMTO base)
        Input:
          om_mu                -- omega+mu
          Sigma                -- local self-energy in DMFT base
          max_metropolis_steps --

        Output:
          TDOS,pdos[natom,lmax,nspin]   -- total and partial DOS

        Used in this subroutine:
          cmp_DOS               -- this actually computes DOS
          
        """
        dDOS = self.cmp_DOS(om_mu, Sigma, max_metropolis_steps)
        
        TDOS=0
        lmax = max(self.ilsb)
        pdos = zeros((self.natom,lmax+1,self.nspin), dtype=float)
        for p in range(len(dDOS)):
            iatom = self.bndind[p,0]-1
            l = self.bndind[p,1]
            s = self.bndind[p,3]-1
            pdos[iatom,l,s] += dDOS[p,p].real
            TDOS += dDOS[p,p].real

        pDOS=[]
        for iatom in range(self.natom):
            isort = self.isrt[iatom]-1
            for l in range(self.ilsb[isort]+1):
                if (not self.ltpan[l,iatom]): continue
                for s in range(self.nspin):
                    pDOS.append(pdos[iatom,l,s])
                
        return (TDOS, pDOS)

    def RhoDiff(self, rho_old, rho_new):
        """Computes and prints the difference between steps
           Input:
                  rho_old[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- total_charge old
                  rho_new[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- total_charge new

           Output:
                  diff[4]                       -- Difference of charge, various magnetizations

           Used in this routine:
                  fh_info                       -- info file 
                  isrt[natom]                   -- sort of each atom
                  rweight[nsort][nrad_isort+1]  -- weights for the radial integration
                  dh[nsort]                     -- weights of radial mesh
        """
        # some dimensions
        nsort = len(self.dh)
        natom = len(rho_old)
        nspin = shape(rho_old[0])[2]
        norbs = shape(rho_old[0])[3]

        nsym = zeros(natom, dtype=int)
        nrd  = zeros(nsort, dtype=int)
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            nsym[iatom] = shape(rho_old[iatom])[1]
            nrd[isort] = shape(rho_old[iatom])[0] 

        # Computes and stores integration weights
        weight=[]  # weight[nsort][nrad_isort+1]
        for isort in range(nsort):
            weight_ = zeros(nrd[isort], dtype=float)
            
            for irad in range(nrd[isort]):
                weight_[irad] = self.rweight[isort][irad]*self.dh[isort]/(4*pi)
                
            weight_[0]  *= 0.5
            weight_[-1] *= 0.5
            
            weight.append(weight_)
            
        # spins and orbits in list
        s = [0, nspin-1]
        o = [0, norbs-1]

        diff = zeros(4, dtype=float)  # difference
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            for isym in range(nsym[iatom]): # over all components of density
                trho = zeros((4,2), dtype=complex)
                tsum = zeros(4, dtype=float)
                for irad in range(nrd[isort]):

                    trho[0,0] = (rho_old[iatom][irad,isym,s[0],o[0]]+rho_old[iatom][irad,isym,s[1],o[0]])*0.5*nspin
                    trho[1,0] = (rho_old[iatom][irad,isym,s[0],o[0]]-rho_old[iatom][irad,isym,s[1],o[0]])
                    trho[2,0] =  rho_old[iatom][irad,isym,s[0],o[1]]
                    trho[3,0] =  rho_old[iatom][irad,isym,s[1],o[1]]

                    trho[0,1] = (rho_new[iatom][irad,isym,s[0],o[0]]+rho_new[iatom][irad,isym,s[1],o[0]])*0.5*nspin
                    trho[1,1] = (rho_new[iatom][irad,isym,s[0],o[0]]-rho_new[iatom][irad,isym,s[1],o[0]])
                    trho[2,1] =  rho_new[iatom][irad,isym,s[0],o[1]]
                    trho[3,1] =  rho_new[iatom][irad,isym,s[1],o[1]]

                    for i in range(4): tsum[i] += abs(trho[i,1]-trho[i,0])**2*weight[isort][irad]

#                    print iatom, isym, irad, abs(trho[0,1]-trho[0,0])**2*weight[isort][irad], abs(trho[3,1]-trho[3,0])**2*weight[isort][irad], tsum[3]
                    
                for i in range(4): diff[i] += sqrt(tsum[i])
#                print iatom, isym, sqrt(tsum[0]), sqrt(tsum[1]), sqrt(tsum[2]), sqrt(tsum[3])

        # Writing difference
        print 'Charge density self-consistency=', diff[0]
        if nspin>1 :
           print 'Zmagnetization self-consistency=', diff[1]
           print 'Mmagnetization self-consistency=', diff[2]
           print 'Pmagnetization self-consistency=', diff[3]

        print >> self.fh_info, 'Charge density self-consistency=', diff[0]
        if nspin>1 :
           print >> self.fh_info, 'Zmagnetization self-consistency=', diff[1]
           print >> self.fh_info, 'Mmagnetization self-consistency=', diff[2]
           print >> self.fh_info, 'Pmagnetization self-consistency=', diff[3]

        return diff

    def BroydenMix(self, rho_old, rho_new, alpha_large=0.6, alpha0=0.3, alpha_rho=1., alpha_mag=1., restart=False):
        """ Mixes total density according to one of the Broyden-type mixing: Vanderbilt or Johnson

            Input:
               rho_old[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- total_charge old
               rho_new[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- total_charge new
               alpha_large                     -- mixing of most of the density l>0
               alpha_rho                       -- broyden mixing for density
               alpha_mag                       -- broyden mixing for magnetization
               alpha0                          -- starting mixing in broyden
               restart                         -- if broyden mixing should be restarted

            Output:
               rho_out[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]   -- mixed total_charge

            Used in this routine:
               fh_info                         -- info file
               inpdir                          -- input directory
               isrt[natom]                     -- sort of each atom -- equivalent to is in fortran
               rweight[nsort][nrad_isort+1]    -- weights for the radial integration
               dh[nsort]                       -- radial mesh dh
               SASA[nsort]                     -- atomic sphere radius
               SMT[nsort]                      -- MT-sphere radius

        """
        # some dimensions
        nsort = len(self.dh)
        natom = len(rho_old)
        nspin = shape(rho_old[0])[2]
        norbs = shape(rho_old[0])[3]

        # Filenames of files storing old charge density
        filename_rho = []
        filename_mag = []
        for iatom in range(natom):
            filename_rho.append(self.inpdir + '/jsmixr.' + str(iatom) + '.tmp')
            filename_mag.append(self.inpdir + '/jsmixm.' + str(iatom) + '.tmp')

        nsym = zeros(natom, dtype=int)
        nrd  = zeros(nsort, dtype=int)
        for iatom in range(natom):
            isort = self.isrt[iatom]-1
            nsym[iatom] = shape(rho_old[iatom])[1]
            nrd[isort] = shape(rho_old[iatom])[0] 

        # Computes and stores integration weights
        weight  = []  # weight[nsort][nrad_isort+1]
        sweight = []  # sqrt(weight)
        for isort in range(nsort):
            weight_ = zeros(nrd[isort], dtype=float)
            sweight_ = zeros(nrd[isort], dtype=float) 
            for irad in range(nrd[isort]):
                weight_[irad] = self.rweight[isort][irad]*self.dh[isort]/(4*pi)
                sweight_[irad] = sqrt(weight_[irad])
                
            weight_[0]  *= 0.5
            weight_[-1] *= 0.5
            sweight_[0]  = sqrt(weight_[0])
            sweight_[-1] = sqrt(weight_[-1])
            
            weight.append(weight_)
            sweight.append(sweight_)
            

        # Simple mixing valid for l>0 (isyim>0)
        # We will mix all components
        rho_out = []
        for iatom in range(natom):
            rho_out.append(array(rho_old[iatom]*(1-alpha_large) + rho_new[iatom]*alpha_large, order='F'))

        
        s = [0, nspin-1]
        o = [0, norbs-1]

        totCharge=0
        for iatom in range(len(rho_old)):

            isym = 0 # Broyden mixing only for the lowest rho of l=0
            isort = self.isrt[iatom]-1
            zrho = zeros(nrd[isort], dtype=complex)
            Grho = zeros(nrd[isort], dtype=complex)
            zmag = zeros(nrd[isort], dtype=complex)
            Gmag = zeros(nrd[isort], dtype=complex)
            xrho = zeros(nrd[isort], dtype=float)
            Frho = zeros(nrd[isort], dtype=float)
            xmag = zeros(nrd[isort], dtype=float)
            Fmag = zeros(nrd[isort], dtype=float)

            # Preparing density for mixing
            for irad in range(nrd[isort]):
                rh_old = (rho_old[iatom][irad,isym,s[0],o[0]]+rho_old[iatom][irad,isym,s[1],o[0]])*0.5*nspin
                mg_old = (rho_old[iatom][irad,isym,s[0],o[0]]-rho_old[iatom][irad,isym,s[1],o[0]])
                rh_new = (rho_new[iatom][irad,isym,s[0],o[0]]+rho_new[iatom][irad,isym,s[1],o[0]])*0.5*nspin
                mg_new = (rho_new[iatom][irad,isym,s[0],o[0]]-rho_new[iatom][irad,isym,s[1],o[0]])

                zrho[irad] = rh_old
                Grho[irad] = rh_new - rh_old
                zmag[irad] = mg_old
                Gmag[irad] = mg_new - mg_old
                xrho[irad] = zrho[irad].real*sweight[isort][irad]
                Frho[irad] = Grho[irad].real*sweight[isort][irad]
                xmag[irad] = zmag[irad].real*sweight[isort][irad]
                Fmag[irad] = Gmag[irad].real*sweight[isort][irad]

            yrho = mix.johnson_mix(filename_rho[iatom], Frho, xrho, restart, alpha0, alpha_rho) # density
            ymag = mix.johnson_mix(filename_mag[iatom], Fmag, xmag, restart, alpha0, alpha_mag) # magnetization

            for irad in range(1,nrd[isort]):
                Frho[irad] = yrho[irad]*self.rweight[isort][irad]/sweight[isort][irad]
            Frho[0] = 0
            
            tCharge = integrate.simps(Frho)*self.dh[isort]/sqrt(4*pi)
            tCharge += (4*pi/3.)*(self.SASA[isort]**3-self.SMT[isort]**3)*yrho[-1]/sweight[isort][-1]/sqrt(4*pi)
            totCharge += tCharge

            print 'BroydenMix: Partial charge after Broyden for iatom', iatom, 'is', tCharge
            print >> self.fh_info, 'BroydenMix: Partial charge after Broyden for iatom', iatom, 'is', tCharge
            
            if min(yrho) >= 0: # Accepting broyden mixing since it leads to positive charge
                for irad in range(1,nrd[isort]):
                    zrho[irad] = yrho[irad]/sweight[isort][irad]
                    zmag[irad] = ymag[irad]/sweight[isort][irad]
                rho0_old = (rho_old[iatom][0,isym,s[0],o[0]]+rho_old[iatom][0,isym,s[1],o[0]])*0.5*nspin
                rho0_new = (rho_new[iatom][0,isym,s[0],o[0]]+rho_new[iatom][0,isym,s[1],o[0]])*0.5*nspin
                mag0_old = (rho_old[iatom][0,isym,s[0],o[0]]-rho_old[iatom][0,isym,s[1],o[0]])
                mag0_new = (rho_new[iatom][0,isym,s[0],o[0]]-rho_new[iatom][0,isym,s[1],o[0]])
                zrho[0]= rho0_old*(1-alpha_large) + rho0_new*alpha_large
                zmag[0]= mag0_old*(1-alpha_large) + mag0_new*alpha_large

                rho_out[iatom][:,isym,s[0],o[0]] = (zrho[:]+zmag[:])/nspin
                rho_out[iatom][:,isym,s[1],o[0]] = (zrho[:]-zmag[:])/nspin
            else:             # Leave linear mixing and restart broyden
                print 'BroydenMix: CHARGE DENSITY IS NOT POSITIVE\nBroydenMix: RESTARTING BROYDEN'
                print >> self.fh_info, 'BroydenMix: CHARGE DENSITY IS NOT POSITIVE\nBroydenMix: RESTARTING BROYDEN'
                # Removing files...
                try:
                    os.remove(filename_rho[iatom])
                except OSError:
                    print 'Could not remove', filename_rho[iatom]
                    
                try:
                    os.remove(filename_mag[iatom])
                except OSError:
                    print 'Could not remove', filename_rho[iatom]
                    
        print 'BroydenMix: Total charge after Broyden is', totCharge
        print >> self.fh_info, 'BroydenMix: Total charge after Broyden is', totCharge
        
        return rho_out

    def N_correlated(self, mocc):
        """Computed nf (correlated occupancy) given the total density matrix compuated by crho.cmp_occupancy

           Input:
                 mocc        --  density matrix
                 
           Output:
                 nf[natom]   -- array of correlated occupancy ( correlated bands are determined from cix file)
                 
           Used in this subroutine:
                 ztransform                       -- fortran routine to transfrom between cubic and spheric/relativistic harmonics
                 T2C[ndim,ndim]                   -- transformation between cubic and spheric/relativistic harmonics
                 Sigind[ndim,ndim]                -- index of correlated bands as determined from the cix file
                 bndindJ[ndim,4] = (iatom,l,j,mj) -- index in j-base gives atom,l,j,mj
        """
        # The code below prints nf
        mocc_copy = array(cpy.deepcopy(mocc),order='F')
        la.ztransform(mocc_copy, self.T2C,'N') # From spheric -> cubic
        # correlated index and therefore Nf is defined in cubis/relativistic
        nf = zeros(self.natom, dtype=float)
        for i in range(self.ndim):
            if (self.Sigind[i,i]!=0):
                if self.nspin > 1 and self.norbs > 1: iatom = self.bndindJ[i,0]-1
                else: iatom = self.bndind[i,0]-1
                nf[iatom] += mocc_copy[i,i]
        # Printing
        for iatom in range(len(nf)):
            if (nf[iatom]!=0.0):
                print 'nf[', iatom, ']= ', nf[iatom]
                print >> self.fh_info, 'nf[', iatom, ']= ', nf[iatom]
        return nf

    def DensityMatrixCoeff(self, mu, Sigma):
        """Computes the Density matrix coefficients nxx = [nHH, nHJ, nJH, nJJ].
        Uses integral-integration weights which are useful for constant self-energy
        This routine can be used for LDA or LDA+U type of calculation

        Input:  mu    -- the chemical potential
                Sigma -- self-energy in DMFT base (cubic/relativistic harmonics)
               
        Output: nxx   -- density matrix coefficients nxx=[nHH, nHJ. nJH, nJJ]

        Used in this routine:
                cmp_Gloc_coeff     -- member function to compute gxx coefficients of the Green's function
                symmetrize         -- member function to simulate simulation ower entire Brillouin zone
                give_density_coeff -- member function computes density coefficients nxx from green's function coefficients gxx
        """
        self.add_sqrt2 = True
        
        gxx = self.cmp_Gloc_coeff(mu+0j, Sigma, tetra.tetra_weights_int, self.lbndind)

        self.symmetrize(gxx)
    
        nxx = self.give_density_coeff(gxx)

        if (self.norbs>1 and self.add_sqrt2):
            for i in range(len(nxx)):
                n1 = shape(nxx[i])[0]/2
                n2 = shape(nxx[i])[1]/2
                nxx[i][0:n1, n2:2*n2] *= sqrt(2.)
                nxx[i][n1:2*n1, 0:n2] *= -sqrt(2.)
                
        return nxx

    def ZeroNonCorrelated(self, gloc):
        """Sets to zero the off-diagonal green's function between the heavy and the light block
        """
        for si in self.vSigind:
            # complement of si
            complem = range(self.ndim)
            for i in si: complem.remove(i)
            # set to zero field which correspond to
            # p in si and q not in si
            for p in si:
                for q in complem:
                    gloc[p,q]=0
                    gloc[q,p]=0
        

    def cmp_tgf2(self, gxx):
        """Using the LMTO coefficients of the green's functions (LMTO representation)
        computes Green's function in the DMFT base. This works only for the current
        implementation of the DMFT base. If the DMFT base is changed, this routine needs
        to be updated.
        The transformation from LMTO to DMFT base is currently:
        Utk = dioloc*(h-j*Sk)

        Input:
             gxx       -- the coefficients of Gf in LMTO base (ghh,ghj,gjh,gjj)
        Output:
             gf        -- green's function in DMFT base
        """
        gf = crho.cmp_tgf(gxx[0], gxx[1], gxx[2], gxx[3], self.hh, self.jj, self.dioloc, self.norb)
        
        #gf = array(zeros((self.ndim,self.ndim), dtype=complex), order='F')
        #for ph in range(self.ndim):
        #    pj = ph + (self.lndim/2-self.ndim/2)*int(ph/self.norb)
        #    pj = ph
        #    for qh in range(self.ndim):
        #        qj = qh + (self.lndim/2-self.ndim/2)*int(qh/self.norb)
        #        qj = qh
        #        gf0 = self.hh[ph]*gxx[0][ph,qh]*conj(self.hh[qh])-self.hh[ph]*gxx[1][ph,qj]*conj(self.jj[qh])-self.jj[ph]*gxx[2][pj,qh]*conj(self.hh[qh])+self.jj[ph]*gxx[3][pj,qj]*conj(self.jj[qh])
        #        gf[ph,qh] = self.dioloc[ph]*gf0*self.dioloc[qh]
        #
        
        la.ztransform(gf, self.T2C,'N') # From spheric -> cubic/relativistic
        return gf
    


def cmp_valence_charge(efmu, cmp_weights, Et, itt, max_metro_steps, zvalence):
    """Small function to be used for computing chemical potential in LDA+U.
    This function is called by root-finding routine
    """
    # Computes tetrahedron weights
    wgh = cmp_weights(efmu, Et, itt, max_metro_steps)
    # Compute charge in valence band at current chemical potential
    return sum(wgh).imag*(-1./pi)-zvalence

def zcmp_valence_charge(efmu, cmp_weights, Et, itt, max_metro_steps, zvalence):
    """Small function to be used for computing chemical potential in LDA+U.
    This function is called by root-finding routine
    """
    # Computes tetrahedron weights
    wgh = cmp_weights(-20+1e-10j, efmu+1e-10j, Et, itt, max_metro_steps)
    # Compute charge in valence band at current chemical potential
    return sum(wgh).imag*(-1./pi)-zvalence

