import tutils as utl
import auxil as aux
from scipy import *
import sys as s

class LmtScsf(object):
    " Class [reads from|writes to] Lmtart scs file "
    
    def __init__(self):
        "Initializes few things "
        pass

    def read(self, fname):
        """ Read Lmtart scs file
          Output:
              rho[natom][nrad_isort+1,nsym_iatom, nspin, norbs]    -- total charge density
              rho_cor0[nsort][nrad_isort+1,nspin]                  -- core charge
              pot_frz0[nsort][nrad_isort+1,nspin]                  -- KS-potential
              efmu                                                 -- chemical potential
              Enu                                                  -- linearization energies
          Saves the following data as class members:
              text              -- first line in the scs file
              sdata_0           -- contains (natom,nsort,nspin,norbs)
              sisrt0            -- isrt(natom)
              sdata_1           -- information about basis vectors,  nuclear charge, smt spheres,..
              sdata_2           -- radial mesh information such as nrad, nsym, ilsv
              R0                -- radial mesh for all sorts
              sdata_3           -- information about basis vectors again
              sdata_4           -- nkap, ilst, ilsb
              sdata_5           -- Enu's, Dnu's
              scsf_made         -- if the iteration reached self-consistency
              
          Disentangles the following arrays and data:
              from sdata_0:
                 natom0, nsort0, nspin0, norbs0
              from sisrt0
                 isrt0[natom0]
              from sdata_1:
                 rbas0[3,3]
                 gbas0[3,3]
                 parr0[1]
                 orth0[2]
                 strain0[3,3,2]
                 tao0[3,natom0]
                 tau0[3,natom0]
                 znuc0[nsort0]
                 smt0[nsort0]
                 axmag0[3,nsort0]
              from sdata_2:
                 nrad0[nsort]
                 nsym0[natom]
                 ilsv0[nsort]
              from R0:
                 Rm[nsort][nrad_isort+1]
              from:
                 rho0[natom][nrad_isort+1,nsym_iatom, nspin, norbs]
              from: 
                 rho_cor0[nsort][nrad_isort,nspin]
                 pot_frz0[nsort][nrad_isort,nspin]
              from sdata_3:
                 rbas1[3,3]
                 gbas1[3,3]
                 orth1[2]
                 parr1[1]
                 strain1[3,3,2]
                 nplw0[1]
                 nfftg0[3]
                 tao1[3,natom0]
                 tau1[3,natom0]
              from sdata_4:
                 nkap[1]
                 ilst[nsort0]
                 ilsv[nsort0]
              from sdata_5:
                 eny0[nspin][nsort][ilst_isort+1]
                 dny0[nspin][nsort][ilst_isort+1]
                 mny0[nsort][ilst_isort+1]
                 nod0[nsort][ilst_isort+1]
              from last_line
                 efmu
                 scsf_made
          """
        
        fh_scs = open(fname, 'r')

        self.text = fh_scs.next()
        if (self.text[1]!="<") :
            print >> s.stderr, "The file ", filename, "seems not to be Lmtart scs file! Exiting.....\n"

        # First part of data
        self.sdata_0 = utl.read_data(fh_scs, 4)
        (natom0, nsort0, nspin0, norbs0) =  map(int, self.sdata_0)
        (self.natom0, self.nsort0, self.nspin0, self.norbs0) = (natom0, nsort0, nspin0, norbs0)
         
        # Second part of data
        self.sisrt0 = utl.read_data(fh_scs, natom0)
        self.isrt0 = map(int, self.sisrt0)

        # Third part of data
        dimension = [(3,3),(3,3),(1,),(2,),(3,3,2),(3,natom0),(3,natom0),(nsort0,),(nsort0,),(3,nsort0)]
        self.sdata_1 = utl.read_data(fh_scs, utl.total_dimension(dimension))
        (rbas0, gbas0, parr0, orth0, strain0, tao0, tau0, znuc0, smt0, axmag0) =  utl.data_reshape(map(float, self.sdata_1), dimension)

        # Fourth part of data
        dimension = [(nsort0,),(natom0,),(nsort0,)]
        self.sdata_2 = utl.read_data(fh_scs, utl.total_dimension(dimension))
        (nrad0, nsym0, ilsv0) = utl.data_reshape(map(int, self.sdata_2), dimension)
        self.nsym0 = nsym0
        
        # reading Radial mesh 
        tot_rad = sum(nrad0)+nsort0
        # Reading all radial meshes into single array
        self.R0 = utl.read_data(fh_scs, tot_rad)
        R0 = map(float, self.R0)
        # And inserting data into proper place
        Rm = []
        ofset=0
        for isort in range(nsort0):
            N = nrad0[isort]+1
            Rm.append(R0[ofset:ofset+N])
            ofset += nrad0[isort]+1

        # reading total charge density
        # rho[natom][nrad_isort+1,nsym_iatom, nspin, norbs]
        self.lmval = zeros((max(nsym0),nsort0) ,dtype=int, order='F')
        rho0 = []
        for iatom in range(natom0):
            isort=self.isrt0[iatom]-1
            rhos = zeros((nrad0[isort]+1, nsym0[iatom], nspin0, norbs0), dtype=complex, order='F')
            for iorbs in range(norbs0):
                for iupdn in range(nspin0):
                    for isym in range(nsym0[iatom]):
                        self.lmval[isym,isort] = utl.read_type(fh_scs, int, 1)[0]
                        rhos[:, isym, iupdn, iorbs] = utl.read_complex(fh_scs, nrad0[isort]+1)
            rho0.append(rhos)
            
        # rho_cor0[nsort][nrad_isort+1,nspin]
        # pot_frz0[nsort][nrad_isort+1,nspin]
        rho_cor0=[]
        pot_frz0=[]
        for isort in range(nsort0):
            N = nrad0[isort]+1
            trho = zeros((N,nspin0), dtype=float, order='F')
            tpot = zeros((N,nspin0), dtype=float, order='F')
            for ispin in range(nspin0):
                trho[:,ispin] = utl.read_type(fh_scs, float, N)
                tpot[:,ispin] = utl.read_type(fh_scs, float, N)
                
            rho_cor0.append(trho)
            pot_frz0.append(tpot)


        # Fifth part of data
        dimension = [(3,3), (3,3), (2,), (1,), (3,3,2), (1,), (3,), (3,natom0), (3,natom0)]
        self.sdata_3 = utl.read_data(fh_scs, utl.total_dimension(dimension))
        (rbas1, gbas1, orth1, parr1, strain1, nplw0, nfftg0, tao1, tau1) = utl.data_reshape(map(float,self.sdata_3), dimension)

        # Here is  a gap in scs file
        dimension = [(1,), (nsort0,), (nsort0,)]
        self.sdata_4 = utl.read_data(fh_scs, utl.total_dimension(dimension))
        (nkap0, ilst0, ilsb0) = utl.data_reshape(map(int,self.sdata_4), dimension)

        # Last part of data
        isize = (2*nspin0+2)*(sum(ilst0)+nsort0)
        self.sdata_5 = utl.read_data(fh_scs, isize)
        data_5 = map(float, self.sdata_5)

        ofset = 0
        eny0=[]
        for ispin in range(nspin0):
            teny = []
            for isort in range(nsort0):
                N = ilst0[isort]+1
                teny.append(data_5[ofset:ofset+N])
                ofset += N
            eny0.append(teny)

        Enu = eny0 # array(eny0)
        
        dny0=[]
        for ispin in range(nspin0):
            tdny = []
            for isort in range(nsort0):
                N = ilst0[isort]+1
                tdny.append(data_5[ofset:ofset+N])
                ofset += N
            dny0.append(tdny)

        mny0 = []
        for isort in range(nsort0):
            N = ilst0[isort]+1
            mny0.append(map(int,data_5[ofset:ofset+N]))
            ofset += N

        nod0 = []
        for isort in range(nsort0):
            N = ilst0[isort]+1
            nod0.append(map(int,data_5[ofset:ofset+N]))
            ofset += N


        # Last line from scs file
        line = fh_scs.next().split()
        efmu = float(line[0])
        self.scsf_made = line[1]

        return (rho0, rho_cor0, pot_frz0, efmu, Enu)

    def write(self, filename, rho, rho_cor, pot_frz, efmu, Enu):
        """
          Writes scs file to disc

          Input:
              rho[natom][nrad_isort+1,nsym_iatom, nspin, norbs]    -- total charge density
              rho_cor[nsort][nrad_isort,nspin]                     -- core charge
              pot_frz[nsort][nrad_isort,nspin]                     -- KS-potential
              efmu  !!!! in eV !!!!!                               -- chemical potential in eV
              Enu                                                  -- linearization energy
          class members used:
              text              -- first line in the scs file
              sdata_0           -- contains (natom,nsort,nspin,norbs)
              sisrt0            -- isrt(natom)
              sdata_1           -- information about basis vectors,  nuclear charge, smt spheres,..
              sdata_2           -- radial mesh information such as nrad, nsym, ilsv
              R0                -- radial mesh for all sorts
              sdata_3           -- information about basis vectors again
              sdata_4           -- nkap, ilst, ilsb
              sdata_5           -- Enu's, Dnu's
              scsf_made         -- if the iteration reached self-consistency
              
          """
        fh_scs = open(filename, 'w')
        fh_scs.write(self.text)
        fh_scs.write(reduce(lambda a,b: a+" "+b, self.sdata_0) + "\n")
        fh_scs.write(reduce(lambda a,b: a+" "+b, self.sisrt0) + "\n")
        fh_scs.write(reduce(lambda a,b: a+"\n"+b, self.sdata_1) + "\n")
        fh_scs.write(reduce(lambda a,b: a+" "+b, self.sdata_2) + "\n")
        fh_scs.write(reduce(lambda a,b: a+"\n"+b, self.R0) + "\n")

        for iatom in range(self.natom0):
            isort=self.isrt0[iatom]-1
            for iorbs in range(self.norbs0):
                for iupdn in range(self.nspin0):
                    for isym in range(self.nsym0[iatom]):
                        fh_scs.write("%d\n" % self.lmval[isym,isort])
                        for irad in range(shape(rho[iatom])[0]):
                            rh = rho[iatom][irad, isym, iupdn, iorbs]
                            fh_scs.write("(%.20E,%.20E)\n" % (rh.real, rh.imag))
                            

        for isort in range(self.nsort0):
            for ispin in range(self.nspin0):
                for irad in range(shape(rho_cor[isort])[0]):
                    fh_scs.write("%.20E \n" % rho_cor[isort][irad, ispin])
                for irad in range(shape(pot_frz[isort])[0]):
                    fh_scs.write("%.20E \n" % pot_frz[isort][irad, ispin])
            
        fh_scs.write(reduce(lambda a,b: a+"\n"+b, self.sdata_3) + "\n")

        # The gap
        print >> fh_scs
        for iorbs in range(self.norbs0):
            for ispin in range (self.nspin0):
                print >> fh_scs
                print >> fh_scs
        
        
        fh_scs.write(reduce(lambda a,b: a+" "+b, self.sdata_4) + "\n")

        Enuf = utl.flatten(Enu)
        for i in range(len(Enuf)):  self.sdata_5[i] = str(Enuf[i])
        
        fh_scs.write(reduce(lambda a,b: a+"\n"+b, self.sdata_5) + "\n")
        Ry2eV = 13.6058
        fh_scs.write("%.20E  %s \n" % (efmu/Ry2eV, self.scsf_made))
        
if __name__ == '__main__':
    
#    inpdir = "/home/haule/realMaterial/source/fully-self-consistent/new_generation/ksum_1/test"
#    inpdir = "/home/haule/tmp"
#    fname = "Cm.scs"

#    inpdir = "/home/haule/realMaterial/database/Ce2O3/"
#    fname = "Ce2O3.scs"
    fname = "/home/haule/realMaterial/database/Cm/Cm.scs"
    scsf = LmtScsf()

    (rho, rho_cor, pot_frz, efmu) = scsf.read(fname)
    scsf.write('try.scs', rho, rho_cor, pot_frz, efmu)
